@@ -2,39 +2,48 @@
  *
  * FFT.c
  *
- * Based on FFT.cpp from Audacity, which contained the following text:
+ * Based on FFT.cpp from Audacity, with the following permission from
+ * its author, Dominic Mazzoni (in particular, relicensing the code
+ * for use in SoX):
  *
- *     This file contains a few FFT routines, including a real-FFT
- *     routine that is almost twice as fast as a normal complex FFT,
- *     and a power spectrum routine when you know you don't care
- *     about phase information.
+ *  I hereby license you under the LGPL all of the code in FFT.cpp
+ *  from any version of Audacity, with the exception of the windowing
+ *  function code, as I wrote the rest of the line [sic] that appears
+ *  in any version of Audacity (or they are derived from Don Cross or
+ *  NR, which is okay).
  *
- *     Some of this code was based on a free implementation of an FFT
- *     by Don Cross, available on the web at:
+ *  -- Dominic Mazzoni <dominic@audacityteam.org>, 18th November 2006
  *
- *        http://www.intersrv.com/~dcross/fft.html
+ * As regards the windowing function, WindowFunc, Dominic granted a
+ * license to it too, writing on the same day:
  *
- *     The basic algorithm for his code was based on Numerican Recipes
- *     in Fortran.  I optimized his code further by reducing array
- *     accesses, caching the bit reversal table, and eliminating
- *     float-to-double conversions, and I added the routines to
- *     calculate a real FFT and a real power spectrum.
+ *  OK, we're good. That's the original version that I wrote, before
+ *  others contributed.
+ *
+ * Some of this code was based on a free implementation of an FFT
+ * by Don Cross, available on the web at:
+ *
+ * http://www.intersrv.com/~dcross/fft.html [no longer, it seems]
+ *
+ * The basic algorithm for his code was based on Numerical Recipes
+ * in Fortran.
  *
  * This file is now part of SoX, and is copyright Ian Turner and others.
  * 
  * SoX is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
  * Foobar is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with SoX; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, write to the Free Software
+ * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
+ * USA.
  */
 
 #include <stdlib.h>
@@ -42,18 +51,14 @@
 #include <math.h>
 #include <assert.h>
 
+#include "st_i.h"
+
 #include "FFT.h"
 
 int **gFFTBitTable = NULL;
 const int MaxFastBits = 16;
 
-/* Declare Static functions */
-static int IsPowerOfTwo(int x);
-static int NumberOfBitsNeeded(int PowerOfTwo);
-static int ReverseBits(int index, int NumBits);
-static void InitFFT();
-
-int IsPowerOfTwo(int x)
+static int IsPowerOfTwo(int x)
 {
    if (x < 2)
       return 0;
@@ -61,13 +66,13 @@ int IsPowerOfTwo(int x)
    return !(x & (x-1));         /* Thanks to 'byang' for this cute trick! */
 }
 
-int NumberOfBitsNeeded(int PowerOfTwo)
+static int NumberOfBitsNeeded(int PowerOfTwo)
 {
    int i;
 
    if (PowerOfTwo < 2) {
-      fprintf(stderr, "Error: FFT called with size %d\n", PowerOfTwo);
-      exit(1);
+      st_debug("Error: FFT called with size %d", PowerOfTwo);
+      exit(2);
    }
 
    for (i = 0;; i++)
@@ -75,7 +80,7 @@ int NumberOfBitsNeeded(int PowerOfTwo)
          return i;
 }
 
-int ReverseBits(int index, int NumBits)
+static int ReverseBits(int index, int NumBits)
 {
    int i, rev;
 
@@ -88,25 +93,19 @@ int ReverseBits(int index, int NumBits)
 }
 
 
-/* This function allocates about 250k (actually (2**16)-2 ints) which is never
- * freed, to use as a lookup table for bit-reversal. The good news is that
- * we bascially need this until the very end, so the fact that it's not freed
- * is OK. */
-void InitFFT()
+/* This function permanently allocates about 250Kb (actually (2**16)-2 ints). */
+static void InitFFT(void)
 {
    int len, b;
-
-   gFFTBitTable = (int**)calloc(MaxFastBits, sizeof(*gFFTBitTable));
    
-   len = 2;
-   for (b = 1; b <= MaxFastBits; b++) {
+   gFFTBitTable = (int**)xcalloc(MaxFastBits, sizeof(*gFFTBitTable));
+   
+   for (b = 1, len = 2; b <= MaxFastBits; b++) {
       int i;
 
-      gFFTBitTable[b - 1] = (int*)calloc(len, sizeof(**gFFTBitTable));
-
-      for (i = 0; i < len; i++) {
+      gFFTBitTable[b - 1] = (int*)xcalloc(len, sizeof(**gFFTBitTable));
+      for (i = 0; i < len; i++)
         gFFTBitTable[b - 1][i] = ReverseBits(i, b);
-      }
 
       len <<= 1;
    }
@@ -118,10 +117,9 @@ void InitFFT()
 /*
  * Complex Fast Fourier Transform
  */
-
 void FFT(int NumSamples,
          int InverseTransform,
-         float *RealIn, float *ImagIn, float *RealOut, float *ImagOut)
+         const float *RealIn, float *ImagIn, float *RealOut, float *ImagOut)
 {
    int NumBits;                 /* Number of bits needed to store indices */
    int i, j, k, n;
@@ -131,8 +129,8 @@ void FFT(int NumSamples,
    float tr, ti;                /* temp real, temp imaginary */
 
    if (!IsPowerOfTwo(NumSamples)) {
-      fprintf(stderr, "%d is not a power of two\n", NumSamples);
-      exit(1);
+      st_debug("%d is not a power of two", NumSamples);
+      exit(2);
    }
 
    if (!gFFTBitTable)
@@ -229,25 +227,21 @@ void FFT(int NumSamples,
  * i4  <->  imag[n/2-i]
  */
 
-void RealFFT(int NumSamples, float *RealIn, float *RealOut, float *ImagOut)
+void RealFFT(int NumSamples, const float *RealIn, float *RealOut, float *ImagOut)
 {
    int Half = NumSamples / 2;
-   int i;
-
+   int i, i3;
    float theta = M_PI / Half;
    float wtemp = (float) sin(0.5 * theta);
    float wpr = -2.0 * wtemp * wtemp;
    float wpi = (float) sin(theta);
    float wr = 1.0 + wpr;
    float wi = wpi;
-
-   int i3;
-
    float h1r, h1i, h2r, h2i;
+   float *tmpReal, *tmpImag;
 
-
-   float *tmpReal = (float*)calloc(Half, sizeof(float));
-   float *tmpImag = (float*)calloc(Half, sizeof(float));
+   tmpReal = (float*)xcalloc(NumSamples, sizeof(float));
+   tmpImag = tmpReal + Half;
 
    for (i = 0; i < Half; i++) {
       tmpReal[i] = RealIn[2 * i];
@@ -256,30 +250,29 @@ void RealFFT(int NumSamples, float *RealIn, float *RealOut, float *ImagOut)
 
    FFT(Half, 0, tmpReal, tmpImag, RealOut, ImagOut);
 
-
    for (i = 1; i < Half / 2; i++) {
-
-      i3 = Half - i;
-
-      h1r = 0.5 * (RealOut[i] + RealOut[i3]);
-      h1i = 0.5 * (ImagOut[i] - ImagOut[i3]);
-      h2r = 0.5 * (ImagOut[i] + ImagOut[i3]);
-      h2i = -0.5 * (RealOut[i] - RealOut[i3]);
-
-      RealOut[i] = h1r + wr * h2r - wi * h2i;
-      ImagOut[i] = h1i + wr * h2i + wi * h2r;
-      RealOut[i3] = h1r - wr * h2r + wi * h2i;
-      ImagOut[i3] = -h1i + wr * h2i + wi * h2r;
-
-      wr = (wtemp = wr) * wpr - wi * wpi + wr;
-      wi = wi * wpr + wtemp * wpi + wi;
+     i3 = Half - i;
+       
+     h1r = 0.5 * (RealOut[i] + RealOut[i3]);
+     h1i = 0.5 * (ImagOut[i] - ImagOut[i3]);
+     h2r = 0.5 * (ImagOut[i] + ImagOut[i3]);
+     h2i = -0.5 * (RealOut[i] - RealOut[i3]);
+       
+     RealOut[i] = h1r + wr * h2r - wi * h2i;
+     ImagOut[i] = h1i + wr * h2i + wi * h2r;
+     RealOut[i3] = h1r - wr * h2r + wi * h2i;
+     ImagOut[i3] = -h1i + wr * h2i + wi * h2r;
+       
+     wtemp = wr;
+     wr = wr * wpr - wi * wpi + wr;
+     wi = wi * wpr + wtemp * wpi + wi;
    }
-
-   RealOut[0] = (h1r = RealOut[0]) + ImagOut[0];
+     
+   h1r = RealOut[0];
+   RealOut[0] += ImagOut[0];
    ImagOut[0] = h1r - ImagOut[0];
 
    free(tmpReal);
-   free(tmpImag);
 }
 
 /*
@@ -294,7 +287,7 @@ void RealFFT(int NumSamples, float *RealIn, float *RealOut, float *ImagOut)
  * of its code.
  */
 
-void PowerSpectrum(int NumSamples, float *In, float *Out)
+void PowerSpectrum(int NumSamples, const float *In, float *Out)
 {
   int Half, i, i3;
   float theta, wtemp, wpr, wpi, wr, wi;
@@ -308,60 +301,57 @@ void PowerSpectrum(int NumSamples, float *In, float *Out)
 
   theta = M_PI / Half;
 
-  tmpReal = (float*)calloc(Half, sizeof(float));
-   tmpImag = (float*)calloc(Half, sizeof(float));
-   RealOut = (float*)calloc(Half, sizeof(float));
-   ImagOut = (float*)calloc(Half, sizeof(float));
+  tmpReal = (float*)xcalloc(Half * 4, sizeof(float));
+  tmpImag = tmpReal + Half;
+  RealOut = tmpImag + Half;
+  ImagOut = RealOut + Half;
 
-   for (i = 0; i < Half; i++) {
-      tmpReal[i] = In[2 * i];
-      tmpImag[i] = In[2 * i + 1];
-   }
+  for (i = 0; i < Half; i++) {
+    tmpReal[i] = In[2 * i];
+    tmpImag[i] = In[2 * i + 1];
+  }
 
-   FFT(Half, 0, tmpReal, tmpImag, RealOut, ImagOut);
+  FFT(Half, 0, tmpReal, tmpImag, RealOut, ImagOut);
+  
+  wtemp = (float) sin(0.5 * theta);
 
-   wtemp = (float) sin(0.5 * theta);
+  wpr = -2.0 * wtemp * wtemp;
+  wpi = (float) sin(theta);
+  wr = 1.0 + wpr;
+  wi = wpi;
+    
+  for (i = 1; i < Half / 2; i++) {
+      
+    i3 = Half - i;
+    
+    h1r = 0.5 * (RealOut[i] + RealOut[i3]);
+    h1i = 0.5 * (ImagOut[i] - ImagOut[i3]);
+    h2r = 0.5 * (ImagOut[i] + ImagOut[i3]);
+    h2i = -0.5 * (RealOut[i] - RealOut[i3]);
+    
+    rt = h1r + wr * h2r - wi * h2i;
+    it = h1i + wr * h2i + wi * h2r;
 
-   wpr = -2.0 * wtemp * wtemp;
-   wpi = (float) sin(theta);
-   wr = 1.0 + wpr;
-   wi = wpi;
+    Out[i] = rt * rt + it * it;
 
-   for (i = 1; i < Half / 2; i++) {
+    rt = h1r - wr * h2r + wi * h2i;
+    it = -h1i + wr * h2i + wi * h2r;
+    
+    Out[i3] = rt * rt + it * it;
+    
+    wr = (wtemp = wr) * wpr - wi * wpi + wr;
+    wi = wi * wpr + wtemp * wpi + wi;
+  }
 
-      i3 = Half - i;
-
-      h1r = 0.5 * (RealOut[i] + RealOut[i3]);
-      h1i = 0.5 * (ImagOut[i] - ImagOut[i3]);
-      h2r = 0.5 * (ImagOut[i] + ImagOut[i3]);
-      h2i = -0.5 * (RealOut[i] - RealOut[i3]);
-
-      rt = h1r + wr * h2r - wi * h2i;
-      it = h1i + wr * h2i + wi * h2r;
-
-      Out[i] = rt * rt + it * it;
-
-      rt = h1r - wr * h2r + wi * h2i;
-      it = -h1i + wr * h2i + wi * h2r;
-
-      Out[i3] = rt * rt + it * it;
-
-      wr = (wtemp = wr) * wpr - wi * wpi + wr;
-      wi = wi * wpr + wtemp * wpi + wi;
-   }
-
-   rt = (h1r = RealOut[0]) + ImagOut[0];
-   it = h1r - ImagOut[0];
-   Out[0] = rt * rt + it * it;
-
-   rt = RealOut[Half / 2];
-   it = ImagOut[Half / 2];
-   Out[Half / 2] = rt * rt + it * it;
-   
-   free(tmpReal);
-   free(tmpImag);
-   free(RealOut);
-   free(ImagOut);
+  rt = (h1r = RealOut[0]) + ImagOut[0];
+  it = h1r - ImagOut[0];
+  Out[0] = rt * rt + it * it;
+  
+  rt = RealOut[Half / 2];
+  it = ImagOut[Half / 2];
+  Out[Half / 2] = rt * rt + it * it;
+  
+  free(tmpReal);
 }
 
 /*
