@@ -1,43 +1,18 @@
-/*
- * September 25, 1991
- * Copyright 1991 Guido van Rossum And Sundry Contributors
- * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Guido van Rossum And Sundry Contributors are not responsible for 
- * the consequences of using this software.
- */
-
 /*
  * Sound Tools SGI/Amiga AIFF format.
  * Used by SGI on 4D/35 and Indigo.
  * This is a subformat of the EA-IFF-85 format.
  * This is related to the IFF format used by the Amiga.
  * But, apparently, not the same.
+ * Also AIFF-C format output that is defined in DAVIC 1.4 Part 9 Annex B
+ * (usable for japanese-data-broadcasting, specified by ARIB STD-B24.)
  *
- * Jan 93: new version from Guido Van Rossum that 
- * correctly skips unwanted sections.
- *
- * Jan 94: add loop & marker support
- * Jul 97: added comments I/O by Leigh Smith
- * Nov 97: added verbose chunk comments
- *
- * June 1, 1998 - Chris Bagwell (cbagwell@sprynet.com)
- *   Fixed compile warnings reported by Kjetil Torgrim Homme
- *   <kjetilho@ifi.uio.no>
- *
- * Sept 9, 1998 - fixed loop markers.
- *
- * Feb. 9, 1999 - Small fix to work with invalid headers that include
- *   a INST block with markers that equal 0.  It should ingore those.
- *   Also fix endian problems when ran on Intel machines.  The check
- *   for endianness was being performed AFTER reading the header instead
- *   of before reading it.
- *
- * Nov 25, 1999 - internal functions made static
- *
- * Jul 12, 2000 - Leigh Smith <leigh@tomandandy.com>
- *   Replaced ANNO with COMT chunk writing headers and added COMT
- *   chunk reading
+ * Copyright 1991-2007 Guido van Rossum And Sundry Contributors
+ * 
+ * This source code is freely redistributable and may be used for
+ * any purpose.  This copyright notice must be maintained. 
+ * Guido van Rossum And Sundry Contributors are not responsible for 
+ * the consequences of using this software.
  */
 
 #include "st_i.h"
@@ -47,9 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>     /* For SEEK_* defines if not found in stdio */
 #endif
@@ -64,6 +37,7 @@ typedef struct aiffpriv {
 /* forward declarations */
 static double read_ieee_extended(ft_t);
 static int aiffwriteheader(ft_t, st_size_t);
+static int aifcwriteheader(ft_t, st_size_t);
 static void write_ieee_extended(ft_t, double);
 static double ConvertFromIeeeExtended(unsigned char*);
 static void ConvertToIeeeExtended(double, char *);
@@ -71,14 +45,14 @@ static int textChunk(char **text, char *chunkDescription, ft_t ft);
 static int commentChunk(char **text, char *chunkDescription, ft_t ft);
 static void reportInstrument(ft_t ft);
 
-int st_aiffseek(ft_t ft, st_size_t offset) 
+static int st_aiffseek(ft_t ft, st_size_t offset) 
 {
     aiff_t aiff = (aiff_t ) ft->priv;
     st_size_t new_offset, channel_block, alignment;
 
-    new_offset = offset * ft->info.size;
+    new_offset = offset * ft->signal.size;
     /* Make sure request aligns to a channel block (ie left+right) */
-    channel_block = ft->info.channels * ft->info.size;
+    channel_block = ft->signal.channels * ft->signal.size;
     alignment = new_offset % channel_block;
     /* Most common mistaken is to compute something like
      * "skip everthing upto and including this sample" so
@@ -91,12 +65,12 @@ int st_aiffseek(ft_t ft, st_size_t offset)
     ft->st_errno = st_seeki(ft, new_offset, SEEK_SET);
 
     if (ft->st_errno == ST_SUCCESS)
-        aiff->nsamples = ft->length - (new_offset / ft->info.size);
+        aiff->nsamples = ft->length - (new_offset / ft->signal.size);
 
     return(ft->st_errno);
 }
 
-int st_aiffstartread(ft_t ft) 
+static int st_aiffstartread(ft_t ft) 
 {
         aiff_t aiff = (aiff_t ) ft->priv;
         char buf[5];
@@ -119,7 +93,7 @@ int st_aiffstartread(ft_t ft)
         unsigned short nmarks = 0;
         unsigned short sustainLoopBegin = 0, sustainLoopEnd = 0,
                        releaseLoopBegin = 0, releaseLoopEnd = 0;
-        st_size_t seekto = 0L, ssndsize = 0L;
+        st_size_t seekto = 0, ssndsize = 0;
         char *author;
         char *copyright;
         char *nametext;
@@ -130,13 +104,6 @@ int st_aiffstartread(ft_t ft)
 
         int rc;
 
-        /* AIFF is in Big Endian format.  Swap whats read in on Little */
-        /* Endian machines.                                            */
-        if (ST_IS_LITTLEENDIAN)
-        {
-            ft->swap = ft->swap ? 0 : 1;
-        }
-
         /* FORM chunk */
         if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
         {
@@ -188,7 +155,7 @@ int st_aiffstartread(ft_t ft)
                             else if (strncmp(buf, "NONE", 4) != 0)
                             {
                                 buf[4] = 0;
-                                st_fail_errno(ft,ST_EHDR,"Can not support AIFC files that contain compressed data: %s",buf);
+                                st_fail_errno(ft,ST_EHDR,"AIFC files that contain compressed data are not supported: %s",buf);
                                 return(ST_EOF);
                             }
                         }
@@ -248,6 +215,7 @@ int st_aiffstartread(ft_t ft)
                                  */
                                 if (st_readb(ft, &len) != ST_SUCCESS)
                                     break;
+                                --chunksize;
                                 if (len > chunksize)
                                     len = chunksize;
                                 read_len = len;
@@ -361,7 +329,7 @@ int st_aiffstartread(ft_t ft)
                         if (st_eof(ft))
                                 break;
                         buf[4] = 0;
-                        st_report("AIFFstartread: ignoring '%s' chunk\n", buf);
+                        st_debug("AIFFstartread: ignoring '%s' chunk", buf);
                         st_readdw(ft, &chunksize);
                         if (st_eof(ft))
                                 break;
@@ -403,26 +371,32 @@ int st_aiffstartread(ft_t ft)
         }
 
         if (foundcomm) {
-                ft->info.channels = channels;
-                ft->info.rate = rate;
-                if (ft->info.encoding != -1 && ft->info.encoding != ST_ENCODING_SIGN2)
+                ft->signal.channels = channels;
+                ft->signal.rate = rate;
+                if (ft->signal.encoding != ST_ENCODING_UNKNOWN && ft->signal.encoding != ST_ENCODING_SIGN2)
                     st_report("AIFF only supports signed data.  Forcing to signed.");
-                ft->info.encoding = ST_ENCODING_SIGN2;
+                ft->signal.encoding = ST_ENCODING_SIGN2;
                 if (bits <= 8)
                 {
-                    ft->info.size = ST_SIZE_BYTE;
+                    ft->signal.size = ST_SIZE_BYTE;
                     if (bits < 8)
                         st_report("Forcing data size from %d bits to 8 bits",bits);
                 }
                 else if (bits <= 16)
                 {
-                    ft->info.size = ST_SIZE_WORD;
+                    ft->signal.size = ST_SIZE_16BIT;
                     if (bits < 16)
                         st_report("Forcing data size from %d bits to 16 bits",bits);
                 }
+                else if (bits <= 24)
+                {
+                    ft->signal.size = ST_SIZE_24BIT;
+                    if (bits < 24)
+                        st_report("Forcing data size from %d bits to 24 bits",bits);
+                }
                 else if (bits <= 32)
                 {
-                    ft->info.size = ST_SIZE_DWORD;
+                    ft->signal.size = ST_SIZE_32BIT;
                     if (bits < 32)
                         st_report("Forcing data size from %d bits to 32 bits",bits);
                 }
@@ -432,11 +406,11 @@ int st_aiffstartread(ft_t ft)
                     return(ST_EOF);
                 }
         } else  {
-                if ((ft->info.channels == -1)
-                        || (ft->info.rate == 0)
-                        || (ft->info.encoding == -1)
-                        || (ft->info.size == -1)) {
-                  st_report("You must specify # channels, sample rate, signed/unsigned,\n");
+                if ((ft->signal.channels == 0)
+                        || (ft->signal.rate == 0)
+                        || (ft->signal.encoding == ST_ENCODING_UNKNOWN)
+                        || (ft->signal.size == -1)) {
+                  st_report("You must specify # channels, sample rate, signed/unsigned,");
                   st_report("and 8/16 on the command line.");
                   st_fail_errno(ft,ST_EFMT,"Bogus AIFF file: no COMM section.");
                   return(ST_EOF);
@@ -444,23 +418,23 @@ int st_aiffstartread(ft_t ft)
 
         }
 
-        aiff->nsamples = ssndsize / ft->info.size;
+        aiff->nsamples = ssndsize / ft->signal.size;
 
         /* Cope with 'sowt' CD tracks as read on Macs */
         if (is_sowt)
         {
                 aiff->nsamples -= 4;
-                ft->swap = ft->swap ? 0 : 1;
+                ft->signal.reverse_bytes = !ft->signal.reverse_bytes;
         }
         
         if (foundmark && !foundinstr)
         {
-            st_report("Ignoring MARK chunk since no INSTR found.");
+            st_debug("Ignoring MARK chunk since no INSTR found.");
             foundmark = 0;
         }
         if (!foundmark && foundinstr)
         {
-            st_report("Ignoring INSTR chunk since no MARK found.");
+            st_debug("Ignoring INSTR chunk since no MARK found.");
             foundinstr = 0;
         }
         if (foundmark && foundinstr) {
@@ -519,7 +493,7 @@ static void reportInstrument(ft_t ft)
   int loopNum;
 
   if(ft->instr.nloops > 0)
-    st_report("AIFF Loop markers:\n");
+    st_report("AIFF Loop markers:");
   for(loopNum  = 0; loopNum < ft->instr.nloops; loopNum++) {
     if (ft->loops[loopNum].count) {
       st_report("Loop %d: start: %6d", loopNum, ft->loops[loopNum].start);
@@ -528,15 +502,15 @@ static void reportInstrument(ft_t ft)
       st_report(" count: %6d", ft->loops[loopNum].count);
       st_report(" type:  ");
       switch(ft->loops[loopNum].type & ~ST_LOOP_SUSTAIN_DECAY) {
-      case 0: st_report("off\n"); break;
-      case 1: st_report("forward\n"); break;
-      case 2: st_report("forward/backward\n"); break;
+      case 0: st_report("off"); break;
+      case 1: st_report("forward"); break;
+      case 2: st_report("forward/backward"); break;
       }
     }
   }
-  st_report("Unity MIDI Note: %d\n", ft->instr.MIDInote);
-  st_report("Low   MIDI Note: %d\n", ft->instr.MIDIlow);
-  st_report("High  MIDI Note: %d\n", ft->instr.MIDIhi);
+  st_report("Unity MIDI Note: %d", ft->instr.MIDInote);
+  st_report("Low   MIDI Note: %d", ft->instr.MIDIlow);
+  st_report("High  MIDI Note: %d", ft->instr.MIDIhi);
 }
 
 /* Process a text chunk, allocate memory, display it if verbose and return */
@@ -545,12 +519,7 @@ static int textChunk(char **text, char *chunkDescription, ft_t ft)
   uint32_t chunksize;
   st_readdw(ft, &chunksize);
   /* allocate enough memory to hold the text including a terminating \0 */
-  *text = (char *) malloc((size_t) chunksize + 1);
-  if (*text == NULL)
-  {
-    st_fail_errno(ft,ST_ENOMEM,"AIFF: Couldn't allocate %s header", chunkDescription);
-    return(ST_EOF);
-  }
+  *text = (char *) xmalloc((size_t) chunksize + 1);
   if (st_readbuf(ft, *text, 1, chunksize) != chunksize)
   {
     st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
@@ -567,7 +536,7 @@ static int textChunk(char **text, char *chunkDescription, ft_t ft)
                         return(ST_EOF);
                 }
         }
-  st_report("%-10s   \"%s\"\n", chunkDescription, *text);
+  st_debug("%-10s   \"%s\"", chunkDescription, *text);
   return(ST_SUCCESS);
 }
 
@@ -596,16 +565,12 @@ static int commentChunk(char **text, char *chunkDescription, ft_t ft)
     totalCommentLength += commentLength;
     /* allocate enough memory to hold the text including a terminating \0 */
     if(commentIndex == 0) {
-      *text = (char *) malloc((size_t) totalCommentLength + 1);
+      *text = (char *) xmalloc((size_t) totalCommentLength + 1);
     }
     else {
-      *text = realloc(*text, (size_t) totalCommentLength + 1);
+      *text = xrealloc(*text, (size_t) totalCommentLength + 1);
     }
 
-    if (*text == NULL) {
-        st_fail_errno(ft,ST_ENOMEM,"AIFF: Couldn't allocate %s header", chunkDescription);
-        return(ST_EOF);
-    }
     if (st_readbuf(ft, *text + totalCommentLength - commentLength, 1, commentLength) != commentLength) {
         st_fail_errno(ft,ST_EOF,"AIFF: Unexpected EOF in %s header", chunkDescription);
         return(ST_EOF);
@@ -621,10 +586,10 @@ static int commentChunk(char **text, char *chunkDescription, ft_t ft)
         }
     }
   }
-  st_report("%-10s   \"%s\"\n", chunkDescription, *text);
+  st_debug("%-10s   \"%s\"", chunkDescription, *text);
   /* make sure we read the whole chunk */
   if (totalReadLength < chunksize) {
-       int i;
+       size_t i;
        char c;
        for (i=0; i < chunksize - totalReadLength; i++ ) {
                st_readbuf(ft, &c, 1, 1);
@@ -633,14 +598,12 @@ static int commentChunk(char **text, char *chunkDescription, ft_t ft)
   return(ST_SUCCESS);
 }
 
-st_ssize_t st_aiffread(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_aiffread(ft_t ft, st_sample_t *buf, st_size_t len)
 {
         aiff_t aiff = (aiff_t ) ft->priv;
         st_ssize_t done;
 
-        if (len < 0)
-            return ST_EOF;
-        else if ((st_size_t)len > aiff->nsamples)
+        if ((st_size_t)len > aiff->nsamples)
                 len = aiff->nsamples;
         done = st_rawread(ft, buf, len);
         if (done == 0 && aiff->nsamples != 0)
@@ -649,7 +612,7 @@ st_ssize_t st_aiffread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return done;
 }
 
-int st_aiffstopread(ft_t ft) 
+static int st_aiffstopread(ft_t ft) 
 {
         char buf[5];
         uint32_t chunksize;
@@ -666,10 +629,10 @@ int st_aiffstopread(ft_t ft)
                 if (st_eof(ft))
                         break;
                 buf[4] = '\0';
-                st_warn("Ignoring AIFF tail chunk: '%s', %d bytes long\n", 
+                st_warn("Ignoring AIFF tail chunk: '%s', %d bytes long", 
                         buf, chunksize);
                 if (! strcmp(buf, "MARK") || ! strcmp(buf, "INST"))
-                        st_warn("       You're stripping MIDI/loop info!\n");
+                        st_warn("       You're stripping MIDI/loop info!");
                 while (chunksize-- > 0) 
                 {
                         if (st_readb(ft, (unsigned char *)&trash) == ST_EOF)
@@ -692,7 +655,7 @@ int st_aiffstopread(ft_t ft)
    Strictly spoken this is not legal, but the playaiff utility
    will still be able to play the resulting file. */
 
-int st_aiffstartwrite(ft_t ft)
+static int st_aiffstartwrite(ft_t ft)
 {
         aiff_t aiff = (aiff_t ) ft->priv;
         int rc;
@@ -702,24 +665,16 @@ int st_aiffstartwrite(ft_t ft)
         if (rc)
             return rc;
 
-        /* AIFF is in Big Endian format.  Swap whats read in on Little */
-        /* Endian machines.                                            */
-        if (ST_IS_LITTLEENDIAN)
-        {
-            ft->swap = ft->swap ? 0 : 1;
-        }
-
         aiff->nsamples = 0;
-        if ((ft->info.encoding == ST_ENCODING_ULAW ||
-             ft->info.encoding == ST_ENCODING_ALAW) && 
-            ft->info.size == ST_SIZE_BYTE) {
-                st_report("expanding 8-bit u-law to signed 16 bits");
-                ft->info.encoding = ST_ENCODING_SIGN2;
-                ft->info.size = ST_SIZE_WORD;
+        if (ft->signal.encoding < ST_ENCODING_SIZE_IS_WORD && 
+            ft->signal.size == ST_SIZE_BYTE) {
+                st_report("expanding compressed bytes to signed 16 bits");
+                ft->signal.encoding = ST_ENCODING_SIGN2;
+                ft->signal.size = ST_SIZE_16BIT;
         }
-        if (ft->info.encoding != -1 && ft->info.encoding != ST_ENCODING_SIGN2)
+        if (ft->signal.encoding != ST_ENCODING_UNKNOWN && ft->signal.encoding != ST_ENCODING_SIGN2)
             st_report("AIFF only supports signed data.  Forcing to signed.");
-        ft->info.encoding = ST_ENCODING_SIGN2; /* We have a fixed encoding */
+        ft->signal.encoding = ST_ENCODING_SIGN2; /* We have a fixed encoding */
 
         /* Compute the "very large number" so that a maximum number
            of samples can be transmitted through a pipe without the
@@ -727,10 +682,10 @@ int st_aiffstartwrite(ft_t ft)
            At 48 kHz, 16 bits stereo, this gives ~3 hours of music.
            Sorry, the AIFF format does not provide for an "infinite"
            number of samples. */
-        return(aiffwriteheader(ft, 0x7f000000L / (ft->info.size*ft->info.channels)));
+        return(aiffwriteheader(ft, 0x7f000000 / (ft->signal.size*ft->signal.channels)));
 }
 
-st_ssize_t st_aiffwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_aiffwrite(ft_t ft, const st_sample_t *buf, st_size_t len)
 {
         aiff_t aiff = (aiff_t ) ft->priv;
         aiff->nsamples += len;
@@ -738,11 +693,19 @@ st_ssize_t st_aiffwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return(len);
 }
 
-int st_aiffstopwrite(ft_t ft)
+static int st_aiffstopwrite(ft_t ft)
 {
         aiff_t aiff = (aiff_t ) ft->priv;
         int rc;
 
+        /* If we've written an odd number of bytes, write a padding
+           NUL */
+        if (aiff->nsamples % 2 == 1 && ft->signal.size == 1 && ft->signal.channels == 1)
+        {
+            st_sample_t buf = 0;
+            st_rawwrite(ft, &buf, 1);
+        }
+
         /* Needed because of st_rawwrite().  Call now to flush
          * buffer now before seeking around below.
          */
@@ -755,12 +718,12 @@ int st_aiffstopwrite(ft_t ft)
             st_fail_errno(ft,ST_EOF,"Non-seekable file.");
             return(ST_EOF);
         }
-        if (st_seeki(ft, 0L, SEEK_SET) != 0)
+        if (st_seeki(ft, 0, SEEK_SET) != 0)
         {
                 st_fail_errno(ft,errno,"can't rewind output file to rewrite AIFF header");
                 return(ST_EOF);
         }
-        return(aiffwriteheader(ft, aiff->nsamples / ft->info.channels));
+        return(aiffwriteheader(ft, aiff->nsamples / ft->signal.channels));
 }
 
 static int aiffwriteheader(ft_t ft, st_size_t nframes)
@@ -772,7 +735,7 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
         int i;
         int padded_comment_size = 0;
         int comment_size = 0;
-        st_size_t comment_chunk_size = 0L;
+        st_size_t comment_chunk_size = 0;
 
         /* MARK and INST chunks */
         if (ft->instr.nloops) {
@@ -780,14 +743,17 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
           hsize += 8 /* INST hdr */ + 20; /* INST chunk */
         }
 
-        if (ft->info.encoding == ST_ENCODING_SIGN2 && 
-            ft->info.size == ST_SIZE_BYTE)
+        if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+            ft->signal.size == ST_SIZE_BYTE)
                 bits = 8;
-        else if (ft->info.encoding == ST_ENCODING_SIGN2 && 
-                 ft->info.size == ST_SIZE_WORD)
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_16BIT)
                 bits = 16;
-        else if (ft->info.encoding == ST_ENCODING_SIGN2 && 
-                 ft->info.size == ST_SIZE_DWORD)
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_24BIT)
+                bits = 24;
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_32BIT)
                 bits = 32;
         else
         {
@@ -806,13 +772,13 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
           padded_comment_size = ((comment_size % 2) == 0) ?
                                 comment_size : comment_size + 1;
           /* one comment, timestamp, marker ID and text count */
-          comment_chunk_size = (2L + 4 + 2 + 2 + padded_comment_size);
+          comment_chunk_size = (2 + 4 + 2 + 2 + padded_comment_size);
           hsize += 8 /* COMT hdr */ + comment_chunk_size; 
         }
 
         st_writes(ft, "FORM"); /* IFF header */
         /* file size */
-        st_writedw(ft, hsize + nframes * ft->info.size * ft->info.channels); 
+        st_writedw(ft, hsize + nframes * ft->signal.size * ft->signal.channels); 
         st_writes(ft, "AIFF"); /* File type */
 
         /* Now we write the COMT comment chunk using the precomputed sizes */
@@ -826,7 +792,7 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
 
           /* time stamp of comment, Unix knows of time from 1/1/1970,
              Apple knows time from 1/1/1904 */
-          st_writedw(ft, ((int32_t) time(NULL)) + 2082844800L);
+          st_writedw(ft, ((int32_t) time(NULL)) + 2082844800);
 
           /* A marker ID of 0 indicates the comment is not associated
              with a marker */
@@ -842,10 +808,10 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
         /* COMM chunk -- describes encoding (and #frames) */
         st_writes(ft, "COMM");
         st_writedw(ft, 18); /* COMM chunk size */
-        st_writew(ft, ft->info.channels); /* nchannels */
+        st_writew(ft, ft->signal.channels); /* nchannels */
         st_writedw(ft, nframes); /* number of frames */
         st_writew(ft, bits); /* sample width, in bits */
-        write_ieee_extended(ft, (double)ft->info.rate);
+        write_ieee_extended(ft, (double)ft->signal.rate);
 
         /* MARK chunk -- set markers */
         if (ft->instr.nloops) {
@@ -896,12 +862,134 @@ static int aiffwriteheader(ft_t ft, st_size_t nframes)
         /* SSND chunk -- describes data */
         st_writes(ft, "SSND");
         /* chunk size */
-        st_writedw(ft, 8 + nframes * ft->info.channels * ft->info.size); 
+        st_writedw(ft, 8 + nframes * ft->signal.channels * ft->signal.size); 
         st_writedw(ft, 0); /* offset */
         st_writedw(ft, 0); /* block size */
         return(ST_SUCCESS);
 }
 
+static int st_aifcstartwrite(ft_t ft)
+{
+        aiff_t aiff = (aiff_t ) ft->priv;
+        int rc;
+
+        /* Needed because st_rawwrite() */
+        rc = st_rawstartwrite(ft);
+        if (rc)
+            return rc;
+
+        aiff->nsamples = 0;
+        if (ft->signal.encoding < ST_ENCODING_SIZE_IS_WORD && 
+            ft->signal.size == ST_SIZE_BYTE) {
+                st_report("expanding compressed bytes to signed 16 bits");
+                ft->signal.encoding = ST_ENCODING_SIGN2;
+                ft->signal.size = ST_SIZE_16BIT;
+        }
+        if (ft->signal.encoding != ST_ENCODING_UNKNOWN && ft->signal.encoding != ST_ENCODING_SIGN2)
+            st_report("AIFC only supports signed data.  Forcing to signed.");
+        ft->signal.encoding = ST_ENCODING_SIGN2; /* We have a fixed encoding */
+
+        /* Compute the "very large number" so that a maximum number
+           of samples can be transmitted through a pipe without the
+           risk of causing overflow when calculating the number of bytes.
+           At 48 kHz, 16 bits stereo, this gives ~3 hours of music.
+           Sorry, the AIFC format does not provide for an "infinite"
+           number of samples. */
+        return(aifcwriteheader(ft, 0x7f000000 / (ft->signal.size*ft->signal.channels)));
+}
+
+static int st_aifcstopwrite(ft_t ft)
+{
+        aiff_t aiff = (aiff_t ) ft->priv;
+        int rc;
+
+        /* If we've written an odd number of bytes, write a padding
+           NUL */
+        if (aiff->nsamples % 2 == 1 && ft->signal.size == 1 && ft->signal.channels == 1)
+        {
+            st_sample_t buf = 0;
+            st_rawwrite(ft, &buf, 1);
+        }
+
+        /* Needed because of st_rawwrite().  Call now to flush
+         * buffer now before seeking around below.
+         */
+        rc = st_rawstopwrite(ft);
+        if (rc)
+            return rc;
+
+        if (!ft->seekable)
+        {
+            st_fail_errno(ft,ST_EOF,"Non-seekable file.");
+            return(ST_EOF);
+        }
+        if (st_seeki(ft, 0, SEEK_SET) != 0)
+        {
+                st_fail_errno(ft,errno,"can't rewind output file to rewrite AIFC header");
+                return(ST_EOF);
+        }
+        return(aifcwriteheader(ft, aiff->nsamples / ft->signal.channels));
+}
+
+static int aifcwriteheader(ft_t ft, st_size_t nframes)
+{
+        int hsize =
+                12 /*FVER*/ + 8 /*COMM hdr*/ + 18+4+1+15 /*COMM chunk*/ +
+                8 /*SSND hdr*/ + 12 /*SSND chunk*/;
+        int bits = 0;
+
+        if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+            ft->signal.size == ST_SIZE_BYTE)
+                bits = 8;
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_16BIT)
+                bits = 16;
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_24BIT)
+                bits = 24;
+        else if (ft->signal.encoding == ST_ENCODING_SIGN2 && 
+                 ft->signal.size == ST_SIZE_32BIT)
+                bits = 32;
+        else
+        {
+                st_fail_errno(ft,ST_EFMT,"unsupported output encoding/size for AIFC header");
+                return(ST_EOF);
+        }
+
+        st_writes(ft, "FORM"); /* IFF header */
+        /* file size */
+        st_writedw(ft, hsize + nframes * ft->signal.size * ft->signal.channels); 
+        st_writes(ft, "AIFC"); /* File type */
+
+        /* FVER chunk */
+        st_writes(ft, "FVER");
+        st_writedw(ft, 4); /* FVER chunk size */
+        st_writedw(ft, 0xa2805140); /* version_date(May23,1990,2:40pm) */
+
+        /* COMM chunk -- describes encoding (and #frames) */
+        st_writes(ft, "COMM");
+        st_writedw(ft, 18+4+1+15); /* COMM chunk size */
+        st_writew(ft, ft->signal.channels); /* nchannels */
+        st_writedw(ft, nframes); /* number of frames */
+        st_writew(ft, bits); /* sample width, in bits */
+        write_ieee_extended(ft, (double)ft->signal.rate);
+
+        st_writes(ft, "NONE"); /*compression_type*/
+        st_writeb(ft, 14);
+        st_writes(ft, "not compressed");
+        st_writeb(ft, 0);
+
+        /* SSND chunk -- describes data */
+        st_writes(ft, "SSND");
+        /* chunk size */
+        st_writedw(ft, 8 + nframes * ft->signal.channels * ft->signal.size); 
+        st_writedw(ft, 0); /* offset */
+        st_writedw(ft, 0); /* block size */
+
+        /* Any Private chunks shall appear after the required chunks (FORM,FVER,COMM,SSND) */
+        return(ST_SUCCESS);
+}
+
 static double read_ieee_extended(ft_t ft)
 {
         char buf[10];
@@ -918,7 +1006,7 @@ static void write_ieee_extended(ft_t ft, double x)
         char buf[10];
         ConvertToIeeeExtended(x, buf);
         /*
-        st_report("converted %g to %o %o %o %o %o %o %o %o %o %o",
+        st_debug("converted %g to %o %o %o %o %o %o %o %o %o %o",
                 x,
                 buf[0], buf[1], buf[2], buf[3], buf[4],
                 buf[5], buf[6], buf[7], buf[8], buf[9]);
@@ -936,7 +1024,7 @@ static void write_ieee_extended(ft_t ft, double x)
  *
  * Machine-independent I/O routines for IEEE floating-point numbers.
  *
- * NaN's and infinities are converted to HUGE_VAL or HUGE, which
+ * NaN's and infinities are converted to HUGE_VAL, which
  * happens to be infinity on IEEE machines.  Unfortunately, it is
  * impossible to preserve NaN's in a machine-independent way.
  * Infinities are, however, preserved on IEEE machines.
@@ -962,11 +1050,7 @@ static void write_ieee_extended(ft_t ft, double x)
  * NaN's, and denormalized numbers.
  */
 
-#ifndef HUGE_VAL
-# define HUGE_VAL HUGE
-#endif /*HUGE_VAL*/
-
-# define FloatToUnsigned(f) ((uint32_t)(((int32_t)(f - 2147483648.0)) + 2147483647L) + 1)
+#define FloatToUnsigned(f) ((uint32_t)(((int32_t)(f - 2147483648.0)) + 2147483647) + 1)
 
 static void ConvertToIeeeExtended(double num, char *bytes)
 {
@@ -1029,7 +1113,7 @@ static void ConvertToIeeeExtended(double num, char *bytes)
  *
  * Machine-independent I/O routines for IEEE floating-point numbers.
  *
- * NaN's and infinities are converted to HUGE_VAL or HUGE, which
+ * NaN's and infinities are converted to HUGE_VAL, which
  * happens to be infinity on IEEE machines.  Unfortunately, it is
  * impossible to preserve NaN's in a machine-independent way.
  * Infinities are, however, preserved on IEEE machines.
@@ -1055,11 +1139,7 @@ static void ConvertToIeeeExtended(double num, char *bytes)
  * NaN's, and denormalized numbers.
  */
 
-#ifndef HUGE_VAL
-# define HUGE_VAL HUGE
-#endif /*HUGE_VAL*/
-
-# define UnsignedToFloat(u)         (((double)((int32_t)(u - 2147483647L - 1))) + 2147483648.0)
+#define UnsignedToFloat(u)         (((double)((int32_t)(u - 2147483647 - 1))) + 2147483648.0)
 
 /****************************************************************
  * Extended precision IEEE floating-point conversion routine.
@@ -1102,7 +1182,7 @@ static double ConvertFromIeeeExtended(unsigned char *bytes)
 }
 
 
-static char *aiffnames[] = {
+static const char *aiffnames[] = {
   "aiff",
   "aif",
   NULL
@@ -1111,7 +1191,7 @@ static char *aiffnames[] = {
 static st_format_t st_aiff_format = {
   aiffnames,
   NULL,
-  ST_FILE_STEREO | ST_FILE_LOOPS | ST_FILE_SEEK,
+  ST_FILE_LOOPS | ST_FILE_SEEK | ST_FILE_BIG_END,
   st_aiffstartread,
   st_aiffread,
   st_aiffstopread,
@@ -1125,3 +1205,27 @@ const st_format_t *st_aiff_format_fn(void)
 {
     return &st_aiff_format;
 }
+
+static const char *aifcnames[] = {
+  "aifc",
+  "aiffc",
+  NULL
+};
+
+static st_format_t st_aifc_format = {
+  aifcnames,
+  NULL,
+  ST_FILE_LOOPS | ST_FILE_SEEK | ST_FILE_BIG_END,
+  st_aiffstartread,
+  st_aiffread,
+  st_aiffstopread,
+  st_aifcstartwrite,
+  st_aiffwrite,
+  st_aifcstopwrite,
+  st_aiffseek
+};
+
+const st_format_t *st_aifc_format_fn(void)
+{
+    return &st_aifc_format;
+}
