@@ -23,9 +23,7 @@
 #include "g72x.h"
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 
 /* Magic numbers used in Sun and NeXT audio files */
 #define SUN_MAGIC       0x2e736e64              /* Really '.snd' */
@@ -63,7 +61,7 @@ typedef struct aupriv {
 
 static void auwriteheader(ft_t ft, st_size_t data_size);
 
-static int st_auencodingandsize(int sun_encoding, signed char *encoding, signed char *size)
+static int st_auencodingandsize(int sun_encoding, st_encoding_t * encoding, int * size)
 {
     switch (sun_encoding) {
     case SUN_ULAW:
@@ -80,32 +78,36 @@ static int st_auencodingandsize(int sun_encoding, signed char *encoding, signed
             break;
     case SUN_LIN_16:
             *encoding = ST_ENCODING_SIGN2;
-            *size = ST_SIZE_WORD;
+            *size = ST_SIZE_16BIT;
+            break;
+    case SUN_LIN_24:
+            *encoding = ST_ENCODING_SIGN2;
+            *size = ST_SIZE_24BIT;
             break;
     case SUN_G721:
             *encoding = ST_ENCODING_SIGN2;
-            *size = ST_SIZE_WORD;
+            *size = ST_SIZE_16BIT;
             break;
     case SUN_G723_3:
             *encoding = ST_ENCODING_SIGN2;
-            *size = ST_SIZE_WORD;
+            *size = ST_SIZE_16BIT;
             break;
     case SUN_G723_5:
             *encoding = ST_ENCODING_SIGN2;
-            *size = ST_SIZE_WORD;
+            *size = ST_SIZE_16BIT;
             break;
     case SUN_FLOAT:
             *encoding = ST_ENCODING_FLOAT;
             *size = ST_SIZE_32BIT;
             break;
     default:
-            st_report("encoding: 0x%lx", encoding);
+            st_debug("encoding: 0x%lx", encoding);
             return(ST_EOF);
     }
     return(ST_SUCCESS);
 }
 
-int st_auseek(ft_t ft, st_size_t offset) 
+static int st_auseek(ft_t ft, st_size_t offset) 
 {
     au_t au = (au_t ) ft->priv;
 
@@ -117,9 +119,9 @@ int st_auseek(ft_t ft, st_size_t offset)
     {
         st_size_t new_offset, channel_block, alignment;
 
-        new_offset = offset * ft->info.size;
+        new_offset = offset * ft->signal.size;
         /* Make sure request aligns to a channel block (ie left+right) */
-        channel_block = ft->info.channels * ft->info.size;
+        channel_block = ft->signal.channels * ft->signal.size;
         alignment = new_offset % channel_block;
         /* Most common mistaken is to compute something like
          * "skip everthing upto and including this sample" so
@@ -135,7 +137,7 @@ int st_auseek(ft_t ft, st_size_t offset)
     return(ft->st_errno);
 }
 
-int st_austartread(ft_t ft) 
+static int st_austartread(ft_t ft) 
 {
         /* The following 6 variables represent a Sun sound header on disk.
            The numbers are written as big-endians.
@@ -155,14 +157,6 @@ int st_austartread(ft_t ft)
 
         int rc;
 
-        /* AU is in big endian format.  Swap whats read
-         * in onlittle endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         /* Check the magic word */
         st_readdw(ft, &magic);
         if (magic == DEC_INV_MAGIC) {
@@ -170,18 +164,18 @@ int st_austartread(ft_t ft)
                  * left over from pre-standardize period of testing for
                  * endianess.  Its not hurting though.
                  */
-                ft->swap = ft->swap ? 0 : 1;
-                st_report("Found inverted DEC magic word.  Swapping bytes.");
+                ft->signal.reverse_bytes = !ft->signal.reverse_bytes;
+                st_debug("Found inverted DEC magic word.  Swapping bytes.");
         }
         else if (magic == SUN_INV_MAGIC) {
-                ft->swap = ft->swap ? 0 : 1;
-                st_report("Found inverted Sun/NeXT magic word. Swapping bytes.");
+                ft->signal.reverse_bytes = !ft->signal.reverse_bytes;
+                st_debug("Found inverted Sun/NeXT magic word. Swapping bytes.");
         }
         else if (magic == SUN_MAGIC) {
-                st_report("Found Sun/NeXT magic word");
+                st_debug("Found Sun/NeXT magic word");
         }
         else if (magic == DEC_MAGIC) {
-                st_report("Found DEC magic word");
+                st_debug("Found DEC magic word");
         }
         else
         {
@@ -209,8 +203,8 @@ int st_austartread(ft_t ft)
         p->dec_routine = NULL;
         p->in_buffer = 0;
         p->in_bits = 0;
-        if(st_auencodingandsize(encoding, &(ft->info.encoding),
-                             &(ft->info.size)) == ST_EOF)
+        if(st_auencodingandsize(encoding, &(ft->signal.encoding),
+                             &(ft->signal.size)) == ST_EOF)
         {
             st_fail_errno(ft,ST_EFMT,"Unsupported encoding in Sun/NeXT header.\nOnly U-law, signed bytes, signed words, ADPCM, and 32-bit floats are supported.");
             return(ST_EOF);
@@ -233,19 +227,28 @@ int st_austartread(ft_t ft)
                 break;
         }
 
+
         /* Read the sampling rate */
         st_readdw(ft, &sample_rate);
-        ft->info.rate = sample_rate;
+        if (ft->signal.rate == 0 || ft->signal.rate == sample_rate)
+            ft->signal.rate = sample_rate;
+        else
+            st_report("User options overriding rate read in .au header");
 
         /* Read the number of channels */
         st_readdw(ft, &channels);
-        ft->info.channels = (int) channels;
+        if (ft->signal.channels == 0 || ft->signal.channels == channels)
+            ft->signal.channels = channels;
+        else
+            st_report("User options overriding channels read in .au header");
+
 
         /* Skip the info string in header; print it if verbose */
         hdr_size -= SUN_HDRSIZE; /* #bytes already read */
         if (hdr_size > 0) {
                 /* Allocate comment buffer */
-                buf = (char *) malloc(hdr_size+1);              
+                buf = (char *) xmalloc(hdr_size+1);
+                
                 for(i = 0; i < hdr_size; i++) {
                         st_readb(ft, (unsigned char *)&(buf[i]));
                         if (st_eof(ft))
@@ -255,7 +258,7 @@ int st_austartread(ft_t ft)
                         }
                 }
                 /* Buffer should already be null terminated but
-                 * just in case we malloced an extra byte and 
+                 * just in case we xmalloced an extra byte and 
                  * force the last byte to be 0 anyways.
                  * This should help work with a greater array of
                  * software.
@@ -263,10 +266,9 @@ int st_austartread(ft_t ft)
                 buf[hdr_size] = '\0';
 
                 ft->comment = buf;
-                st_report("Input file %s: Sun header info: %s", ft->filename, buf);
         }
         /* Needed for seeking */
-        ft->length = data_size/ft->info.size;
+        ft->length = data_size/ft->signal.size;
         if(ft->seekable)
                 p->dataStart = st_tell(ft);
 
@@ -287,7 +289,7 @@ int st_austartread(ft_t ft)
    if it is not, the unspecified size remains in the header
    (this is legal). */
 
-int st_austartwrite(ft_t ft) 
+static int st_austartwrite(ft_t ft) 
 {
         au_t p = (au_t ) ft->priv;
         int rc;
@@ -297,14 +299,6 @@ int st_austartwrite(ft_t ft)
         if (rc)
             return rc;
 
-        /* AU is in big endian format.  Swap whats read in
-         * on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         p->data_size = 0;
         auwriteheader(ft, SUN_UNSPEC);
         return(ST_SUCCESS);
@@ -334,7 +328,7 @@ static int unpack_input(ft_t ft, unsigned char *code)
         return (p->in_bits > 0);
 }
 
-st_ssize_t st_auread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_auread(ft_t ft, st_sample_t *buf, st_size_t samp)
 {
         au_t p = (au_t ) ft->priv;
         unsigned char code;
@@ -345,21 +339,21 @@ st_ssize_t st_auread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
         while (samp > 0 && unpack_input(ft, &code) >= 0) {
                 *buf++ = ST_SIGNED_WORD_TO_SAMPLE(
                         (*p->dec_routine)(code, AUDIO_ENCODING_LINEAR,
-                                          &p->state));
+                                          &p->state),);
                 samp--;
                 done++;
         }
         return done;
 }
 
-st_ssize_t st_auwrite(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_auwrite(ft_t ft, const st_sample_t *buf, st_size_t samp)
 {
         au_t p = (au_t ) ft->priv;
-        p->data_size += samp * ft->info.size;
+        p->data_size += samp * ft->signal.size;
         return(st_rawwrite(ft, buf, samp));
 }
 
-int st_austopwrite(ft_t ft)
+static int st_austopwrite(ft_t ft)
 {
         au_t p = (au_t ) ft->priv;
         int rc;
@@ -374,7 +368,7 @@ int st_austopwrite(ft_t ft)
         /* Attempt to update header */
         if (ft->seekable)
         {
-          if (st_seeki(ft, 0L, 0) != 0)
+          if (st_seeki(ft, 0, 0) != 0)
           {
                 st_fail_errno(ft,errno,"Can't rewind output file to rewrite Sun header.");
                 return(ST_EOF);
@@ -394,12 +388,14 @@ static int st_ausunencoding(int size, int encoding)
                 sun_encoding = SUN_ALAW;
         else if (encoding == ST_ENCODING_SIGN2 && size == ST_SIZE_BYTE)
                 sun_encoding = SUN_LIN_8;
-        else if (encoding == ST_ENCODING_SIGN2 && size == ST_SIZE_WORD)
+        else if (encoding == ST_ENCODING_SIGN2 && size == ST_SIZE_16BIT)
                 sun_encoding = SUN_LIN_16;
+        else if (encoding == ST_ENCODING_SIGN2 && size == ST_SIZE_24BIT)
+                sun_encoding = SUN_LIN_24;
         else if (encoding == ST_ENCODING_FLOAT && size == ST_SIZE_32BIT)
                 sun_encoding = SUN_FLOAT;
         else
-                sun_encoding = -1;
+                sun_encoding = ST_ENCODING_UNKNOWN;
         return sun_encoding;
 }
 
@@ -413,14 +409,14 @@ static void auwriteheader(ft_t ft, st_size_t data_size)
         int   x;
         int   comment_size;
 
-        if ((encoding = st_ausunencoding(ft->info.size, ft->info.encoding)) == -1) {
+        if ((encoding = st_ausunencoding(ft->signal.size, ft->signal.encoding)) == -1) {
                 st_report("Unsupported output encoding/size for Sun/NeXT header or .AU format not specified.");
-                st_report("Only U-law, A-law signed bytes, and signed words are supported.");
-                st_report("Defaulting to 8khz u-law\n");
+                st_report("Only U-law, A-law, and signed bytes/words/tri-bytes are supported.");
+                st_report("Defaulting to 8khz u-law");
                 encoding = SUN_ULAW;
-                ft->info.encoding = ST_ENCODING_ULAW;
-                ft->info.size = ST_SIZE_BYTE;
-                ft->info.rate = 8000;  /* strange but true */
+                ft->signal.encoding = ST_ENCODING_ULAW;
+                ft->signal.size = ST_SIZE_BYTE;
+                ft->signal.rate = 8000;  /* strange but true */
         }
 
         magic = SUN_MAGIC;
@@ -430,7 +426,7 @@ static void auwriteheader(ft_t ft, st_size_t data_size)
          * useful when there is no comments.
          */
         if (ft->comment == NULL)
-                ft->comment = "SOX";
+                ft->comment = xstrdup("SOX");
 
         hdr_size = SUN_HDRSIZE;
 
@@ -446,10 +442,10 @@ static void auwriteheader(ft_t ft, st_size_t data_size)
 
         st_writedw(ft, encoding);
 
-        sample_rate = ft->info.rate;
+        sample_rate = ft->signal.rate;
         st_writedw(ft, sample_rate);
 
-        channels = ft->info.channels;
+        channels = ft->signal.channels;
         st_writedw(ft, channels);
 
         st_writes(ft, ft->comment);
@@ -463,7 +459,7 @@ static void auwriteheader(ft_t ft, st_size_t data_size)
 }
 
 /* SPARC .au w/header */
-static char *aunames[] = {
+static const char *aunames[] = {
   "au",
   "snd",
   NULL
@@ -472,7 +468,7 @@ static char *aunames[] = {
 static st_format_t st_au_format = {
   aunames,
   NULL,
-  ST_FILE_STEREO | ST_FILE_SEEK,
+  ST_FILE_SEEK | ST_FILE_BIG_END,
   st_austartread,
   st_auread,
   st_rawstopread,
