@@ -63,18 +63,11 @@ typedef struct avrstuff {
  */
 
 
-int st_avrstartread(ft_t ft) 
+static int st_avrstartread(ft_t ft) 
 {
   avr_t avr = (avr_t)ft->priv;
   int rc;
 
-  /* AVR is a Big Endian format.  Swap whats read in on Little */
-  /* Endian machines.                                          */
-  if (ST_IS_LITTLEENDIAN)
-  {
-          ft->swap = ft->swap ? 0 : 1;
-  }
-
   st_reads(ft, avr->magic, 4);
 
   if (strncmp (avr->magic, AVR_MAGIC, 4)) {
@@ -86,18 +79,18 @@ int st_avrstartread(ft_t ft)
 
   st_readw (ft, &(avr->mono));
   if (avr->mono) {
-    ft->info.channels = 2;
+    ft->signal.channels = 2;
   }
   else {
-    ft->info.channels = 1;
+    ft->signal.channels = 1;
   }
 
   st_readw (ft, &(avr->rez));
   if (avr->rez == 8) {
-    ft->info.size = ST_SIZE_BYTE;
+    ft->signal.size = ST_SIZE_BYTE;
   }
   else if (avr->rez == 16) {
-    ft->info.size = ST_SIZE_WORD;
+    ft->signal.size = ST_SIZE_16BIT;
   }
   else {
     st_fail_errno(ft,ST_EFMT,"AVR: unsupported sample resolution");
@@ -106,10 +99,10 @@ int st_avrstartread(ft_t ft)
 
   st_readw (ft, &(avr->sign));
   if (avr->sign) {
-    ft->info.encoding = ST_ENCODING_SIGN2;
+    ft->signal.encoding = ST_ENCODING_SIGN2;
   }
   else {
-    ft->info.encoding = ST_ENCODING_UNSIGNED;
+    ft->signal.encoding = ST_ENCODING_UNSIGNED;
   }
 
   st_readw (ft, &(avr->loop));
@@ -123,7 +116,7 @@ int st_avrstartread(ft_t ft)
    *
    * Just masking the upper byte out.
    */
-  ft->info.rate = (avr->rate & 0x00ffffff);
+  ft->signal.rate = (avr->rate & 0x00ffffff);
 
   st_readdw (ft, &(avr->size));
 
@@ -148,18 +141,11 @@ int st_avrstartread(ft_t ft)
   return(ST_SUCCESS);
 }
 
-int st_avrstartwrite(ft_t ft) 
+static int st_avrstartwrite(ft_t ft) 
 {
   avr_t avr = (avr_t)ft->priv;
   int rc;
 
-  /* AVR is a Big Endian format.  Swap whats read in on Little */
-  /* Endian machines.                                          */
-  if (ST_IS_LITTLEENDIAN)
-  {
-          ft->swap = ft->swap ? 0 : 1;
-  }
-
   if (!ft->seekable) {
     st_fail_errno(ft,ST_EOF,"AVR: file is not seekable");
     return(ST_EOF);
@@ -183,10 +169,10 @@ int st_avrstartwrite(ft_t ft)
   st_writeb(ft, 0);
 
   /* mono */
-  if (ft->info.channels == 1) {
+  if (ft->signal.channels == 1) {
     st_writew (ft, 0);
   }
-  else if (ft->info.channels == 2) {
+  else if (ft->signal.channels == 2) {
     st_writew (ft, 0xffff);
   }
   else {
@@ -195,10 +181,10 @@ int st_avrstartwrite(ft_t ft)
   }
 
   /* rez */
-  if (ft->info.size == ST_SIZE_BYTE) {
+  if (ft->signal.size == ST_SIZE_BYTE) {
     st_writew (ft, 8);
   }
-  else if (ft->info.size == ST_SIZE_WORD) {
+  else if (ft->signal.size == ST_SIZE_16BIT) {
     st_writew (ft, 16);
   }
   else {
@@ -207,10 +193,10 @@ int st_avrstartwrite(ft_t ft)
   }
 
   /* sign */
-  if (ft->info.encoding == ST_ENCODING_SIGN2) {
+  if (ft->signal.encoding == ST_ENCODING_SIGN2) {
     st_writew (ft, 0xffff);
   }
-  else if (ft->info.encoding == ST_ENCODING_UNSIGNED) {
+  else if (ft->signal.encoding == ST_ENCODING_UNSIGNED) {
     st_writew (ft, 0);
   }
   else {
@@ -225,7 +211,7 @@ int st_avrstartwrite(ft_t ft)
   st_writew (ft, 0xffff);
 
   /* rate */
-  st_writedw (ft, ft->info.rate);
+  st_writedw (ft, ft->signal.rate);
 
   /* size */
   /* Don't know the size yet. */
@@ -260,7 +246,7 @@ int st_avrstartwrite(ft_t ft)
   return(ST_SUCCESS);
 }
 
-st_ssize_t st_avrwrite(ft_t ft, st_sample_t *buf, st_ssize_t nsamp) 
+static st_size_t st_avrwrite(ft_t ft, const st_sample_t *buf, st_size_t nsamp) 
 {
   avr_t avr = (avr_t)ft->priv;
 
@@ -269,29 +255,29 @@ st_ssize_t st_avrwrite(ft_t ft, st_sample_t *buf, st_ssize_t nsamp)
   return (st_rawwrite (ft, buf, nsamp));
 }
 
-int st_avrstopwrite(ft_t ft) 
+static int st_avrstopwrite(ft_t ft) 
 {
   avr_t avr = (avr_t)ft->priv;
   int rc;
 
-  int size = avr->size / ft->info.channels;
+  int size = avr->size / ft->signal.channels;
 
   rc = st_rawstopwrite(ft);
   if (rc)
       return rc;
 
   /* Fix size */
-  st_seeki(ft, 26L, SEEK_SET);
+  st_seeki(ft, 26, SEEK_SET);
   st_writedw (ft, size);
 
   /* Fix lend */
-  st_seeki(ft, 34L, SEEK_SET);
+  st_seeki(ft, 34, SEEK_SET);
   st_writedw (ft, size);
 
   return(ST_SUCCESS);
 }
 
-static char *avrnames[] = {
+static const char *avrnames[] = {
   "avr",
   NULL
 };
@@ -299,7 +285,7 @@ static char *avrnames[] = {
 static st_format_t st_avr_format = {
   avrnames,
   NULL,
-  ST_FILE_STEREO,
+  ST_FILE_BIG_END,
   st_avrstartread,
   st_rawread,
   st_format_nothing,
