@@ -39,7 +39,7 @@ typedef struct cdrstuff {
  *      mono/stereo/quad.
  */
 
-int st_cdrstartread(ft_t ft) 
+static int st_cdrstartread(ft_t ft) 
 {
         int rc;
 
@@ -48,22 +48,15 @@ int st_cdrstartread(ft_t ft)
         if (rc)
             return rc;
 
-        /* CDR is in Big Endian format.  Swap whats read in on */
-        /* Little Endian machines.                             */
-        if (ST_IS_LITTLEENDIAN)
-        { 
-            ft->swap = ft->swap ? 0 : 1;
-        }
-
-        ft->info.rate = 44100L;
-        ft->info.size = ST_SIZE_WORD;
-        ft->info.encoding = ST_ENCODING_SIGN2;
-        ft->info.channels = 2;
+        ft->signal.rate = 44100;
+        ft->signal.size = ST_SIZE_16BIT;
+        ft->signal.encoding = ST_ENCODING_SIGN2;
+        ft->signal.channels = 2;
         ft->comment = NULL;
 
 /* Need length for seeking */
         if(ft->seekable){
-                ft->length = st_filelength(ft)/ST_SIZE_WORD;
+                ft->length = st_filelength(ft)/ST_SIZE_16BIT;
         } else {
                 ft->length = 0;
         }
@@ -78,7 +71,7 @@ int st_cdrstartread(ft_t ft)
  * Return number of samples read.
  */
 
-st_ssize_t st_cdrread(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_cdrread(ft_t ft, st_sample_t *buf, st_size_t len) 
 {
 
         return st_rawread(ft, buf, len);
@@ -88,24 +81,17 @@ st_ssize_t st_cdrread(ft_t ft, st_sample_t *buf, st_ssize_t len)
  * Do anything required when you stop reading samples.  
  * Don't close input file! 
  */
-int st_cdrstopread(ft_t ft) 
+static int st_cdrstopread(ft_t ft) 
 {
         /* Needed because of rawread() */
         return st_rawstopread(ft);
 }
 
-int st_cdrstartwrite(ft_t ft) 
+static int st_cdrstartwrite(ft_t ft) 
 {
         cdr_t cdr = (cdr_t) ft->priv;
         int rc;
 
-        /* CDR is in Big Endian format.  Swap whats written out on */
-        /* Little Endian Machines.                                 */
-        if (ST_IS_LITTLEENDIAN)
-        {
-            ft->swap = ft->swap ? 0 : 1;
-        }
-
         /* Needed because of rawwrite() */
         rc = st_rawstartwrite(ft);
         if (rc)
@@ -113,15 +99,15 @@ int st_cdrstartwrite(ft_t ft)
 
         cdr->samples = 0;
 
-        ft->info.rate = 44100L;
-        ft->info.size = ST_SIZE_WORD;
-        ft->info.encoding = ST_ENCODING_SIGN2;
-        ft->info.channels = 2;
+        ft->signal.rate = 44100;
+        ft->signal.size = ST_SIZE_16BIT;
+        ft->signal.encoding = ST_ENCODING_SIGN2;
+        ft->signal.channels = 2;
 
         return(ST_SUCCESS);
 }
 
-st_ssize_t st_cdrwrite(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_cdrwrite(ft_t ft, const st_sample_t *buf, st_size_t len) 
 {
         cdr_t cdr = (cdr_t) ft->priv;
 
@@ -135,7 +121,7 @@ st_ssize_t st_cdrwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
  * samples.  We write -32768 for each sample to pad it out.
  */
 
-int st_cdrstopwrite(ft_t ft) 
+static int st_cdrstopwrite(ft_t ft) 
 {
         cdr_t cdr = (cdr_t) ft->priv;
         int padsamps = SECTORSIZE - (cdr->samples % SECTORSIZE);
@@ -160,7 +146,8 @@ int st_cdrstopwrite(ft_t ft)
         return(ST_SUCCESS);
 }
 
-static char *cdrnames[] = {
+static const char *cdrnames[] = {
+  "cdda",
   "cdr",
   NULL
 };
@@ -168,7 +155,7 @@ static char *cdrnames[] = {
 static st_format_t st_cdr_format = {
   cdrnames,
   NULL,
-  ST_FILE_STEREO | ST_FILE_SEEK,
+  ST_FILE_SEEK | ST_FILE_BIG_END,
   st_cdrstartread,
   st_cdrread,
   st_cdrstopread,
