@@ -70,7 +70,7 @@ typedef struct {
  * Don't do initialization now.
  * The 'info' fields are not yet filled in.
  */
-int st_compand_getopts(eff_t effp, int n, char **argv) 
+static int st_compand_getopts(eff_t effp, int n, char **argv) 
 {
     compand_t l = (compand_t) effp->priv;
 
@@ -96,13 +96,9 @@ int st_compand_getopts(eff_t effp, int n, char **argv)
       }
 
       rates = 1 + commas/2;
-      if ((l->attackRate = (double *)malloc(sizeof(double) * rates)) == NULL ||
-          (l->decayRate  = (double *)malloc(sizeof(double) * rates)) == NULL ||
-          (l->volume     = (double *)malloc(sizeof(double) * rates)) == NULL)
-      {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
+      l->attackRate = (double *)xmalloc(sizeof(double) * rates);
+      l->decayRate = (double *)xmalloc(sizeof(double) * rates);
+      l->volume = (double *)xmalloc(sizeof(double) * rates);
       l->expectedChannels = rates;
       l->delay_buf = NULL;
 
@@ -124,18 +120,14 @@ int st_compand_getopts(eff_t effp, int n, char **argv)
       if (commas % 2 == 0) /* There must be an even number of
                               transfer parameters */
       {
-        st_fail("compander: Odd number of transfer function parameters\n"
+        st_fail("compander: Odd number of transfer function parameters"
              "Each input value in dB must have a corresponding output value");
         return (ST_EOF);
       }
 
       tfers = 3 + commas/2; /* 0, 0 at start; 1, 1 at end */
-      if ((l->transferIns  = (double *)malloc(sizeof(double) * tfers)) == NULL ||
-          (l->transferOuts = (double *)malloc(sizeof(double) * tfers)) == NULL)
-      {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
+      l->transferIns = (double *)xmalloc(sizeof(double) * tfers);
+      l->transferOuts = (double *)xmalloc(sizeof(double) * tfers);
       l->transferPoints = tfers;
       l->transferIns[0] = 0.0; l->transferOuts[0] = 0.0;
       l->transferIns[tfers-1] = 1.0; l->transferOuts[tfers-1] = 1.0;
@@ -191,31 +183,23 @@ int st_compand_getopts(eff_t effp, int n, char **argv)
  * Prepare processing.
  * Do all initializations.
  */
-int st_compand_start(eff_t effp)
+static int st_compand_start(eff_t effp)
 {
   compand_t l = (compand_t) effp->priv;
   int i;
 
-# ifdef DEBUG
-  {
-    fprintf(stderr, "Starting compand effect\n");
-    fprintf(stderr, "\nRate %ld, size %d, encoding %d, output gain %g.\n",
-           effp->outinfo.rate, effp->outinfo.size, effp->outinfo.encoding,
-           l->outgain);
-    fprintf(stderr, "%d input channel(s) expected: actually %d\n",
-           l->expectedChannels, effp->outinfo.channels);
-    fprintf(stderr, "\nAttack and decay rates\n"
-             "======================\n");
-    for (i = 0; i < l->expectedChannels; ++i)
-      fprintf(stderr, "Channel %d: attack = %-12g decay = %-12g\n",
-             i, l->attackRate[i], l->decayRate[i]);
-    fprintf(stderr, "\nTransfer function (linear values)\n"
-             "=================  =============\n");
-    for (i = 0; i < l->transferPoints; ++i)
-      fprintf(stderr, "%12g -> %-12g\n",
-             l->transferIns[i], l->transferOuts[i]);
-  }
-# endif
+  st_debug("Starting compand effect");
+  st_debug("Rate %ld, size %d, encoding %d, output gain %g.",
+         effp->outinfo.rate, effp->outinfo.size, effp->outinfo.encoding,
+         l->outgain);
+  st_debug("%d input channel(s) expected: actually %d",
+         l->expectedChannels, effp->outinfo.channels);
+  for (i = 0; i < l->expectedChannels; ++i)
+    st_debug("Channel %d: attack = %-12g decay = %-12g",
+           i, l->attackRate[i], l->decayRate[i]);
+  for (i = 0; i < l->transferPoints; ++i)
+    st_debug("Transfer fn (linear): %12g -> %-12g",
+           l->transferIns[i], l->transferOuts[i]);
   
   /* Convert attack and decay rates using number of samples */
 
@@ -234,11 +218,8 @@ int st_compand_start(eff_t effp)
 
   /* Allocate the delay buffer */
   l->delay_buf_size = l->delay * effp->outinfo.rate * effp->outinfo.channels;
-  if (l->delay_buf_size > 0
-   && (l->delay_buf = (st_sample_t *)malloc(sizeof(long) * l->delay_buf_size)) == NULL) {
-    st_fail("Out of memory");
-    return (ST_EOF);
-  }
+  if (l->delay_buf_size > 0)
+    l->delay_buf = (st_sample_t *)xmalloc(sizeof(long) * l->delay_buf_size);
   for (i = 0;  i < l->delay_buf_size;  i++)
     l->delay_buf[i] = 0;
   l->delay_buf_ptr = 0;
@@ -255,7 +236,7 @@ int st_compand_start(eff_t effp)
 
 static void doVolume(double *v, double samp, compand_t l, int chan)
 {
-  double s = samp/ST_SAMPLE_MAX;
+  double s = -samp / ST_SAMPLE_MIN;
   double delta = s - *v;
 
   if (delta > 0.0) /* increase volume according to attack rate */
@@ -268,14 +249,14 @@ static void doVolume(double *v, double samp, compand_t l, int chan)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_compand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_compand_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                     st_size_t *isamp, st_size_t *osamp)
 {
   compand_t l = (compand_t) effp->priv;
   int len =  (*isamp > *osamp) ? *osamp : *isamp;
   int filechans = effp->outinfo.channels;
   int idone,odone;
-  int64_t checkbuf; //if st_sample_t of type int32_t
+  int64_t checkbuf; /* if st_sample_t of type int32_t */
 
   for (idone = 0,odone = 0; idone < len; ibuf += filechans) {
     int chan;
@@ -300,27 +281,26 @@ int st_compand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
     /* Volume memory is updated: perform compand */
 
     for (chan = 0; chan < filechans; ++chan) {
-      double v = l->expectedChannels > 1 ?
-        l->volume[chan] : l->volume[0];
+      double v = l->volume[l->expectedChannels > 1 ? chan : 0];
       double outv;
-      int piece;
 
-      for (piece = 1 /* yes, 1 */;
-           piece < l->transferPoints;
-           ++piece)
-        if (v >= l->transferIns[piece - 1] &&
-            v < l->transferIns[piece])
-          break;
+      if (v == 0)
+        outv = 1;
+      else {
+        int piece;
 
-      outv = l->transferOuts[piece-1] +
-        (l->transferOuts[piece] - l->transferOuts[piece-1]) *
-        (v - l->transferIns[piece-1]) /
-        (l->transferIns[piece] - l->transferIns[piece-1]);
+        for (piece = 1; v > l->transferIns[piece]; ++piece);
+        outv = (l->transferOuts[piece-1] +
+          (l->transferOuts[piece] - l->transferOuts[piece-1]) *
+          (v - l->transferIns[piece-1]) /
+          (l->transferIns[piece] - l->transferIns[piece-1])) / v;
+      }
+      outv *= l->outgain;
 
       if (l->delay_buf_size <= 0)
       {
-        checkbuf = ibuf[chan]*(outv/v)*l->outgain;
-        ST_SAMPLE_CLIP(checkbuf);
+        checkbuf = ibuf[chan] * outv;
+        ST_SAMPLE_CLIP_COUNT(checkbuf, effp->clips);
         obuf[odone] = checkbuf;
 
         idone++;
@@ -330,9 +310,9 @@ int st_compand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
       {
         if (l->delay_buf_cnt >= l->delay_buf_size)
         {
-            l->delay_buf_full=1; //delay buffer is now definetly full
-            checkbuf = l->delay_buf[l->delay_buf_ptr]*(outv/v)*l->outgain;
-            ST_SAMPLE_CLIP(checkbuf);
+            l->delay_buf_full=1; /* delay buffer is now definetly full */
+            checkbuf = l->delay_buf[l->delay_buf_ptr] * outv;
+            ST_SAMPLE_CLIP_COUNT(checkbuf, effp->clips);
             obuf[odone] = checkbuf;
 
             odone++;
@@ -341,7 +321,7 @@ int st_compand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         else
         {
             l->delay_buf_cnt++;
-            idone++; //no "odone++" because we did not fill obuf[...]
+            idone++; /* no "odone++" because we did not fill obuf[...] */
         }
         l->delay_buf[l->delay_buf_ptr++] = ibuf[chan];
         l->delay_buf_ptr %= l->delay_buf_size;
@@ -356,7 +336,7 @@ int st_compand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 /*
  * Drain out compander delay lines.
  */
-int st_compand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_compand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
   compand_t l = (compand_t) effp->priv;
   st_size_t done;
@@ -384,18 +364,27 @@ int st_compand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 /*
  * Clean up compander effect.
  */
-int st_compand_stop(eff_t effp)
+static int st_compand_stop(eff_t effp)
 {
   compand_t l = (compand_t) effp->priv;
 
   free((char *) l->delay_buf);
+
+  l->delay_buf = NULL;
+
+  return (ST_SUCCESS);
+}
+
+static int delete(eff_t effp)
+{
+  compand_t l = (compand_t) effp->priv;
+
   free((char *) l->transferOuts);
   free((char *) l->transferIns);
   free((char *) l->volume);
   free((char *) l->decayRate);
   free((char *) l->attackRate);
 
-  l->delay_buf = NULL;
   l->transferOuts = NULL;
   l->transferIns = NULL;
   l->volume = NULL;
@@ -416,7 +405,8 @@ static st_effect_t st_compand_effect = {
    st_compand_start,
    st_compand_flow,
    st_compand_drain,
-   st_compand_stop
+   st_compand_stop,
+  delete
 };
 
 const st_effect_t *st_compand_effect_fn(void)
