@@ -40,21 +40,13 @@
 #include <string.h>
 #include <time.h>
 #include <stdio.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>     /* For SEEK_* defines if not found in stdio */
 #endif
 
 #include "cvsdfilt.h"
 
-/* ---------------------------------------------------------------------- */
-
-#ifndef HAVE_MEMMOVE
-#define memmove(dest,src,len) (bcopy((src),(dest),(len)))
-#endif
-
 /* ---------------------------------------------------------------------- */
 /*
  * private data structures
@@ -92,9 +84,10 @@ struct cvsdpriv {
         } bit;
         unsigned bytes_written;
         unsigned cvsd_rate;
-        char swapbits;
 };
 
+static int debug_count = 0;
+
 /* ---------------------------------------------------------------------- */
 
 static float float_conv(float *fp1, float *fp2,int n)
@@ -121,13 +114,11 @@ static void cvsdstartcommon(ft_t ft)
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
         
-        p->cvsd_rate = (ft->info.rate <= 24000) ? 16000 : 32000;
-        ft->info.rate = 8000;
-        ft->info.channels = 1;
-        ft->info.size = ST_SIZE_WORD; /* make output format default to words */
-        ft->info.encoding = ST_ENCODING_SIGN2;
-        p->swapbits = ft->swap;
-        ft->swap = 0;
+        p->cvsd_rate = (ft->signal.rate <= 24000) ? 16000 : 32000;
+        ft->signal.rate = 8000;
+        ft->signal.channels = 1;
+        ft->signal.size = ST_SIZE_16BIT; /* make output format default to words */
+        ft->signal.encoding = ST_ENCODING_SIGN2;
         /*
          * initialize the decoder
          */
@@ -146,20 +137,20 @@ static void cvsdstartcommon(ft_t ft)
          * initialize bit shift register
          */
         p->bit.shreg = p->bit.cnt = 0;
-        p->bit.mask = p->swapbits ? 0x80 : 1;
+        p->bit.mask = 1;
         /*
          * count the bytes written
          */
         p->bytes_written = 0;
         p->com.v_min = 1;
         p->com.v_max = -1;
-        st_report("cvsd: bit rate %dbit/s, bits from %s\n", p->cvsd_rate,
-               p->swapbits ? "msb to lsb" : "lsb to msb");
+        st_report("cvsd: bit rate %dbit/s, bits from %s", p->cvsd_rate,
+               ft->signal.reverse_bits ? "msb to lsb" : "lsb to msb");
 }
 
 /* ---------------------------------------------------------------------- */
 
-int st_cvsdstartread(ft_t ft) 
+static int st_cvsdstartread(ft_t ft) 
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
         float *fp1;
@@ -185,7 +176,7 @@ int st_cvsdstartread(ft_t ft)
 
 /* ---------------------------------------------------------------------- */
 
-int st_cvsdstartwrite(ft_t ft) 
+static int st_cvsdstartwrite(ft_t ft) 
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
         float *fp1;
@@ -207,7 +198,7 @@ int st_cvsdstartwrite(ft_t ft)
 
 /* ---------------------------------------------------------------------- */
 
-int st_cvsdstopwrite(ft_t ft)
+static int st_cvsdstopwrite(ft_t ft)
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 
@@ -215,7 +206,7 @@ int st_cvsdstopwrite(ft_t ft)
                 st_writeb(ft, p->bit.shreg);
                 p->bytes_written++;
         }
-        st_report("cvsd: min slope %f, max slope %f\n", 
+        st_debug("cvsd: min slope %f, max slope %f", 
                p->com.v_min, p->com.v_max);     
 
         return (ST_SUCCESS);
@@ -223,11 +214,11 @@ int st_cvsdstopwrite(ft_t ft)
 
 /* ---------------------------------------------------------------------- */
 
-int st_cvsdstopread(ft_t ft)
+static int st_cvsdstopread(ft_t ft)
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
 
-        st_report("cvsd: min value %f, max value %f\n", 
+        st_debug("cvsd: min value %f, max value %f", 
                p->com.v_min, p->com.v_max);
 
         return(ST_SUCCESS);
@@ -235,46 +226,18 @@ int st_cvsdstopread(ft_t ft)
 
 /* ---------------------------------------------------------------------- */
 
-#undef DEBUG
-
-#ifdef DEBUG
-static struct {
-        FILE *f1;
-        FILE *f2;
-        int cnt
-} dbg = { NULL, NULL, 0 };
-#endif
-
-st_ssize_t st_cvsdread(ft_t ft, st_sample_t *buf, st_ssize_t nsamp) 
+static st_size_t st_cvsdread(ft_t ft, st_sample_t *buf, st_size_t nsamp) 
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
-        int done = 0;
+        st_size_t done = 0;
         float oval;
         
-#ifdef DEBUG
-        if (!dbg.f1) {
-                if (!(dbg.f1 = fopen("dbg1", "w")))
-                {
-                        st_fail_errno(ft,errno,"debugging");
-                        return (0);
-                }
-                fprintf(dbg.f1, "\"input\"\n");
-        }
-        if (!dbg.f2) {
-                if (!(dbg.f2 = fopen("dbg2", "w")))
-                {
-                        st_fail_errno(ft,errno,"debugging");
-                        return (0);
-                }
-                fprintf(dbg.f2, "\"recon\"\n");
-        }
-#endif
         while (done < nsamp) {
                 if (!p->bit.cnt) {
                         if (st_readb(ft, &(p->bit.shreg)) == ST_EOF)
                                 return done;
                         p->bit.cnt = 8;
-                        p->bit.mask = p->swapbits ? 0x80 : 1;
+                        p->bit.mask = 1;
                 }
                 /*
                  * handle one bit
@@ -282,10 +245,7 @@ st_ssize_t st_cvsdread(ft_t ft, st_sample_t *buf, st_ssize_t nsamp)
                 p->bit.cnt--;
                 p->com.overload = ((p->com.overload << 1) | 
                                    (!!(p->bit.shreg & p->bit.mask))) & 7;
-                if (p->swapbits)
-                        p->bit.mask >>= 1;
-                else
-                        p->bit.mask <<= 1;
+                p->bit.mask <<= 1;
                 p->com.mla_int *= p->com.mla_tc0;
                 if ((p->com.overload == 0) || (p->com.overload == 7))
                         p->com.mla_int += p->com.mla_tc1;
@@ -304,13 +264,10 @@ st_ssize_t st_cvsdread(ft_t ft, st_sample_t *buf, st_ssize_t nsamp)
                                           (p->cvsd_rate < 24000) ? 
                                           dec_filter_16 : dec_filter_32, 
                                           DEC_FILTERLEN);
-#ifdef DEBUG
-                        fprintf(dbg.f1, "%f %f\n", (double)dbg.cnt, 
-                                (double)p->com.mla_int);
-                        fprintf(dbg.f2, "%f %f\n", (double)dbg.cnt, 
-                                (double)oval);
-                        dbg.cnt++;
-#endif          
+                        st_debug_more("input %d %f\n", debug_count, p->com.mla_int);
+                        st_debug_more("recon %d %f\n", debug_count, oval);
+                        debug_count++;
+
                         if (oval > p->com.v_max)
                                 p->com.v_max = oval;
                         if (oval < p->com.v_min)
@@ -325,30 +282,12 @@ st_ssize_t st_cvsdread(ft_t ft, st_sample_t *buf, st_ssize_t nsamp)
 
 /* ---------------------------------------------------------------------- */
 
-st_ssize_t st_cvsdwrite(ft_t ft, st_sample_t *buf, st_ssize_t nsamp) 
+static st_size_t st_cvsdwrite(ft_t ft, const st_sample_t *buf, st_size_t nsamp) 
 {
         struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
-        int done = 0;
+        st_size_t done = 0;
         float inval;
 
-#ifdef DEBUG
-        if (!dbg.f1) {
-                if (!(dbg.f1 = fopen("dbg1", "w")))
-                {
-                        st_fail_errno(ft,errno,"debugging");
-                        return (0);
-                }
-                fprintf(dbg.f1, "\"input\"\n");
-        }
-        if (!dbg.f2) {
-                if (!(dbg.f2 = fopen("dbg2", "w")))
-                {
-                        st_fail_errno(ft,errno,"debugging");
-                        return (0);
-                }
-                fprintf(dbg.f2, "\"recon\"\n");
-        }
-#endif
         for(;;) {
                 /*
                  * check if the next input is due
@@ -390,20 +329,13 @@ st_ssize_t st_cvsdwrite(ft_t ft, st_sample_t *buf, st_ssize_t nsamp)
                         st_writeb(ft, p->bit.shreg);
                         p->bytes_written++;
                         p->bit.shreg = p->bit.cnt = 0;
-                        p->bit.mask = p->swapbits ? 0x80 : 1;
-                } else {
-                        if (p->swapbits)
-                                p->bit.mask >>= 1;
-                        else
-                                p->bit.mask <<= 1;
-                }
+                        p->bit.mask = 1;
+                } else
+                        p->bit.mask <<= 1;
                 p->com.phase += p->com.phase_inc;
-#ifdef DEBUG
-                fprintf(dbg.f1, "%f %f\n", (double)dbg.cnt, (double)inval);
-                fprintf(dbg.f2, "%f %f\n", (double)dbg.cnt, 
-                        (double)p->c.enc.recon_int);
-                dbg.cnt++;
-#endif  
+                st_debug_more("input %d %f\n", debug_count, inval);
+                st_debug_more("recon %d %f\n", debug_count, p->c.enc.recon_int);
+                debug_count++;
         }
 }
 
@@ -430,37 +362,6 @@ struct dvms_header {
 
 #define DVMS_HEADER_LEN 120
 
-/* ---------------------------------------------------------------------- */
-/* FIXME: Move these to misc.c */
-static uint32_t get32(unsigned char **p)
-{
-        uint32_t val = (((*p)[3]) << 24) | (((*p)[2]) << 16) | 
-                (((*p)[1]) << 8) | (**p);
-        (*p) += 4;
-        return val;
-}
-
-static uint16_t get16(unsigned char **p)
-{
-        unsigned val = (((*p)[1]) << 8) | (**p);
-        (*p) += 2;
-        return val;
-}
-
-static void put32(unsigned char **p, uint32_t val)
-{
-        *(*p)++ = val & 0xff;
-        *(*p)++ = (val >> 8) & 0xff;
-        *(*p)++ = (val >> 16) & 0xff;
-        *(*p)++ = (val >> 24) & 0xff;
-}
-
-static void put16(unsigned char **p, int16_t val)
-{
-        *(*p)++ = val & 0xff;
-        *(*p)++ = (val >> 8) & 0xff;
-}
-
 /* ---------------------------------------------------------------------- */
 
 static int dvms_read_header(ft_t ft, struct dvms_header *hdr)
@@ -479,24 +380,24 @@ static int dvms_read_header(ft_t ft, struct dvms_header *hdr)
         pch = hdrbuf;
         memcpy(hdr->Filename, pch, sizeof(hdr->Filename));
         pch += sizeof(hdr->Filename);
-        hdr->Id = get16(&pch);
-        hdr->State = get16(&pch);
-        hdr->Unixtime = get32(&pch);
-        hdr->Usender = get16(&pch);
-        hdr->Ureceiver = get16(&pch);
-        hdr->Length = get32(&pch);
-        hdr->Srate = get16(&pch);
-        hdr->Days = get16(&pch);
-        hdr->Custom1 = get16(&pch);
-        hdr->Custom2 = get16(&pch);
+        hdr->Id = get16_le(&pch);
+        hdr->State = get16_le(&pch);
+        hdr->Unixtime = get32_le(&pch);
+        hdr->Usender = get16_le(&pch);
+        hdr->Ureceiver = get16_le(&pch);
+        hdr->Length = get32_le(&pch);
+        hdr->Srate = get16_le(&pch);
+        hdr->Days = get16_le(&pch);
+        hdr->Custom1 = get16_le(&pch);
+        hdr->Custom2 = get16_le(&pch);
         memcpy(hdr->Info, pch, sizeof(hdr->Info));
         pch += sizeof(hdr->Info);
         memcpy(hdr->extend, pch, sizeof(hdr->extend));
         pch += sizeof(hdr->extend);
-        hdr->Crc = get16(&pch);
+        hdr->Crc = get16_le(&pch);
         if (sum != hdr->Crc) 
         {
-                st_report("DVMS header checksum error, read %u, calculated %u\n",
+                st_report("DVMS header checksum error, read %u, calculated %u",
                      hdr->Crc, sum);
                 return (ST_EOF);
         }
@@ -518,16 +419,16 @@ static int dvms_write_header(ft_t ft, struct dvms_header *hdr)
 
         memcpy(pch, hdr->Filename, sizeof(hdr->Filename));
         pch += sizeof(hdr->Filename);
-        put16(&pch, hdr->Id);
-        put16(&pch, hdr->State);
-        put32(&pch, hdr->Unixtime);
-        put16(&pch, hdr->Usender);
-        put16(&pch, hdr->Ureceiver);
-        put32(&pch, hdr->Length);
-        put16(&pch, hdr->Srate);
-        put16(&pch, hdr->Days);
-        put16(&pch, hdr->Custom1);
-        put16(&pch, hdr->Custom2);
+        put16_le(&pch, hdr->Id);
+        put16_le(&pch, hdr->State);
+        put32_le(&pch, hdr->Unixtime);
+        put16_le(&pch, hdr->Usender);
+        put16_le(&pch, hdr->Ureceiver);
+        put32_le(&pch, hdr->Length);
+        put16_le(&pch, hdr->Srate);
+        put16_le(&pch, hdr->Days);
+        put16_le(&pch, hdr->Custom1);
+        put16_le(&pch, hdr->Custom2);
         memcpy(pch, hdr->Info, sizeof(hdr->Info));
         pch += sizeof(hdr->Info);
         memcpy(pch, hdr->extend, sizeof(hdr->extend));
@@ -535,7 +436,7 @@ static int dvms_write_header(ft_t ft, struct dvms_header *hdr)
         for(i = sizeof(hdrbuf), sum = 0; i > /*2*/3; i--) /* Deti bug */
                 sum += *pchs++;
         hdr->Crc = sum;
-        put16(&pch, hdr->Crc);
+        put16_le(&pch, hdr->Crc);
         if (st_seeki(ft, 0, SEEK_SET) < 0)
         {
                 st_report("seek failed\n: %s",strerror(errno));
@@ -543,7 +444,7 @@ static int dvms_write_header(ft_t ft, struct dvms_header *hdr)
         }
         if (st_writebuf(ft, hdrbuf, sizeof(hdrbuf), 1) != 1)
         {
-                st_report("%s\n",strerror(errno));
+                st_report("%s",strerror(errno));
                 return (ST_EOF);
         }
         return (ST_SUCCESS);
@@ -577,48 +478,45 @@ static void make_dvms_hdr(ft_t ft, struct dvms_header *hdr)
 
 /* ---------------------------------------------------------------------- */
 
-int st_dvmsstartread(ft_t ft) 
+static int st_dvmsstartread(ft_t ft) 
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
         struct dvms_header hdr;
         int rc;
 
         rc = dvms_read_header(ft, &hdr);
         if (rc){
-            st_fail_errno(ft,ST_EHDR,"unable to read DVMS header\n");
+            st_fail_errno(ft,ST_EHDR,"unable to read DVMS header");
             return rc;
         }
 
-        st_report("DVMS header of source file \"%s\":");
-        st_report("  filename  \"%.14s\"",ft->filename);
-        st_report("  id        0x%x", hdr.Filename);
-        st_report("  state     0x%x", hdr.Id, hdr.State);
-        st_report("  time      %s",ctime(&hdr.Unixtime)); /* ctime generates lf */
-        st_report("  usender   %u", hdr.Usender);
-        st_report("  ureceiver %u", hdr.Ureceiver);
-        st_report("  length    %u", hdr.Length);
-        st_report("  srate     %u", hdr.Srate);
-        st_report("  days      %u", hdr.Days);
-        st_report("  custom1   %u", hdr.Custom1);
-        st_report("  custom2   %u", hdr.Custom2);
-        st_report("  info      \"%.16s\"\n", hdr.Info);
-        ft->info.rate = (hdr.Srate < 240) ? 16000 : 32000;
-        st_report("DVMS rate %dbit/s using %dbit/s deviation %d%%\n", 
-               hdr.Srate*100, ft->info.rate, 
-               ((ft->info.rate - hdr.Srate*100) * 100) / ft->info.rate);
+        st_debug("DVMS header of source file \"%s\":");
+        st_debug("  filename  \"%.14s\"",ft->filename);
+        st_debug("  id        0x%x", hdr.Filename);
+        st_debug("  state     0x%x", hdr.Id, hdr.State);
+        st_debug("  time      %s",ctime(&hdr.Unixtime)); /* ctime generates lf */
+        st_debug("  usender   %u", hdr.Usender);
+        st_debug("  ureceiver %u", hdr.Ureceiver);
+        st_debug("  length    %u", hdr.Length);
+        st_debug("  srate     %u", hdr.Srate);
+        st_debug("  days      %u", hdr.Days);
+        st_debug("  custom1   %u", hdr.Custom1);
+        st_debug("  custom2   %u", hdr.Custom2);
+        st_debug("  info      \"%.16s\"", hdr.Info);
+        ft->signal.rate = (hdr.Srate < 240) ? 16000 : 32000;
+        st_debug("DVMS rate %dbit/s using %dbit/s deviation %d%%", 
+               hdr.Srate*100, ft->signal.rate, 
+               ((ft->signal.rate - hdr.Srate*100) * 100) / ft->signal.rate);
         rc = st_cvsdstartread(ft);
         if (rc)
             return rc;
 
-        p->swapbits = 0;
         return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-int st_dvmsstartwrite(ft_t ft) 
+static int st_dvmsstartwrite(ft_t ft) 
 {
-        struct cvsdpriv *p = (struct cvsdpriv *) ft->priv;
         struct dvms_header hdr;
         int rc;
         
@@ -629,20 +527,19 @@ int st_dvmsstartwrite(ft_t ft)
         make_dvms_hdr(ft, &hdr);
         rc = dvms_write_header(ft, &hdr);
         if (rc){
-                st_fail_errno(ft,rc,"cannot write DVMS header\n");
+                st_fail_errno(ft,rc,"cannot write DVMS header");
             return rc;
         }
 
         if (!ft->seekable)
                st_warn("Length in output .DVMS header will wrong since can't seek to fix it");
 
-        p->swapbits = 0;
         return(ST_SUCCESS);
 }
 
 /* ---------------------------------------------------------------------- */
 
-int st_dvmsstopwrite(ft_t ft)
+static int st_dvmsstopwrite(ft_t ft)
 {
         struct dvms_header hdr;
         int rc;
@@ -653,7 +550,7 @@ int st_dvmsstopwrite(ft_t ft)
             st_warn("File not seekable");
             return (ST_EOF);
         }
-        if (st_seeki(ft, 0L, 0) != 0)
+        if (st_seeki(ft, 0, 0) != 0)
         {
                 st_fail_errno(ft,errno,"Can't rewind output file to rewrite DVMS header.");
                 return(ST_EOF);
@@ -661,7 +558,7 @@ int st_dvmsstopwrite(ft_t ft)
         make_dvms_hdr(ft, &hdr);
         rc = dvms_write_header(ft, &hdr);
         if(rc){
-            st_fail_errno(ft,rc,"cannot write DVMS header\n");
+            st_fail_errno(ft,rc,"cannot write DVMS header");
             return rc;
         }       
         return rc;
@@ -670,7 +567,7 @@ int st_dvmsstopwrite(ft_t ft)
 /* ---------------------------------------------------------------------- */
 
 /* Cont. Variable Slope Delta */
-static char *cvsdnames[] = {
+static const char *cvsdnames[] = {
   "cvs",
   "cvsd",
   NULL
@@ -694,7 +591,7 @@ const st_format_t *st_cvsd_format_fn(void)
     return &st_cvsd_format;
 }
 /* Cont. Variable Solot Delta */
-static char *dvmsnames[] = {
+static const char *dvmsnames[] = {
   "vms",
   "dvms",
   NULL
