@@ -52,7 +52,7 @@
 
 static st_effect_t st_echos_effect;
 
-#define DELAY_BUFSIZ ( 50L * ST_MAXRATE )
+#define DELAY_BUFSIZ ( 50 * ST_MAXRATE )
 #define MAX_ECHOS 7     /* 24 bit x ( 1 + MAX_ECHOS ) = */
                         /* 24 bit x 8 = 32 bit !!!      */
 
@@ -72,7 +72,7 @@ typedef struct echosstuff {
 /*
  * Process options
  */
-int st_echos_getopts(eff_t effp, int n, char **argv) 
+static int st_echos_getopts(eff_t effp, int n, char **argv) 
 {
         echos_t echos = (echos_t) effp->priv;
         int i;
@@ -107,7 +107,7 @@ int st_echos_getopts(eff_t effp, int n, char **argv)
 /*
  * Prepare for processing.
  */
-int st_echos_start(eff_t effp)
+static int st_echos_start(eff_t effp)
 {
         echos_t echos = (echos_t) effp->priv;
         int i;
@@ -116,52 +116,47 @@ int st_echos_start(eff_t effp)
 
         if ( echos->in_gain < 0.0 )
         {
-                st_fail("echos: gain-in must be positive!\n");
+                st_fail("echos: gain-in must be positive!");
                 return (ST_EOF);
         }
         if ( echos->in_gain > 1.0 )
         {
-                st_fail("echos: gain-in must be less than 1.0!\n");
+                st_fail("echos: gain-in must be less than 1.0!");
                 return (ST_EOF);
         }
         if ( echos->out_gain < 0.0 )
         {
-                st_fail("echos: gain-in must be positive!\n");
+                st_fail("echos: gain-in must be positive!");
                 return (ST_EOF);
         }
         for ( i = 0; i < echos->num_delays; i++ ) {
                 echos->samples[i] = echos->delay[i] * effp->ininfo.rate / 1000.0;
                 if ( echos->samples[i] < 1 )
                 {
-                    st_fail("echos: delay must be positive!\n");
+                    st_fail("echos: delay must be positive!");
                     return (ST_EOF);
                 }
-                if ( echos->samples[i] > DELAY_BUFSIZ )
+                if ( echos->samples[i] > (st_ssize_t)DELAY_BUFSIZ )
                 {
-                        st_fail("echos: delay must be less than %g seconds!\n",
+                        st_fail("echos: delay must be less than %g seconds!",
                                 DELAY_BUFSIZ / (float) effp->ininfo.rate );
                         return (ST_EOF);
                 }
                 if ( echos->decay[i] < 0.0 )
                 {
-                    st_fail("echos: decay must be positive!\n" );
+                    st_fail("echos: decay must be positive!" );
                     return (ST_EOF);
                 }
                 if ( echos->decay[i] > 1.0 )
                 {
-                    st_fail("echos: decay must be less than 1.0!\n" );
+                    st_fail("echos: decay must be less than 1.0!" );
                     return (ST_EOF);
                 }
                 echos->counter[i] = 0;
                 echos->pointer[i] = echos->sumsamples;
                 echos->sumsamples += echos->samples[i];
         }
-        if (! (echos->delay_buf = (double *) malloc(sizeof (double) * echos->sumsamples)))
-        {
-                st_fail("echos: Cannot malloc %d bytes!\n", 
-                        sizeof(double) * echos->sumsamples);
-                return(ST_EOF);
-        }
+        echos->delay_buf = (double *) xmalloc(sizeof (double) * echos->sumsamples);
         for ( j = 0; j < echos->sumsamples; ++j )
                 echos->delay_buf[j] = 0.0;
         /* Be nice and check the hint with warning, if... */
@@ -177,7 +172,7 @@ int st_echos_start(eff_t effp)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_echos_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_echos_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                 st_size_t *isamp, st_size_t *osamp)
 {
         echos_t echos = (echos_t) effp->priv;
@@ -198,7 +193,7 @@ int st_echos_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
                 }
                 /* Adjust the output volume and size to 24 bit */
                 d_out = d_out * echos->out_gain;
-                out = st_clip24((st_sample_t) d_out);
+                out = ST_24BIT_CLIP_COUNT((st_sample_t) d_out, effp->clips);
                 *obuf++ = out * 256;
                 /* Mix decay of delays and input */
                 for ( j = 0; j < echos->num_delays; j++ ) {
@@ -220,7 +215,7 @@ int st_echos_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 /*
  * Drain out reverb lines. 
  */
-int st_echos_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_echos_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
         echos_t echos = (echos_t) effp->priv;
         double d_in, d_out;
@@ -238,7 +233,7 @@ int st_echos_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
                 }
                 /* Adjust the output volume and size to 24 bit */
                 d_out = d_out * echos->out_gain;
-                out = st_clip24((st_sample_t) d_out);
+                out = ST_24BIT_CLIP_COUNT((st_sample_t) d_out, effp->clips);
                 *obuf++ = out * 256;
                 /* Mix decay of delays and input */
                 for ( j = 0; j < echos->num_delays; j++ ) {
@@ -266,7 +261,7 @@ int st_echos_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 /*
  * Clean up echos effect.
  */
-int st_echos_stop(eff_t effp)
+static int st_echos_stop(eff_t effp)
 {
         echos_t echos = (echos_t) effp->priv;
 
@@ -283,7 +278,8 @@ static st_effect_t st_echos_effect = {
   st_echos_start,
   st_echos_flow,
   st_echos_drain,
-  st_echos_stop
+  st_echos_stop,
+  st_effect_nothing
 };
 
 const st_effect_t *st_echos_effect_fn(void)
