@@ -1,7 +1,6 @@
-/* This code is based in skel.c
+/*
+ * Ari Moisio <armoi@sci.fi> Aug 29 2000, based on skeleton effect
  * Written by Chris Bagwell (cbagwell@sprynet.com) - March 16, 1999
- * Non-skel parts written by
- * Ari Moisio <armoi@sci.fi> Aug 29 2000.
  *
  * Copyright 1999 Chris Bagwell And Sundry Contributors
  * This source code is freely redistributable and may be used for
@@ -29,7 +28,7 @@ static st_effect_t st_fade_effect;
 /* Private data for fade file */
 typedef struct fadestuff
 { /* These are measured as samples */
-    st_size_t in_start,  in_stop, out_start, out_stop, samplesdone;
+    st_size_t in_start, in_stop, out_start, out_stop, samplesdone;
     char *in_stop_str, *out_start_str, *out_stop_str;
     char in_fadetype, out_fadetype;
     char do_out;
@@ -46,7 +45,7 @@ static double fade_gain(st_size_t index, st_size_t range, char fadetype);
  * The 'info' fields are not yet filled in.
  */
 
-int st_fade_getopts(eff_t effp, int n, char **argv)
+static int st_fade_getopts(eff_t effp, int n, char **argv)
 {
 
     fade_t fade = (fade_t) effp->priv;
@@ -78,16 +77,10 @@ int st_fade_getopts(eff_t effp, int n, char **argv)
         fade->out_fadetype = 'l';
     }
 
-    fade->in_stop_str = (char *)malloc(strlen(argv[0])+1);
-    if (!fade->in_stop_str)
-    {
-        st_fail("Could not allocate memory");
-        return (ST_EOF);
-    }
+    fade->in_stop_str = (char *)xmalloc(strlen(argv[0])+1);
     strcpy(fade->in_stop_str,argv[0]);
     /* Do a dummy parse to see if it will fail */
-    if (st_parsesamples(0, fade->in_stop_str, &fade->in_stop, 't') !=
-            ST_SUCCESS)
+    if (st_parsesamples(0, fade->in_stop_str, &fade->in_stop, 't') == NULL)
     {
         st_fail(st_fade_effect.usage);
         return(ST_EOF);
@@ -100,39 +93,27 @@ int st_fade_getopts(eff_t effp, int n, char **argv)
         /* See if there is fade-in/fade-out times/curves specified. */
         if(t_argno == 1)
         {
-            fade->out_stop_str = (char *)malloc(strlen(argv[t_argno])+1);
-            if (!fade->out_stop_str)
-            {
-                st_fail("Could not allocate memory");
-                return (ST_EOF);
-            }
-             strcpy(fade->out_stop_str,argv[t_argno]);
+            fade->out_stop_str = (char *)xmalloc(strlen(argv[t_argno])+1);
+            strcpy(fade->out_stop_str,argv[t_argno]);
 
-             /* Do a dummy parse to see if it will fail */
-             if (st_parsesamples(0, fade->out_stop_str, 
-                         &fade->out_stop, 't') != ST_SUCCESS)
-             {
-                 st_fail(st_fade_effect.usage);
-                 return(ST_EOF);
-             }
+            /* Do a dummy parse to see if it will fail */
+            if (st_parsesamples(0, fade->out_stop_str, 
+                                &fade->out_stop, 't') == NULL) {
+              st_fail(st_fade_effect.usage);
+              return(ST_EOF);
+            }
         }
         else
         {
-            fade->out_start_str = (char *)malloc(strlen(argv[t_argno])+1);
-            if (!fade->out_start_str)
-            {
-                st_fail("Could not allocate memory");
-                return (ST_EOF);
-            }
-             strcpy(fade->out_start_str,argv[t_argno]);
+            fade->out_start_str = (char *)xmalloc(strlen(argv[t_argno])+1);
+            strcpy(fade->out_start_str,argv[t_argno]);
 
-             /* Do a dummy parse to see if it will fail */
-             if (st_parsesamples(0, fade->out_start_str, 
-                         &fade->out_start, 't') != ST_SUCCESS)
-             {
-                 st_fail(st_fade_effect.usage);
-                 return(ST_EOF);
-             }
+            /* Do a dummy parse to see if it will fail */
+            if (st_parsesamples(0, fade->out_start_str, 
+                                &fade->out_start, 't') == NULL) {
+              st_fail(st_fade_effect.usage);
+              return(ST_EOF);
+            }
         }
     } /* End for(t_argno) */
 
@@ -143,14 +124,14 @@ int st_fade_getopts(eff_t effp, int n, char **argv)
  * Prepare processing.
  * Do all initializations.
  */
-int st_fade_start(eff_t effp)
+static int st_fade_start(eff_t effp)
 {
     fade_t fade = (fade_t) effp->priv;
 
     /* converting time values to samples */
     fade->in_start = 0;
     if (st_parsesamples(effp->ininfo.rate, fade->in_stop_str,
-                        &fade->in_stop, 't') != ST_SUCCESS)
+                        &fade->in_stop, 't') == NULL)
     {
         st_fail(st_fade_effect.usage);
         return(ST_EOF);
@@ -162,7 +143,7 @@ int st_fade_start(eff_t effp)
     {
         fade->do_out = 1;
         if (st_parsesamples(effp->ininfo.rate, fade->out_stop_str,
-                            &fade->out_stop, 't') != ST_SUCCESS)
+                            &fade->out_stop, 't') == NULL)
         {
             st_fail(st_fade_effect.usage);
             return(ST_EOF);
@@ -172,7 +153,7 @@ int st_fade_start(eff_t effp)
         if (fade->out_start_str)
         {
             if (st_parsesamples(effp->ininfo.rate, fade->out_start_str,
-                        &fade->out_start, 't') != ST_SUCCESS)
+                        &fade->out_start, 't') == NULL)
             {
                 st_fail(st_fade_effect.usage);
                 return(ST_EOF);
@@ -201,27 +182,29 @@ int st_fade_start(eff_t effp)
         return(ST_EOF);
     } /* endif fade time sanity */
 
-    /* If lead-in is required it is handled as negative sample numbers */
-    fade->samplesdone = (fade->in_start < 0 ? fade->in_start :0);
-
+    fade->samplesdone = fade->in_start;
     fade->endpadwarned = 0;
 
-    /* fprintf(stderr, "fade: in_start = %d in_stop = %d out_start = %d out_stop = %d\n", fade->in_start, fade->in_stop, fade->out_start, fade->out_stop); */
+    st_debug("fade: in_start = %d in_stop = %d out_start = %d out_stop = %d", fade->in_start, fade->in_stop, fade->out_start, fade->out_stop);
 
-    return(ST_SUCCESS);
+    if (fade->in_start == fade->in_stop && fade->out_start == fade->out_stop)
+      return ST_EFF_NULL;
+
+    return ST_SUCCESS;
 }
 
 /*
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_fade_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_fade_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                  st_size_t *isamp, st_size_t *osamp)
 {
     fade_t fade = (fade_t) effp->priv;
     /* len is total samples, chcnt counts channels */
-    int len = 0, chcnt = 0, t_output = 1, more_output = 1;
+    int len = 0, t_output = 1, more_output = 1;
     st_sample_t t_ibuf;
+    st_size_t chcnt = 0;
 
     len = ((*isamp > *osamp) ? *osamp : *isamp);
 
@@ -230,7 +213,7 @@ int st_fade_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 
     for(; len && more_output; len--)
     {
-        t_ibuf = (fade->samplesdone < 0 ? 0 : *ibuf);
+        t_ibuf = *ibuf;
 
         if ((fade->samplesdone >= fade->in_start) &&
             (!fade->do_out || fade->samplesdone < fade->out_stop))
@@ -265,14 +248,8 @@ int st_fade_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
             t_output = 0;
         } /* endif something to output */
 
-        /* samplesdone < 0 means we are inventing samples right now
-         * and so not consuming (happens when in_start < 0).
-         */
-        if (fade->samplesdone >= 0 )
-        { /* Something to input  */
-            *isamp += 1;
-            ibuf++;
-        } /* endif something accepted as input */
+        *isamp += 1;
+        ibuf++;
 
         if (t_output)
         { /* Output generated, update pointers and counters */
@@ -301,10 +278,11 @@ int st_fade_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 /*
  * Drain out remaining samples if the effect generates any.
  */
-int st_fade_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_fade_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
     fade_t fade = (fade_t) effp->priv;
-    int len, t_chan = 0;
+    int len;
+    st_size_t t_chan = 0;
 
     len = *osamp;
     *osamp = 0;
@@ -341,16 +319,13 @@ int st_fade_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
  * Do anything required when you stop reading samples.
  *      (free allocated memory, etc.)
  */
-int st_fade_stop(eff_t effp)
+static int delete(eff_t effp)
 {
     fade_t fade = (fade_t) effp->priv;
 
-    if (fade->in_stop_str)
-        free(fade->in_stop_str);
-    if (fade->out_start_str)
-        free(fade->out_start_str);
-    if (fade->out_stop_str)
-        free(fade->out_stop_str);
+    free(fade->in_stop_str);
+    free(fade->out_start_str);
+    free(fade->out_stop_str);
     return (ST_SUCCESS);
 }
 
@@ -361,41 +336,37 @@ static double fade_gain(st_size_t index, st_size_t range, char type)
 {
     double retval = 0.0, findex = 0.0;
 
-    findex = 1.0 * index / range;
+    /* TODO: does it really have to be contrained to [0.0, 1.0]? */
+    findex = max(0.0, min(1.0, 1.0 * index / range));
 
-    /* todo: are these really needed */
-    findex = (findex < 0 ? 0.0 : findex);
-    findex = (findex > 1.0 ? 1.0 : findex);
+    switch (type) {
+    case FADE_TRI :             /* triangle */
+      retval = findex;
+      break;
 
-    switch (type)
-    {
-        case FADE_TRI : /* triangle  */
-            retval = findex;
-            break;
+    case FADE_QUARTER :         /* quarter of sinewave */
+      retval = sin(findex * M_PI / 2);
+      break;
 
-        case FADE_QUARTER : /*  quarter of sinewave */
-            retval = sin(findex * M_PI / 2);
-            break;
+    case FADE_HALF :          /* half of sinewave... eh cosine wave */
+      retval = (1 - cos(findex * M_PI )) / 2 ;
+      break;
 
-        case FADE_HALF : /* half of sinewave... eh cosine wave */
-            retval = (1 - cos(findex * M_PI )) / 2 ;
-            break;
+    case FADE_LOG :             /* logarithmic */
+      /* 5 means 100 db attenuation. */
+      /* TODO: should this be adopted with bit depth */
+      retval =  pow(0.1, (1 - findex) * 5);
+      break;
 
-        case FADE_LOG : /* logaritmic */
-            /* 5 means 100 db attenuation. */
-            /* todo: should this be adopted with bit depth        */
-            retval =  pow(0.1, (1 - findex) * 5);
-            break;
+    case FADE_PAR :             /* inverted parabola */
+      retval = (1 - (1 - findex)  * (1 - findex));
+      break;
 
-        case FADE_PAR : /* inverted parabola */
-            retval = (1 - (1 - findex)  * (1 - findex));
-            break;
-
-            /* todo: more fade curves? */
-        default : /* Error indicating wrong fade curve */
-            retval = -1.0;
-            break;
-    } /* endswitch */
+    /* TODO: more fade curves? */
+    default :                  /* Error indicating wrong fade curve */
+      retval = -1.0;
+      break;
+    }
 
     return retval;
 }
@@ -410,7 +381,8 @@ static st_effect_t st_fade_effect = {
   st_fade_start,
   st_fade_flow,
   st_fade_drain,
-  st_fade_stop
+  st_effect_nothing,
+  delete
 };
 
 const st_effect_t *st_fade_effect_fn(void)
