@@ -91,7 +91,7 @@ void g72x_init_state(struct g72x_state *state_ptr)
 {
         int             cnta;
 
-        state_ptr->yl = 34816L;
+        state_ptr->yl = 34816;
         state_ptr->yu = 544;
         state_ptr->dms = 0;
         state_ptr->dml = 0;
@@ -171,12 +171,6 @@ int step_size(struct g72x_state *state_ptr)
  * as a subtraction.
  */
 int quantize(int d, int y, short *table, int size)
-#if 0
-        int             d;      /* Raw difference signal sample */
-        int             y;      /* Step size multiplier */
-        short           *table; /* quantization table */
-        int             size;   /* table size of short integers */
-#endif
 {
         short           dqm;    /* Magnitude of 'd' */
         short           exp;    /* Integer part of base 2 log of 'd' */
@@ -223,11 +217,6 @@ int quantize(int d, int y, short *table, int size)
  * Multiplication is performed in log base 2 domain as addition.
  */
 int reconstruct(int sign, int dqln, int y)
-#if 0
-        int             sign;   /* 0 for non-negative value */
-        int             dqln;   /* G.72x codeword */
-        int             y;      /* Step size multiplier */
-#endif
 {
         short           dql;    /* Log of 'dq' magnitude */
         short           dex;    /* Integer part of log */
@@ -254,16 +243,6 @@ int reconstruct(int sign, int dqln, int y)
  */
 void update(int code_size, int y, int wi, int fi, int dq, int sr, 
             int dqsez, struct g72x_state *state_ptr)
-#if 0
-        int             code_size;      /* distinguish 723_40 with others */
-        int             y;              /* quantizer step size */
-        int             wi;             /* scale factor multiplier */
-        int             fi;             /* for long/short term energies */
-        int             dq;             /* quantized prediction difference */
-        int             sr;             /* reconstructed signal */
-        int             dqsez;          /* difference from 2-pole predictor */
-        struct g72x_state *state_ptr;   /* coder state pointer */
-#endif
 {
         int             cnt;
         short           mag, exp;       /* Adaptive predictor, FLOAT A */
@@ -394,7 +373,7 @@ void update(int code_size, int y, int wi, int fi, int dq, int sr,
                 state_ptr->dq[cnt] = state_ptr->dq[cnt-1];
         /* FLOAT A : convert dq[0] to 4-bit exp, 6-bit mantissa f.p. */
         if (mag == 0) {
-                state_ptr->dq[0] = (dq >= 0) ? 0x20 : 0xFC20;
+                state_ptr->dq[0] = (dq >= 0) ? 0x20 : (short)0xFC20;
         } else {
                 exp = quan(mag, power2, 15);
                 state_ptr->dq[0] = (dq >= 0) ?
@@ -409,12 +388,12 @@ void update(int code_size, int y, int wi, int fi, int dq, int sr,
         } else if (sr > 0) {
                 exp = quan(sr, power2, 15);
                 state_ptr->sr[0] = (exp << 6) + ((sr << 6) >> exp);
-        } else if (sr > -32768L) {
+        } else if (sr > -32768) {
                 mag = -sr;
                 exp = quan(mag, power2, 15);
                 state_ptr->sr[0] =  (exp << 6) + ((mag << 6) >> exp) - 0x400;
         } else
-                state_ptr->sr[0] = 0xFC20;
+                state_ptr->sr[0] = (short)(0xFC20);
 
         /* DELAY A */
         state_ptr->pk[1] = state_ptr->pk[0];
@@ -466,14 +445,6 @@ void update(int code_size, int y, int wi, int fi, int dq, int sr,
  *      adjusted A-law or u-law compressed sample.
  */
 int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short *qtab)
-#if 0
-        int             sr;     /* decoder output linear PCM sample */
-        int             se;     /* predictor estimate sample */
-        int             y;      /* quantizer step size */
-        int             i;      /* decoder input code */
-        int             sign;
-        short           *qtab;
-#endif
 {
         unsigned char   sp;     /* A-law compressed 8-bit code */
         short           dx;     /* prediction error */
@@ -482,7 +453,7 @@ int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short *qtab)
         int             im;     /* biased magnitude of i */
         int             imx;    /* biased magnitude of id */
 
-        if (sr <= -32768L)
+        if (sr <= -32768)
                 sr = -1;
         sp = st_13linear2alaw(((sr >> 1) << 3));/* short to A-law compression */
         dx = (st_alaw2linear16(sp) >> 2) - se;  /* 16-bit prediction error */
@@ -516,14 +487,6 @@ int tandem_adjust_alaw(int sr, int se, int y, int i, int sign, short *qtab)
 }
 
 int tandem_adjust_ulaw(int sr, int se, int y, int i, int sign, short *qtab)
-#if 0
-        int             sr;     /* decoder output linear PCM sample */
-        int             se;     /* predictor estimate sample */
-        int             y;      /* quantizer step size */
-        int             i;      /* decoder input code */
-        int             sign;
-        short           *qtab;
-#endif
 {
         unsigned char   sp;     /* u-law compressed 8-bit code */
         short           dx;     /* prediction error */
@@ -532,7 +495,7 @@ int tandem_adjust_ulaw(int sr, int se, int y, int i, int sign, short *qtab)
         int             im;     /* biased magnitude of i */
         int             imx;    /* biased magnitude of id */
 
-        if (sr <= -32768L)
+        if (sr <= -32768)
                 sr = 0;
         sp = st_14linear2ulaw((sr << 2));/* short to u-law compression */
         dx = (st_ulaw2linear16(sp) >> 2) - se;  /* 16-bit prediction error */
