@@ -28,17 +28,13 @@
 
 #include "st_i.h"
 
-#if defined(ENABLE_GSM)
-
-#if defined(EXTERNAL_GSM)
+#ifdef EXTERNAL_GSM
 #include <gsm/gsm.h>
 #else
 #include "libgsm/gsm.h"
 #endif
 
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 
 #define MAXCHANS 16
 
@@ -62,18 +58,18 @@ static int gsmstart_rw(ft_t ft, int w)
         struct gsmpriv *p = (struct gsmpriv *) ft->priv;
         int ch;
         
-        ft->info.encoding = ST_ENCODING_GSM;
-        ft->info.size = ST_SIZE_BYTE;
-        if (!ft->info.rate)
-                ft->info.rate = 8000;
+        ft->signal.encoding = ST_ENCODING_GSM;
+        ft->signal.size = ST_SIZE_BYTE;
+        if (!ft->signal.rate)
+                ft->signal.rate = 8000;
 
-        if (ft->info.channels == -1)
-            ft->info.channels = 1;
+        if (ft->signal.channels == 0)
+            ft->signal.channels = 1;
 
-        p->channels = ft->info.channels;
+        p->channels = ft->signal.channels;
         if (p->channels > MAXCHANS || p->channels <= 0)
         {
-                st_fail_errno(ft,ST_EFMT,"gsm: channels(%d) must be in 1-16", ft->info.channels);
+                st_fail_errno(ft,ST_EFMT,"gsm: channels(%d) must be in 1-16", ft->signal.channels);
                 return(ST_EOF);
         }
 
@@ -85,19 +81,19 @@ static int gsmstart_rw(ft_t ft, int w)
                         return (ST_EOF);
                 }
         }
-        p->frames = (gsm_byte*) malloc(p->channels*FRAMESIZE);
-        p->samples = (gsm_signal*) malloc(BLOCKSIZE * (p->channels+1) * sizeof(gsm_signal));
+        p->frames = (gsm_byte*) xmalloc(p->channels*FRAMESIZE);
+        p->samples = (gsm_signal*) xmalloc(BLOCKSIZE * (p->channels+1) * sizeof(gsm_signal));
         p->sampleTop = p->samples + BLOCKSIZE*p->channels;
         p->samplePtr = (w)? p->samples : p->sampleTop;
         return (ST_SUCCESS);
 }
 
-int st_gsmstartread(ft_t ft) 
+static int st_gsmstartread(ft_t ft) 
 {
         return gsmstart_rw(ft,0);
 }
 
-int st_gsmstartwrite(ft_t ft)
+static int st_gsmstartwrite(ft_t ft)
 {
         return gsmstart_rw(ft,1);
 }
@@ -109,9 +105,9 @@ int st_gsmstartwrite(ft_t ft)
  * Return number of samples read.
  */
 
-st_ssize_t st_gsmread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_gsmread(ft_t ft, st_sample_t *buf, st_size_t samp)
 {
-        int done = 0;
+        size_t done = 0;
         int r, ch, chans;
         gsm_signal *gbuff;
         struct gsmpriv *p = (struct gsmpriv *) ft->priv;
@@ -122,7 +118,7 @@ st_ssize_t st_gsmread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
         {
                 while (p->samplePtr < p->sampleTop && done < samp)
                         buf[done++] = 
-                            ST_SIGNED_WORD_TO_SAMPLE(*(p->samplePtr)++);
+                            ST_SIGNED_WORD_TO_SAMPLE(*(p->samplePtr)++,);
 
                 if (done>=samp) break;
 
@@ -187,16 +183,16 @@ static int gsmflush(ft_t ft)
         return (ST_SUCCESS);
 }
 
-st_ssize_t st_gsmwrite(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_gsmwrite(ft_t ft, const st_sample_t *buf, st_size_t samp)
 {
-        int done = 0;
+        size_t done = 0;
         struct gsmpriv *p = (struct gsmpriv *) ft->priv;
 
         while (done < samp)
         {
                 while ((p->samplePtr < p->sampleTop) && (done < samp))
                         *(p->samplePtr)++ = 
-                            ST_SAMPLE_TO_SIGNED_WORD(buf[done++]);
+                            ST_SAMPLE_TO_SIGNED_WORD(buf[done++], ft->clips);
 
                 if (p->samplePtr == p->sampleTop)
                 {
@@ -210,7 +206,7 @@ st_ssize_t st_gsmwrite(ft_t ft, st_sample_t *buf, st_ssize_t samp)
         return done;
 }
 
-int st_gsmstopread(ft_t ft)
+static int st_gsmstopread(ft_t ft)
 {
         struct gsmpriv *p = (struct gsmpriv *) ft->priv;
         int ch;
@@ -223,7 +219,7 @@ int st_gsmstopread(ft_t ft)
         return (ST_SUCCESS);
 }
 
-int st_gsmstopwrite(ft_t ft)
+static int st_gsmstopwrite(ft_t ft)
 {
         int rc;
         struct gsmpriv *p = (struct gsmpriv *) ft->priv;
@@ -239,7 +235,7 @@ int st_gsmstopwrite(ft_t ft)
 }
 
 /* GSM 06.10 */
-static char *gsmnames[] = {
+static const char *gsmnames[] = {
   "gsm",
   NULL
 };
@@ -261,4 +257,3 @@ const st_format_t *st_gsm_format_fn(void)
 {
     return &st_gsm_format;
 }
-#endif /* ENABLE_GSM */
