@@ -15,18 +15,17 @@
  * April 28, 1998 - Chris Bagwell (cbagwell@sprynet.com)
  *
  *  Rearranged some functions so that they are declared before they are
- *  used.  Clears up some compiler warnings.  Because this functions passed
- *  foats, it helped out some dump compilers pass stuff on the stack
- *  correctly.
+ *  used, clearing up some compiler warnings.  Because these functions
+ *  passed floats, it helped some dumb compilers pass stuff on the
+ *  stack correctly.
  *
  */
 
 #include "st_i.h"
+#include <assert.h>
 #include <string.h>
 #include <stdlib.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 
 /* Dictionary entry for Huffman (de)compression */
 typedef struct {
@@ -48,11 +47,14 @@ struct readpriv {
         int nrbits;
         uint32_t current;
         short sample;
+        /* Dictionary */
+        dictent *de;
+        int32_t new_checksum;
+        int nbits;
+        int32_t curword;
 };
 
-static int skipbytes(ft_t, int);
-
-int st_hcomstartread(ft_t ft)
+static int st_hcomstartread(ft_t ft)
 {
         struct readpriv *p = (struct readpriv *) ft->priv;
         int i;
@@ -63,16 +65,8 @@ int st_hcomstartread(ft_t ft)
         int rc;
 
 
-        /* hcom is in big endian format.  Swap whats
-         * read in on little machine
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         /* Skip first 65 bytes of header */
-        rc = skipbytes(ft, 65);
+        rc = st_skipbytes(ft, 65);
         if (rc)
             return rc;
 
@@ -84,7 +78,7 @@ int st_hcomstartread(ft_t ft)
         }
 
         /* Skip to byte 83 */
-        rc = skipbytes(ft, 83-69);
+        rc = st_skipbytes(ft, 83-69);
         if (rc)
             return rc;
 
@@ -93,7 +87,7 @@ int st_hcomstartread(ft_t ft)
         st_readdw(ft, &rsrcsize); /* bytes 87-90 */
 
         /* Skip the rest of the header (total 128 bytes) */
-        rc = skipbytes(ft, 128-91);
+        rc = st_skipbytes(ft, 128-91);
         if (rc != 0)
             return rc;
 
@@ -122,30 +116,23 @@ int st_hcomstartread(ft_t ft)
         st_readw(ft, &dictsize);
 
         /* Translate to sox parameters */
-        ft->info.encoding = ST_ENCODING_UNSIGNED;
-        ft->info.size = ST_SIZE_BYTE;
-        ft->info.rate = 22050 / divisor;
-        ft->info.channels = 1;
+        ft->signal.encoding = ST_ENCODING_UNSIGNED;
+        ft->signal.size = ST_SIZE_BYTE;
+        ft->signal.rate = 22050 / divisor;
+        ft->signal.channels = 1;
 
         /* Allocate memory for the dictionary */
-        p->dictionary = (dictent *) malloc(511 * sizeof(dictent));
-        if (p->dictionary == NULL)
-        {
-                st_fail_errno(ft,ST_ENOMEM,"can't malloc memory for Huffman dictionary");
-                return (ST_EOF);
-        }
+        p->dictionary = (dictent *)xmalloc(511 * sizeof(dictent));
 
         /* Read dictionary */
         for(i = 0; i < dictsize; i++) {
                 st_readw(ft, (unsigned short *)&(p->dictionary[i].dict_leftson));
                 st_readw(ft, (unsigned short *)&(p->dictionary[i].dict_rightson));
-                /*
-                st_report("%d %d",
+                st_debug("%d %d",
                        p->dictionary[i].dict_leftson,
                        p->dictionary[i].dict_rightson);
-                       */
         }
-        rc = skipbytes(ft, 1); /* skip pad byte */
+        rc = st_skipbytes(ft, 1); /* skip pad byte */
         if (rc)
             return rc;
 
@@ -153,7 +140,7 @@ int st_hcomstartread(ft_t ft)
         p->checksum = checksum;
         p->deltacompression = compresstype;
         if (!p->deltacompression)
-                st_report("HCOM data using value compression");
+                st_debug("HCOM data using value compression");
         p->huffcount = huffcount;
         p->cksum = 0;
         p->dictentry = 0;
@@ -162,21 +149,7 @@ int st_hcomstartread(ft_t ft)
         return (ST_SUCCESS);
 }
 
-/* FIXME: Move to misc.c */
-static int skipbytes(ft_t ft, int n)
-{
-    unsigned char trash;
-        while (--n >= 0) {
-                if (st_readb(ft, &trash) == ST_EOF)
-                {
-                        st_fail_errno(ft,ST_EOF,"unexpected EOF in Mac header");
-                        return(ST_EOF);
-                }
-        }
-        return(ST_SUCCESS);
-}
-
-st_ssize_t st_hcomread(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_hcomread(ft_t ft, st_sample_t *buf, st_size_t len)
 {
         register struct readpriv *p = (struct readpriv *) ft->priv;
         int done = 0;
@@ -192,7 +165,7 @@ st_ssize_t st_hcomread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                         return (0);
                 }
                 p->sample = sample_rate;
-                *buf++ = (p->sample - 128) * 0x1000000L;
+                *buf++ = ST_UNSIGNED_BYTE_TO_SAMPLE(p->sample,);
                 p->huffcount--;
                 p->nrbits = 0;
                 done++;
@@ -212,7 +185,7 @@ st_ssize_t st_hcomread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                         p->cksum += p->current;
                         p->nrbits = 32;
                 }
-                if(p->current & 0x80000000L) {
+                if(p->current & 0x80000000) {
                         p->dictentry =
                                 p->dictionary[p->dictentry].dict_rightson;
                 } else {
@@ -228,10 +201,7 @@ st_ssize_t st_hcomread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                                 p->sample = 0;
                         p->sample = (p->sample + datum) & 0xff;
                         p->huffcount--;
-                        if (p->sample == 0)
-                                *buf++ = -127 * 0x1000000L;
-                        else
-                                *buf++ = (p->sample - 128) * 0x1000000L;
+                        *buf++ = ST_UNSIGNED_BYTE_TO_SAMPLE(p->sample,);
                         p->dictentry = 0;
                         done++;
                         len--;
@@ -243,7 +213,7 @@ st_ssize_t st_hcomread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return done;
 }
 
-int st_hcomstopread(ft_t ft)
+static int st_hcomstopread(ft_t ft)
 {
         register struct readpriv *p = (struct readpriv *) ft->priv;
 
@@ -263,26 +233,18 @@ int st_hcomstopread(ft_t ft)
 }
 
 struct writepriv {
-        unsigned char *data;    /* Buffer allocated with malloc */
-        unsigned int size;      /* Size of allocated buffer */
-        unsigned int pos;       /* Where next byte goes */
+  unsigned char *data;          /* Buffer allocated with xmalloc */
+  st_size_t size;               /* Size of allocated buffer */
+  st_size_t pos;                /* Where next byte goes */
 };
 
 #define BUFINCR (10*BUFSIZ)
 
-int st_hcomstartwrite(ft_t ft)
+static int st_hcomstartwrite(ft_t ft)
 {
         register struct writepriv *p = (struct writepriv *) ft->priv;
 
-        /* hcom is inbigendian format.  Swap whats
-         * read in on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
-        switch (ft->info.rate) {
+        switch (ft->signal.rate) {
         case 22050:
         case 22050/2:
         case 22050/3:
@@ -292,288 +254,216 @@ int st_hcomstartwrite(ft_t ft)
                 st_fail_errno(ft,ST_EFMT,"unacceptable output rate for HCOM: try 5512, 7350, 11025 or 22050 hertz");
                 return (ST_EOF);
         }
-        ft->info.size = ST_SIZE_BYTE;
-        ft->info.encoding = ST_ENCODING_UNSIGNED;
-        ft->info.channels = 1;
+        ft->signal.size = ST_SIZE_BYTE;
+        ft->signal.encoding = ST_ENCODING_UNSIGNED;
+        ft->signal.channels = 1;
 
         p->size = BUFINCR;
         p->pos = 0;
-        p->data = (unsigned char *) malloc(p->size);
-        if (p->data == NULL)
-        {
-                st_fail_errno(ft,ST_ENOMEM,"can't malloc buffer for uncompressed HCOM data");
-                return (ST_EOF);
-        }
+        p->data = (unsigned char *) xmalloc(p->size);
         return (ST_SUCCESS);
 }
 
-st_ssize_t st_hcomwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_hcomwrite(ft_t ft, const st_sample_t *buf, st_size_t len)
 {
-        register struct writepriv *p = (struct writepriv *) ft->priv;
-        st_sample_t datum;
-        st_ssize_t save_len = len;
+  struct writepriv *p = (struct writepriv *) ft->priv;
+  st_sample_t datum;
+  st_size_t i;
 
-        if (len == 0)
-            return (0);
+  if (len == 0)
+    return 0;
 
-        if (p->pos + len > p->size) {
-                p->size = ((p->pos + len) / BUFINCR + 1) * BUFINCR;
-                p->data = (unsigned char *) realloc(p->data, p->size);
-                if (p->data == NULL)
-                {
-                    st_fail_errno(ft,ST_ENOMEM,"can't realloc buffer for uncompressed HCOM data");
-                    return (0);
-                }
-        }
+  if (p->pos + len > p->size) {
+    p->size = ((p->pos + len) / BUFINCR + 1) * BUFINCR;
+    p->data = (unsigned char *)xrealloc(p->data, p->size);
+  }
 
-        while (--len >= 0) {
-                datum = *buf++;
-                datum >>= 24;
-                datum ^= 128;
-                p->data[p->pos++] = datum;
-        }
+  for (i = 0; i < len; i++) {
+    datum = *buf++;
+    p->data[p->pos++] = ST_SAMPLE_TO_UNSIGNED_BYTE(datum, ft->clips);
+  }
 
-        return (save_len - len);
+  return len;
 }
 
-/* Some global compression stuff hcom uses.  hcom currently has problems */
-/* compiling here.  It could really use some cleaning up by someone that */
-/* understands this format. */
-
-/* XXX This uses global variables -- one day these should all be
-   passed around in a structure instead. Use static so we don't
-   polute global name space. */
-
-/* SJB: FIXME: dangerous static variables, need to analyse code */
-
-static dictent dictionary[511];
-static dictent *de;
-static long codes[256];
-static long codesize[256];
-static int32_t checksum;
-
-static void makecodes(int e, int c, int s, int b)
+static void makecodes(int e, int c, int s, int b, dictent newdict[511], long codes[256], long codesize[256])
 {
-  if(dictionary[e].dict_leftson < 0) {
-    codes[dictionary[e].dict_rightson] = c;
-    codesize[dictionary[e].dict_rightson] = s;
+  assert(b);                    /* Prevent stack overflow */
+  if (newdict[e].dict_leftson < 0) {
+    codes[newdict[e].dict_rightson] = c;
+    codesize[newdict[e].dict_rightson] = s;
   } else {
-    makecodes(dictionary[e].dict_leftson, c, s + 1, b << 1);
-    makecodes(dictionary[e].dict_rightson, c + b, s + 1, b << 1);
+    makecodes(newdict[e].dict_leftson, c, s + 1, b << 1, newdict, codes, codesize);
+    makecodes(newdict[e].dict_rightson, c + b, s + 1, b << 1, newdict, codes, codesize);
   }
 }
 
-static int nbits;
-static int32_t curword;
-
-/* FIXME: Place in misc.c */
-static void putlong(unsigned char *c, int32_t v)
+static void putcode(ft_t ft, long codes[256], long codesize[256], unsigned char c, unsigned char **df)
 {
-  *c++ = (v >> 24) & 0xff;
-  *c++ = (v >> 16) & 0xff;
-  *c++ = (v >> 8) & 0xff;
-  *c++ = v & 0xff;
-}
-
-static void putshort(unsigned char *c, short v)
-{
-  *c++ = (v >> 8) & 0xff;
-  *c++ = v & 0xff;
-}
-
-
-static void putcode(unsigned char c, unsigned char **df)
-{
-long code, size;
-int i;
+  struct readpriv *p = (struct readpriv *) ft->priv;
+  long code, size;
+  int i;
 
   code = codes[c];
   size = codesize[c];
   for(i = 0; i < size; i++) {
-    curword = (curword << 1);
-    if(code & 1) curword += 1;
-    nbits++;
-    if(nbits == 32) {
-      putlong(*df, curword);
-      checksum += curword;
-      (*df) += 4;
-      nbits = 0;
-      curword = 0;
+    p->curword <<= 1;
+    if (code & 1)
+      p->curword += 1;
+    p->nbits++;
+    if (p->nbits == 32) {
+      put32_be(df, p->curword);
+      p->new_checksum += p->curword;
+      p->nbits = 0;
+      p->curword = 0;
     }
-    code = code >> 1;
+    code >>= 1;
   }
 }
 
-static int compress(unsigned char **df, int32_t *dl, float fr)
+static void compress(ft_t ft, unsigned char **df, int32_t *dl, float fr)
 {
+  struct readpriv *p = (struct readpriv *) ft->priv;
   int32_t samplerate;
   unsigned char *datafork = *df;
-  unsigned char *ddf;
+  unsigned char *ddf, *dfp;
   short dictsize;
   int frequtable[256];
+  long codes[256], codesize[256];
+  dictent newdict[511];
   int i, sample, j, k, d, l, frequcount;
 
   sample = *datafork;
-  for(i = 0; i < 256; i++) frequtable[i] = 0;
-  for(i = 1; i < *dl; i++) {
+  memset(frequtable, 0, sizeof(frequtable));
+  memset(codes, 0, sizeof(codes));
+  memset(codesize, 0, sizeof(codesize));
+  memset(newdict, 0, sizeof(newdict));
+  
+  for (i = 1; i < *dl; i++) {
     d = (datafork[i] - (sample & 0xff)) & 0xff; /* creates absolute entries LMS */
     sample = datafork[i];
     datafork[i] = d;
-#if 0                           /* checking our table is accessed correctly */
-    if(d < 0 || d > 255)
-      printf("d is outside array bounds %d\n", d);
-#endif
+    assert(d >= 0 && d <= 255); /* check our table is accessed correctly */
     frequtable[d]++;
   }
-  de = dictionary;
-  for(i = 0; i < 256; i++) if(frequtable[i] != 0) {
-    de->frequ = -frequtable[i];
-    de->dict_leftson = -1;
-    de->dict_rightson = i;
-    de++;
-  }
-  frequcount = de - dictionary;
-  for(i = 0; i < frequcount; i++) {
-    for(j = i + 1; j < frequcount; j++) {
-      if(dictionary[i].frequ > dictionary[j].frequ) {
-        k = dictionary[i].frequ;
-        dictionary[i].frequ = dictionary[j].frequ;
-        dictionary[j].frequ = k;
-        k = dictionary[i].dict_leftson;
-        dictionary[i].dict_leftson = dictionary[j].dict_leftson;
-        dictionary[j].dict_leftson = k;
-        k = dictionary[i].dict_rightson;
-        dictionary[i].dict_rightson = dictionary[j].dict_rightson;
-        dictionary[j].dict_rightson = k;
+  p->de = newdict;
+  for (i = 0; i < 256; i++)
+    if (frequtable[i] != 0) {
+      p->de->frequ = -frequtable[i];
+      p->de->dict_leftson = -1;
+      p->de->dict_rightson = i;
+      p->de++;
+    }
+  frequcount = p->de - newdict;
+  for (i = 0; i < frequcount; i++) {
+    for (j = i + 1; j < frequcount; j++) {
+      if (newdict[i].frequ > newdict[j].frequ) {
+        k = newdict[i].frequ;
+        newdict[i].frequ = newdict[j].frequ;
+        newdict[j].frequ = k;
+        k = newdict[i].dict_leftson;
+        newdict[i].dict_leftson = newdict[j].dict_leftson;
+        newdict[j].dict_leftson = k;
+        k = newdict[i].dict_rightson;
+        newdict[i].dict_rightson = newdict[j].dict_rightson;
+        newdict[j].dict_rightson = k;
       }
     }
   }
-  while(frequcount > 1) {
+  while (frequcount > 1) {
     j = frequcount - 1;
-    de->frequ = dictionary[j - 1].frequ;
-    de->dict_leftson = dictionary[j - 1].dict_leftson;
-    de->dict_rightson = dictionary[j - 1].dict_rightson;
-    l = dictionary[j - 1].frequ + dictionary[j].frequ;
-    for(i = j - 2; i >= 0; i--) {
-      if(l >= dictionary[i].frequ) break;
-      dictionary[i + 1] = dictionary[i];
-    }
+    p->de->frequ = newdict[j - 1].frequ;
+    p->de->dict_leftson = newdict[j - 1].dict_leftson;
+    p->de->dict_rightson = newdict[j - 1].dict_rightson;
+    l = newdict[j - 1].frequ + newdict[j].frequ;
+    for (i = j - 2; i >= 0 && l < newdict[i].frequ; i--)
+      newdict[i + 1] = newdict[i];
     i = i + 1;
-    dictionary[i].frequ = l;
-    dictionary[i].dict_leftson = j;
-    dictionary[i].dict_rightson = de - dictionary;
-    de++;
+    newdict[i].frequ = l;
+    newdict[i].dict_leftson = j;
+    newdict[i].dict_rightson = p->de - newdict;
+    p->de++;
     frequcount--;
   }
-  dictsize = de - dictionary;
-  for(i = 0; i < 256; i++) {
-    codes[i] = 0;
-    codesize[i] = 0;
-  }
-  makecodes(0, 0, 0, 1);
+  dictsize = p->de - newdict;
+  makecodes(0, 0, 0, 1, newdict, codes, codesize);
   l = 0;
-  for(i = 0; i < 256; i++) {
-          l += frequtable[i] * codesize[i];
-  }
+  for (i = 0; i < 256; i++)
+    l += frequtable[i] * codesize[i];
   l = (((l + 31) >> 5) << 2) + 24 + dictsize * 4;
-  st_report("  Original size: %6d bytes", *dl);
-  st_report("Compressed size: %6d bytes", l);
-  if((datafork = (unsigned char *)malloc((unsigned)l)) == NULL)
-  {
-    return (ST_ENOMEM);
-  }
+  st_debug("  Original size: %6d bytes", *dl);
+  st_debug("Compressed size: %6d bytes", l);
+  datafork = (unsigned char *)xmalloc((unsigned)l);
   ddf = datafork + 22;
   for(i = 0; i < dictsize; i++) {
-    putshort(ddf, dictionary[i].dict_leftson);
-    ddf += 2;
-    putshort(ddf, dictionary[i].dict_rightson);
-    ddf += 2;
+    put16_be(&ddf, newdict[i].dict_leftson);
+    put16_be(&ddf, newdict[i].dict_rightson);
   }
   *ddf++ = 0;
   *ddf++ = *(*df)++;
-  checksum = 0;
-  nbits = 0;
-  curword = 0;
-  for(i = 1; i < *dl; i++) putcode(*(*df)++, &ddf);
-  if(nbits != 0) {
+  p->new_checksum = 0;
+  p->nbits = 0;
+  p->curword = 0;
+  for (i = 1; i < *dl; i++)
+    putcode(ft, codes, codesize, *(*df)++, &ddf);
+  if (p->nbits != 0) {
     codes[0] = 0;
-    codesize[0] = 32 - nbits;
-    putcode(0, &ddf);
+    codesize[0] = 32 - p->nbits;
+    putcode(ft, codes, codesize, 0, &ddf);
   }
-  strncpy((char *) datafork, "HCOM", 4);
-  putlong(datafork + 4, *dl);
-  putlong(datafork + 8, checksum);
-  putlong(datafork + 12, 1L);
+  strncpy((char *)datafork, "HCOM", 4);
+  dfp = datafork + 4;
+  put32_be(&dfp, *dl);
+  put32_be(&dfp, p->new_checksum);
+  put32_be(&dfp, 1);
   samplerate = 22050 / (int32_t)fr;
-  putlong(datafork + 16, samplerate);
-  putshort(datafork + 20, dictsize);
+  put32_be(&dfp, samplerate);
+  put16_be(&dfp, dictsize);
   *df = datafork;               /* reassign passed pointer to new datafork */
   *dl = l;                      /* and its compressed length */
-
-  return (ST_SUCCESS);
 }
 
-/* FIXME: Place in misc.c */
-static void padbytes(ft_t ft, int n)
-{
-        while (--n >= 0)
-            st_writeb(ft, '\0');
-}
-
-
 /* End of hcom utility routines */
 
-int st_hcomstopwrite(ft_t ft)
+static int st_hcomstopwrite(ft_t ft)
 {
-        register struct writepriv *p = (struct writepriv *) ft->priv;
-        unsigned char *compressed_data = p->data;
-        uint32_t compressed_len = p->pos;
-        int rc;
+  struct writepriv *p = (struct writepriv *) ft->priv;
+  unsigned char *compressed_data = p->data;
+  st_size_t compressed_len = p->pos;
+  int rc = ST_SUCCESS;
 
-        /* Compress it all at once */
-        rc = compress(&compressed_data, (int32_t *)&compressed_len, (double) ft->info.rate);
-        free((char *) p->data);
+  /* Compress it all at once */
+  if (compressed_len)
+    compress(ft, &compressed_data, (int32_t *)&compressed_len, (double) ft->signal.rate);
+  free((char *)p->data);
 
-        if (rc){
-        st_fail_errno(ft, rc,"can't malloc buffer for compressed HCOM data");
-            return 0;
-        }
+  /* Write the header */
+  st_writebuf(ft, (void *)"\000\001A", 1, 3); /* Dummy file name "A" */
+  st_padbytes(ft, 65-3);
+  st_writes(ft, "FSSD");
+  st_padbytes(ft, 83-69);
+  st_writedw(ft, (uint32_t)compressed_len); /* compressed_data size */
+  st_writedw(ft, 0); /* rsrc size */
+  st_padbytes(ft, 128 - 91);
+  if (st_error(ft)) {
+    st_fail_errno(ft, errno, "write error in HCOM header");
+    rc = ST_EOF;
+  } else if (st_writebuf(ft, compressed_data, 1, compressed_len) != compressed_len) {
+    /* Write the compressed_data fork */
+    st_fail_errno(ft, errno, "can't write compressed HCOM data");
+    rc = ST_EOF;
+  }
+  free((char *)compressed_data);
 
-        /* Write the header */
-        st_writebuf(ft, (void *)"\000\001A", 1, 3); /* Dummy file name "A" */
-        padbytes(ft, 65-3);
-        st_writes(ft, "FSSD");
-        padbytes(ft, 83-69);
-        st_writedw(ft, (uint32_t) compressed_len); /* compressed_data size */
-        st_writedw(ft, (uint32_t) 0); /* rsrc size */
-        padbytes(ft, 128 - 91);
-        if (st_error(ft))
-        {
-                st_fail_errno(ft,errno,"write error in HCOM header");
-                return (ST_EOF);
-        }
+  if (rc == ST_SUCCESS)
+    /* Pad the compressed_data fork to a multiple of 128 bytes */
+    st_padbytes(ft, 128 - (int) (compressed_len % 128));
 
-        /* Write the compressed_data fork */
-        if (st_writebuf(ft, compressed_data, 1, (int)compressed_len) != compressed_len)
-        {
-                st_fail_errno(ft,errno,"can't write compressed HCOM data");
-                rc = ST_EOF;
-        }
-        else
-            rc = ST_SUCCESS;
-        free((char *) compressed_data);
-
-        if (rc)
-            return rc;
-
-        /* Pad the compressed_data fork to a multiple of 128 bytes */
-        padbytes(ft, 128 - (int) (compressed_len%128));
-
-        return (ST_SUCCESS);
+  return rc;
 }
 
 /* Mac FSSD/HCOM */
-static char *hcomnames[] = {
+static const char *hcomnames[] = {
   "hcom",
   NULL
 };
@@ -581,7 +471,7 @@ static char *hcomnames[] = {
 static st_format_t st_hcom_format = {
   hcomnames,
   NULL,
-  0,
+  ST_FILE_BIG_END,
   st_hcomstartread,
   st_hcomread,
   st_hcomstopread,
