@@ -1,29 +1,20 @@
 /*
- * July 5, 1991
- * Copyright 1991 Lance Norskog And Sundry Contributors
+ * Sound Tools MAUD file format driver, by Lutz Vieweg 1993
+ *
+ * supports: mono and stereo, linear, a-law and u-law reading and writing
+ *
+ * Copyright 1998-2006 Chris Bagwell and SoX Contributors
  * This source code is freely redistributable and may be used for
  * any purpose.  This copyright notice must be maintained. 
  * Lance Norskog And Sundry Contributors are not responsible for 
  * the consequences of using this software.
  */
 
-/*
- * Sound Tools MAUD file format driver, by Lutz Vieweg 1993
- *
- * supports: mono and stereo, linear, a-law and u-law reading and writing
- *
- * March 3, 1999 - cbagwell
- *   Changed to use rawread for reading.
- *
- */
-
 #include "st_i.h"
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>     /* For SEEK_* defines if not found in stdio */
 #endif
@@ -42,7 +33,7 @@ static void maudwriteheader(ft_t);
  *      size and encoding of samples, 
  *      mono/stereo/quad.
  */
-int st_maudstartread(ft_t ft) 
+static int st_maudstartread(ft_t ft) 
 {
         struct maudstuff * p = (struct maudstuff *) ft->priv;
         
@@ -64,14 +55,6 @@ int st_maudstartread(ft_t ft)
         if (rc)
             return rc;
 
-        /* maud is in big endian format.  Swap whats read in
-         * on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-        
         /* read FORM chunk */
         if (st_reads(ft, buf, 4) == ST_EOF || strncmp(buf, "FORM", 4) != 0)
         {
@@ -93,7 +76,7 @@ int st_maudstartread(ft_t ft)
                 
                 /*
                 buf[4] = 0;
-                st_report("chunk %s",buf);
+                st_debug("chunk %s",buf);
                 */
                 
                 if (strncmp(buf,"MHDR",4) == 0) {
@@ -105,7 +88,7 @@ int st_maudstartread(ft_t ft)
                             return(ST_EOF);
                         }
                         
-                        /* fseek(ft->fp,12,SEEK_CUR); */
+                        /* fseeko(ft->fp,12,SEEK_CUR); */
 
                         /* number of samples stored in MDAT */
                         st_readdw(ft, &(p->nsamples));
@@ -124,15 +107,15 @@ int st_maudstartread(ft_t ft)
                             return (ST_EOF);
                         }
                         
-                        ft->info.rate = nom / denom;
+                        ft->signal.rate = nom / denom;
                         
                         st_readw(ft, &chaninf); /* channel information */
                         switch (chaninf) {
                         case 0:
-                                ft->info.channels = 1;
+                                ft->signal.channels = 1;
                                 break;
                         case 1:
-                                ft->info.channels = 2;
+                                ft->signal.channels = 2;
                                 break;
                         default:
                                 st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels in file");
@@ -140,7 +123,7 @@ int st_maudstartread(ft_t ft)
                         }
                         
                         st_readw(ft, &chaninf); /* number of channels (mono: 1, stereo: 2, ...) */
-                        if (chaninf != ft->info.channels) 
+                        if (chaninf != ft->signal.channels) 
                         {
                                 st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels in file");
                             return(ST_EOF);
@@ -153,20 +136,20 @@ int st_maudstartread(ft_t ft)
                         st_readdw(ft, &trash32);
                         
                         if (bitpersam == 8 && chaninf == 0) {
-                                ft->info.size = ST_SIZE_BYTE;
-                                ft->info.encoding = ST_ENCODING_UNSIGNED;
+                                ft->signal.size = ST_SIZE_BYTE;
+                                ft->signal.encoding = ST_ENCODING_UNSIGNED;
                         }
                         else if (bitpersam == 8 && chaninf == 2) {
-                                ft->info.size = ST_SIZE_BYTE;
-                                ft->info.encoding = ST_ENCODING_ALAW;
+                                ft->signal.size = ST_SIZE_BYTE;
+                                ft->signal.encoding = ST_ENCODING_ALAW;
                         }
                         else if (bitpersam == 8 && chaninf == 3) {
-                                ft->info.size = ST_SIZE_BYTE;
-                                ft->info.encoding = ST_ENCODING_ULAW;
+                                ft->signal.size = ST_SIZE_BYTE;
+                                ft->signal.encoding = ST_ENCODING_ULAW;
                         }
                         else if (bitpersam == 16 && chaninf == 0) {
-                                ft->info.size = ST_SIZE_WORD;
-                                ft->info.encoding = ST_ENCODING_SIGN2;
+                                ft->signal.size = ST_SIZE_16BIT;
+                                ft->signal.encoding = ST_ENCODING_SIGN2;
                         }
                         else 
                         {
@@ -183,12 +166,7 @@ int st_maudstartread(ft_t ft)
                         st_readdw(ft, &chunksize);
                         if (chunksize & 1)
                                 chunksize++;
-                        chunk_buf = (char *) malloc(chunksize + 1);
-                        if (!chunk_buf)
-                        {
-                            st_fail_errno(ft,ST_ENOMEM,"Couldn't alloc resources");
-                            return(ST_EOF);
-                        }
+                        chunk_buf = (char *) xmalloc(chunksize + 1);
                         if (st_readbuf(ft, chunk_buf, 1, (int)chunksize) 
                             != chunksize)
                         {
@@ -196,7 +174,7 @@ int st_maudstartread(ft_t ft)
                                 return(ST_EOF);
                         }
                         chunk_buf[chunksize] = '\0';
-                        st_report("%s",chunk_buf);
+                        st_debug("%s",chunk_buf);
                         free(chunk_buf);
                         
                         continue;
@@ -220,29 +198,7 @@ int st_maudstartread(ft_t ft)
         return(ST_SUCCESS);
 }
 
-/*
- * Read up to len samples from file.
- * Convert to signed longs.
- * Place in buf[].
- * Return number of samples read.
- */
-
-st_ssize_t st_maudread(ft_t ft, st_sample_t *buf, st_ssize_t len) 
-{
-        return (st_rawread(ft, buf, len));
-}
-
-/*
- * Do anything required when you stop reading samples.  
- * Don't close input file! 
- */
-int st_maudstopread(ft_t ft) 
-{
-        /* Needed because of rawread() */
-        return st_rawstopread(ft);
-}
-
-int st_maudstartwrite(ft_t ft) 
+static int st_maudstartwrite(ft_t ft) 
 {
         struct maudstuff * p = (struct maudstuff *) ft->priv;
         int rc;
@@ -252,14 +208,6 @@ int st_maudstartwrite(ft_t ft)
         if (rc)
             return rc;
 
-        /* maud is in big endian format.  Swap whats read in
-         * on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-        
         /* If you have to seek around the output file */
         if (! ft->seekable) 
         {
@@ -267,24 +215,24 @@ int st_maudstartwrite(ft_t ft)
             return (ST_EOF);
         }
         
-        if (ft->info.channels != 1 && ft->info.channels != 2) {
+        if (ft->signal.channels != 1 && ft->signal.channels != 2) {
                 st_fail_errno(ft,ST_EFMT,"MAUD: unsupported number of channels, unable to store");
                 return(ST_EOF);
         }
-        if (ft->info.size == ST_SIZE_WORD) ft->info.encoding = ST_ENCODING_SIGN2;
-        if (ft->info.encoding == ST_ENCODING_ULAW || 
-            ft->info.encoding == ST_ENCODING_ALAW) ft->info.size = ST_SIZE_BYTE;
-        if (ft->info.size == ST_SIZE_BYTE && 
-            ft->info.encoding == ST_ENCODING_SIGN2) 
-            ft->info.encoding = ST_ENCODING_UNSIGNED;
+        if (ft->signal.size == ST_SIZE_16BIT) ft->signal.encoding = ST_ENCODING_SIGN2;
+        if (ft->signal.encoding == ST_ENCODING_ULAW || 
+            ft->signal.encoding == ST_ENCODING_ALAW) ft->signal.size = ST_SIZE_BYTE;
+        if (ft->signal.size == ST_SIZE_BYTE && 
+            ft->signal.encoding == ST_ENCODING_SIGN2) 
+            ft->signal.encoding = ST_ENCODING_UNSIGNED;
         
-        p->nsamples = 0x7f000000L;
+        p->nsamples = 0x7f000000;
         maudwriteheader(ft);
         p->nsamples = 0;
         return (ST_SUCCESS);
 }
 
-st_ssize_t st_maudwrite(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_maudwrite(ft_t ft, const st_sample_t *buf, st_size_t len) 
 {
         struct maudstuff * p = (struct maudstuff *) ft->priv;
         
@@ -293,7 +241,7 @@ st_ssize_t st_maudwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return st_rawwrite(ft, buf, len);
 }
 
-int st_maudstopwrite(ft_t ft) 
+static int st_maudstopwrite(ft_t ft) 
 {
         int rc;
 
@@ -304,7 +252,7 @@ int st_maudstopwrite(ft_t ft)
 
         /* All samples are already written out. */
         
-        if (st_seeki(ft, 0L, 0) != 0) 
+        if (st_seeki(ft, 0, 0) != 0) 
         {
             st_fail_errno(ft,errno,"can't rewind output file to rewrite MAUD header");
             return(ST_EOF);
@@ -320,14 +268,14 @@ static void maudwriteheader(ft_t ft)
         struct maudstuff * p = (struct maudstuff *) ft->priv;
         
         st_writes(ft, "FORM");
-        st_writedw(ft, (p->nsamples*ft->info.size) + MAUDHEADERSIZE);  /* size of file */
+        st_writedw(ft, (p->nsamples*ft->signal.size) + MAUDHEADERSIZE);  /* size of file */
         st_writes(ft, "MAUD"); /* File type */
         
         st_writes(ft, "MHDR");
         st_writedw(ft,  8*4); /* number of bytes to follow */
         st_writedw(ft, p->nsamples);  /* number of samples stored in MDAT */
         
-        switch (ft->info.encoding) {
+        switch (ft->signal.encoding) {
                 
         case ST_ENCODING_UNSIGNED:
                 st_writew(ft, (int) 8); /* number of bits per sample as stored in MDAT */
@@ -344,13 +292,15 @@ static void maudwriteheader(ft_t ft)
                 st_writew(ft, (int) 8); /* number of bits per sample as stored in MDAT */
                 st_writew(ft, (int) 16); /* number of bits per sample after decompression */
                 break;
-                
+
+        default:
+                break;
         }
         
-        st_writedw(ft, ft->info.rate); /* clock source frequency */
+        st_writedw(ft, ft->signal.rate); /* clock source frequency */
         st_writew(ft, (int) 1); /* clock devide */
         
-        if (ft->info.channels == 1) {
+        if (ft->signal.channels == 1) {
                 st_writew(ft, (int) 0); /* channel information */
                 st_writew(ft, (int) 1); /* number of channels (mono: 1, stereo: 2, ...) */
         }
@@ -359,7 +309,7 @@ static void maudwriteheader(ft_t ft)
                 st_writew(ft, (int) 2);
         }
         
-        switch (ft->info.encoding) {
+        switch (ft->signal.encoding) {
                 
         case ST_ENCODING_UNSIGNED:
         case ST_ENCODING_SIGN2:
@@ -373,7 +323,9 @@ static void maudwriteheader(ft_t ft)
         case ST_ENCODING_ALAW:
                 st_writew(ft, (int) 2);
                 break;
-                
+
+        default:
+                break;
         }
         
         st_writedw(ft, 0); /* reserved */
@@ -385,11 +337,11 @@ static void maudwriteheader(ft_t ft)
         st_writes(ft, "file create by Sound eXchange ");
         
         st_writes(ft, "MDAT");
-        st_writedw(ft, p->nsamples * ft->info.size ); /* samples in file */
+        st_writedw(ft, p->nsamples * ft->signal.size ); /* samples in file */
 }
 
 /* Amiga MAUD */
-static char *maudnames[] = {
+static const char *maudnames[] = {
   "maud",
   NULL,
 };
@@ -397,10 +349,10 @@ static char *maudnames[] = {
 static st_format_t st_maud_format = {
   maudnames,
   NULL,
-  ST_FILE_STEREO,
+  ST_FILE_BIG_END,
   st_maudstartread,
-  st_maudread,
-  st_maudstopread,
+  st_rawread,
+  st_rawstopread,
   st_maudstartwrite,
   st_maudwrite,
   st_maudstopwrite,
