@@ -67,7 +67,7 @@
  *   the companders have different prediction intervals (volume
  *   application delays).  Note that the predictive mode of the
  *   limiter needs some TLC - in fact, a rewrite - since what's really
- *   useful is to assure that a waveform won't be clipped, be slewing
+ *   useful is to assure that a waveform won't be clipped, by slewing
  *   the volume in advance so that the peak is at limit (or below, if
  *   there's a higher subsequent peak visible in the lookahead window)
  *   once it's reached.  */
@@ -94,16 +94,8 @@ typedef struct butterworth_crossover {
 static int lowpass_setup (butterworth_crossover_t butterworth, double frequency, st_rate_t rate, int nchan) {
   double c;
 
-  if (! (butterworth->xy_low = (struct xy *)malloc(nchan * sizeof(struct xy)))) {
-    st_fail("Out of memory");
-    return (ST_EOF);
-  }
-  memset(butterworth->xy_low,0,nchan * sizeof(struct xy));
-  if (! (butterworth->xy_high = (struct xy *)malloc(nchan * sizeof(struct xy)))) {
-    st_fail("Out of memory");
-    return (ST_EOF);
-  }
-  memset(butterworth->xy_high,0,nchan * sizeof(struct xy));
+  butterworth->xy_low = (struct xy *)xcalloc(nchan, sizeof(struct xy));
+  butterworth->xy_high = (struct xy *)xcalloc(nchan, sizeof(struct xy));
 
   /* lowpass setup */
   butterworth->frequency_low = frequency/1.3;
@@ -132,7 +124,7 @@ static int lowpass_setup (butterworth_crossover_t butterworth, double frequency,
   return (ST_SUCCESS);
 }
 
-static int lowpass_flow(butterworth_crossover_t butterworth, int nChan, st_sample_t *ibuf, st_sample_t *lowbuf, st_sample_t *highbuf,
+static int lowpass_flow(eff_t effp, butterworth_crossover_t butterworth, int nChan, st_sample_t *ibuf, st_sample_t *lowbuf, st_sample_t *highbuf,
                          int len) {
   int chan;
   double in, out;
@@ -168,11 +160,10 @@ static int lowpass_flow(butterworth_crossover_t butterworth, int nChan, st_sampl
       butterworth->xy_low[chan].y [1] = butterworth->xy_low[chan].y [0];
       butterworth->xy_low[chan].y [0] = out;
 
-      ST_SAMPLE_CLIP(out);
+      ST_SAMPLE_CLIP_COUNT(out, effp->clips);
 
       *lowbufptr = out;
 
-
       out =
         butterworth->a_high[0] * in +
         butterworth->a_high [1] * butterworth->xy_high[chan].x [0] +
@@ -185,7 +176,7 @@ static int lowpass_flow(butterworth_crossover_t butterworth, int nChan, st_sampl
       butterworth->xy_high[chan].y [1] = butterworth->xy_high[chan].y [0];
       butterworth->xy_high[chan].y [0] = out;
 
-      ST_SAMPLE_CLIP(out);
+      ST_SAMPLE_CLIP_COUNT(out, effp->clips);
 
       /* don't forget polarity reversal of high pass! */
 
@@ -200,7 +191,7 @@ static int lowpass_flow(butterworth_crossover_t butterworth, int nChan, st_sampl
 }
 
 typedef struct comp_band {
-  int expectedChannels; /* Also flags that channels aren't to be treated
+  st_size_t expectedChannels; /* Also flags that channels aren't to be treated
                            individually when = 1 and input not mono */
   int transferPoints;   /* Number of points specified on the transfer
                            function */
@@ -214,16 +205,16 @@ typedef struct comp_band {
   double topfreq;       /* upper bound crossover frequency */
   struct butterworth_crossover filter;
   st_sample_t *delay_buf;   /* Old samples, used for delay processing */
-  st_ssize_t delay_size;    /* lookahead for this band (in samples) - function of delay, above */
+  st_size_t delay_size;    /* lookahead for this band (in samples) - function of delay, above */
   st_ssize_t delay_buf_ptr; /* Index into delay_buf */
-  st_ssize_t delay_buf_cnt; /* No. of active entries in delay_buf */
+  st_size_t delay_buf_cnt; /* No. of active entries in delay_buf */
 } *comp_band_t;
 
 typedef struct {
   int nBands;
   st_sample_t *band_buf1, *band_buf2, *band_buf3;
   int band_buf_len;
-  st_ssize_t delay_buf_size;/* Size of delay_buf in samples */
+  st_size_t delay_buf_size;/* Size of delay_buf in samples */
   struct comp_band *bands;
 } *compand_t;
 
@@ -236,7 +227,7 @@ typedef struct {
 static int st_mcompand_getopts_1(comp_band_t l, int n, char **argv)
 {
       char *s;
-      int rates, tfers, i, commas;
+      st_size_t rates, tfers, i, commas;
 
       /* Start by checking the attack and decay rates */
 
@@ -251,21 +242,10 @@ static int st_mcompand_getopts_1(comp_band_t l, int n, char **argv)
       }
 
       rates = 1 + commas/2;
-      if ((l->attackRate = (double *)malloc(sizeof(double) * rates)) == NULL ||
-          (l->decayRate  = (double *)malloc(sizeof(double) * rates)) == NULL)
-      {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
-
-      if ((l->volume = (double *)malloc(sizeof(double) * rates)) == NULL)
-      {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
-
+      l->attackRate = (double *)xmalloc(sizeof(double) * rates);
+      l->decayRate  = (double *)xmalloc(sizeof(double) * rates);
+      l->volume = (double *)xmalloc(sizeof(double) * rates);
       l->expectedChannels = rates;
-
       l->delay_buf = NULL;
 
       /* Now tokenise the rates string and set up these arrays.  Keep
@@ -292,12 +272,8 @@ static int st_mcompand_getopts_1(comp_band_t l, int n, char **argv)
       }
 
       tfers = 3 + commas/2; /* 0, 0 at start; 1, 1 at end */
-      if ((l->transferIns  = (double *)malloc(sizeof(double) * tfers)) == NULL ||
-          (l->transferOuts = (double *)malloc(sizeof(double) * tfers)) == NULL)
-      {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
+      l->transferIns  = (double *)xmalloc(sizeof(double) * tfers);
+      l->transferOuts = (double *)xmalloc(sizeof(double) * tfers);
       l->transferPoints = tfers;
       l->transferIns[0] = 0.0; l->transferOuts[0] = 0.0;
       l->transferIns[tfers-1] = 1.0; l->transferOuts[tfers-1] = 1.0;
@@ -311,7 +287,7 @@ static int st_mcompand_getopts_1(comp_band_t l, int n, char **argv)
         l->transferIns[i]  = pow(10.0, atof(s)/20.0);
         if (l->transferIns[i] > 1.0)
         {
-          st_fail("dB values are relative to maximum input, and, ipso facto, "
+          st_fail("dB values are relative to maximum input, and, ipso facto, \n"
                "cannot exceed 0");
           return (ST_EOF);
         }
@@ -319,7 +295,7 @@ static int st_mcompand_getopts_1(comp_band_t l, int n, char **argv)
           --(l->transferPoints);
         if (i > 0 && l->transferIns[i] <= l->transferIns[i-1])
         {
-          st_fail("Transfer function points don't have strictly ascending "
+          st_fail("Transfer function points don't have strictly ascending \n"
                "input amplitude");
           return (ST_EOF);
         }
@@ -380,7 +356,7 @@ static int parse_subarg(char *s, char **subargv, int *subargc) {
       return ST_SUCCESS;
 }
 
-int st_mcompand_getopts(eff_t effp, int n, char **argv) 
+static int st_mcompand_getopts(eff_t effp, int n, char **argv) 
 {
   char *subargv[6], *cp;
   int subargc, i, len;
@@ -398,11 +374,7 @@ int st_mcompand_getopts(eff_t effp, int n, char **argv)
   }
   c->nBands = (n+1)>>1;
 
-  if (! (c->bands = (struct comp_band *)malloc(c->nBands * sizeof(struct comp_band)))) {
-    st_fail("Out of memory");
-    return ST_EOF;
-  }
-  memset(c->bands,0,c->nBands * sizeof(struct comp_band));
+  c->bands = (struct comp_band *)xcalloc(c->nBands, sizeof(struct comp_band));
 
   for (i=0;i<c->nBands;++i) {
     len = strlen(argv[i<<1]);
@@ -432,11 +404,12 @@ int st_mcompand_getopts(eff_t effp, int n, char **argv)
  * Prepare processing.
  * Do all initializations.
  */
-int st_mcompand_start(eff_t effp)
+static int st_mcompand_start(eff_t effp)
 {
   compand_t c = (compand_t) effp->priv;
   comp_band_t l;
-  int band, i;
+  st_size_t i;
+  int band;
   
   for (band=0;band<c->nBands;++band) {
     l = &c->bands[band];
@@ -463,15 +436,8 @@ int st_mcompand_start(eff_t effp)
     }
 
     /* Allocate the delay buffer */
-    if (c->delay_buf_size > 0) {
-      if ((l->delay_buf = (st_sample_t *)malloc(sizeof(long) * c->delay_buf_size)) == NULL) {
-        st_fail("Out of memory");
-        return (ST_EOF);
-      }
-      for (i = 0;  i < c->delay_buf_size;  i++)
-        l->delay_buf[i] = 0;
-
-    }
+    if (c->delay_buf_size > 0)
+      l->delay_buf = (st_sample_t *)xcalloc(sizeof(long), c->delay_buf_size);
     l->delay_buf_ptr = 0;
     l->delay_buf_cnt = 0;
 
@@ -497,7 +463,7 @@ static void doVolume(double *v, double samp, comp_band_t l, int chan)
     *v += delta * l->decayRate[chan];
 }
 
-static int st_mcompand_flow_1(compand_t c, comp_band_t l, st_sample_t *ibuf, st_sample_t *obuf, int len, int filechans)
+static int st_mcompand_flow_1(compand_t c, comp_band_t l, const st_sample_t *ibuf, st_sample_t *obuf, int len, int filechans)
 {
   int done, chan;
 
@@ -542,9 +508,9 @@ static int st_mcompand_flow_1(compand_t c, comp_band_t l, st_sample_t *ibuf, st_
       if (c->delay_buf_size <= 0)
         obuf[done++] = ibuf[chan]*(outv/v)*l->outgain;
       else {
-
-
-        /* note that this lookahead algorithm is really lame.  the response to a peak is released before the peak arrives.  fix! */
+        /* FIXME: note that this lookahead algorithm is really lame:
+           the response to a peak is released before the peak
+           arrives. */
 
         /* because volume application delays differ band to band, but
            total delay doesn't, the volume is applied in an iteration
@@ -574,44 +540,44 @@ static int st_mcompand_flow_1(compand_t c, comp_band_t l, st_sample_t *ibuf, st_
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_mcompand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_mcompand_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                      st_size_t *isamp, st_size_t *osamp) {
   compand_t c = (compand_t) effp->priv;
   comp_band_t l;
-  int len = ((*isamp > *osamp) ? *osamp : *isamp);
+  int len = min(*isamp, *osamp);
   int band, i;
-  st_sample_t *abuf, *bbuf, *cbuf, *oldabuf;
+  st_sample_t *abuf, *bbuf, *cbuf, *oldabuf, *ibuf_copy;
   double out;
 
   if (c->band_buf_len < len) {
-    if ((! (c->band_buf1 = (st_sample_t *)realloc(c->band_buf1,len*sizeof(st_sample_t)))) ||
-        (! (c->band_buf2 = (st_sample_t *)realloc(c->band_buf2,len*sizeof(st_sample_t)))) ||
-        (! (c->band_buf3 = (st_sample_t *)realloc(c->band_buf3,len*sizeof(st_sample_t))))) {
-      st_fail("Out of memory");
-      return (ST_EOF);
-    }
+    c->band_buf1 = (st_sample_t *)xrealloc(c->band_buf1,len*sizeof(st_sample_t));
+    c->band_buf2 = (st_sample_t *)xrealloc(c->band_buf2,len*sizeof(st_sample_t));
+    c->band_buf3 = (st_sample_t *)xrealloc(c->band_buf3,len*sizeof(st_sample_t));
     c->band_buf_len = len;
   }
 
+  ibuf_copy = (st_sample_t *)xmalloc(*isamp * sizeof(st_sample_t));
+  memcpy(ibuf_copy, ibuf, *isamp * sizeof(st_sample_t));
+
   /* split ibuf into bands using butterworths, pipe each band through st_mcompand_flow_1, then add back together and write to obuf */
 
   memset(obuf,0,len * sizeof *obuf);
-  for (band=0,abuf=ibuf,bbuf=c->band_buf2,cbuf=c->band_buf1;band<c->nBands;++band) {
+  for (band=0,abuf=ibuf_copy,bbuf=c->band_buf2,cbuf=c->band_buf1;band<c->nBands;++band) {
     l = &c->bands[band];
 
     if (l->topfreq)
-      lowpass_flow(&l->filter, effp->outinfo.channels, abuf, bbuf, cbuf, len);
+      lowpass_flow(effp, &l->filter, effp->outinfo.channels, abuf, bbuf, cbuf, len);
     else {
       bbuf = abuf;
       abuf = cbuf;
     }
-    if (abuf == ibuf)
+    if (abuf == ibuf_copy)
       abuf = c->band_buf3;
     (void)st_mcompand_flow_1(c,l,bbuf,abuf,len,effp->outinfo.channels);
     for (i=0;i<len;++i)
     {
       out = obuf[i] + abuf[i];
-      ST_SAMPLE_CLIP(out);
+      ST_SAMPLE_CLIP_COUNT(out, effp->clips);
       obuf[i] = out;
     }
     oldabuf = abuf;
@@ -621,10 +587,12 @@ int st_mcompand_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 
   *isamp = *osamp = len;
 
+  free(ibuf_copy);
+
   return ST_SUCCESS;
 }
 
-static int st_mcompand_drain_1(compand_t c, comp_band_t l, st_sample_t *obuf, int maxdrain, int band)
+static int st_mcompand_drain_1(eff_t effp, compand_t c, comp_band_t l, st_sample_t *obuf, int maxdrain)
 {
   int done;
   double out;
@@ -634,7 +602,7 @@ static int st_mcompand_drain_1(compand_t c, comp_band_t l, st_sample_t *obuf, in
    */
   for (done = 0;  done < maxdrain  &&  l->delay_buf_cnt > 0;  done++) {
     out = obuf[done] + l->delay_buf[l->delay_buf_ptr++];
-    ST_SAMPLE_CLIP(out);
+    ST_SAMPLE_CLIP_COUNT(out, effp->clips);
     obuf[done] = out;
     l->delay_buf_ptr %= c->delay_buf_size;
     l->delay_buf_cnt--;
@@ -648,7 +616,7 @@ static int st_mcompand_drain_1(compand_t c, comp_band_t l, st_sample_t *obuf, in
 /*
  * Drain out compander delay lines. 
  */
-int st_mcompand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_mcompand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
   int band, drained, mostdrained = 0;
   compand_t c = (compand_t)effp->priv;
@@ -657,7 +625,7 @@ int st_mcompand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
   memset(obuf,0,*osamp * sizeof *obuf);
   for (band=0;band<c->nBands;++band) {
     l = &c->bands[band];
-    drained = st_mcompand_drain_1(c,l,obuf,*osamp,0);
+    drained = st_mcompand_drain_1(effp, c,l,obuf,*osamp);
     if (drained > mostdrained)
       mostdrained = drained;
   }
@@ -673,43 +641,49 @@ int st_mcompand_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 /*
  * Clean up compander effect.
  */
-int st_mcompand_stop(eff_t effp)
+static int st_mcompand_stop(eff_t effp)
 {
   compand_t c = (compand_t) effp->priv;
   comp_band_t l;
   int band;
 
-  if (c->band_buf1) {
-    free(c->band_buf1);
-    c->band_buf1 = 0;
-  }
-  if (c->band_buf2) {
-    free(c->band_buf2);
-    c->band_buf2 = 0;
-  }
-  if (c->band_buf3) {
-    free(c->band_buf3);
-    c->band_buf3 = 0;
-  }
+  free(c->band_buf1);
+  c->band_buf1 = NULL;
+  free(c->band_buf2);
+  c->band_buf2 = NULL;
+  free(c->band_buf3);
+  c->band_buf3 = NULL;
 
-  for (band=0;band<c->nBands;++band) {
+  for (band = 0; band < c->nBands; band++) {
     l = &c->bands[band];
-    free((char *) l->transferOuts);
-    free((char *) l->transferIns);
-    free((char *) l->decayRate);
-    free((char *) l->attackRate);
-    if (l->delay_buf)
-      free((char *) l->delay_buf);
-    free((char *) l->volume);
+    free(l->delay_buf);
     if (l->topfreq != 0) {
       free(l->filter.xy_low);
       free(l->filter.xy_high);
     }
   }
-  free(c->bands);
-  c->bands = 0;
 
-  return (ST_SUCCESS);
+  return ST_SUCCESS;
+}
+
+static int st_mcompand_delete(eff_t effp)
+{
+  compand_t c = (compand_t) effp->priv;
+  comp_band_t l;
+  int band;
+
+  for (band = 0; band < c->nBands; band++) {
+    l = &c->bands[band];
+    free(l->transferOuts);
+    free(l->transferIns);
+    free(l->decayRate);
+    free(l->attackRate);
+    free(l->volume);
+  }
+  free(c->bands);
+  c->bands = NULL;
+
+  return ST_SUCCESS;
 }
 
 static st_effect_t st_mcompand_effect = {
@@ -726,7 +700,8 @@ static st_effect_t st_mcompand_effect = {
   st_mcompand_start,
   st_mcompand_flow,
   st_mcompand_drain,
-  st_mcompand_stop
+  st_mcompand_stop,
+  st_mcompand_delete
 };
 
 const st_effect_t *st_mcompand_effect_fn(void)
