@@ -38,7 +38,7 @@ typedef struct reddata {
  * Get the options. Filename is mandatory, though a reasonable default would
  * be stdin (if the input file isn't coming from there, of course!)
  */
-int st_noisered_getopts(eff_t effp, int n, char **argv) 
+static int st_noisered_getopts(eff_t effp, int n, char **argv) 
 {
     reddata_t data = (reddata_t) effp->priv;
 
@@ -67,25 +67,28 @@ int st_noisered_getopts(eff_t effp, int n, char **argv)
  * Prepare processing.
  * Do all initializations.
  */
-int st_noisered_start(eff_t effp)
+static int st_noisered_start(eff_t effp)
 {
     reddata_t data = (reddata_t) effp->priv;
     int fchannels = 0;
     int channels = effp->ininfo.channels;
     int i;
-    FILE* ifd;
+    FILE* ifp;
 
-    data->chandata = (chandata_t*)calloc(channels, sizeof(*(data->chandata)));
+    data->chandata = (chandata_t*)xcalloc(channels, sizeof(*(data->chandata)));
+    data->bufdata = 0;
     for (i = 0; i < channels; i ++) {
-        data->chandata[i].noisegate = (float*)calloc(FREQCOUNT, sizeof(float));
-        data->chandata[i].smoothing = (float*)calloc(FREQCOUNT, sizeof(float));
+        data->chandata[i].noisegate = (float*)xcalloc(FREQCOUNT, sizeof(float));
+        data->chandata[i].smoothing = (float*)xcalloc(FREQCOUNT, sizeof(float));
         data->chandata[i].lastwindow = NULL;
     }
-    data->bufdata = 0;
 
     /* Here we actually open the input file. */
-    ifd = fopen(data->profile_filename, "r");
-    if (ifd == NULL) {
+    if (strcmp(data->profile_filename, "-") == 0)
+      ifp = stdin;
+    else
+      ifp = fopen(data->profile_filename, "r");
+    if (ifp == NULL) {
         st_fail("Couldn't open profile file %s: %s",
                 data->profile_filename, strerror(errno));            
         return ST_EOF;
@@ -94,7 +97,7 @@ int st_noisered_start(eff_t effp)
     while (1) {
         int i1;
         float f1;
-        if (2 != fscanf(ifd, " Channel %d: %f", &i1, &f1))
+        if (2 != fscanf(ifp, " Channel %d: %f", &i1, &f1))
             break;
         if (i1 != fchannels) {
             st_fail("noisered: Got channel %d, expected channel %d.",
@@ -104,7 +107,7 @@ int st_noisered_start(eff_t effp)
 
         data->chandata[fchannels].noisegate[0] = f1;
         for (i = 1; i < FREQCOUNT; i ++) {
-            if (1 != fscanf(ifd, ", %f", &f1)) {
+            if (1 != fscanf(ifp, ", %f", &f1)) {
                 st_fail("noisered: Not enough datums for channel %d "
                         "(expected %d, got %d)", fchannels, FREQCOUNT, i);
                 return ST_EOF;
@@ -114,11 +117,12 @@ int st_noisered_start(eff_t effp)
         fchannels ++;
     }
     if (fchannels != channels) {
-        st_fail("noisered: channel mismatch: %d in input, %d in profile.\n",
+        st_fail("noisered: channel mismatch: %d in input, %d in profile.",
                 channels, fchannels);
         return ST_EOF;
     }
-    fclose(ifd);
+    if (strcmp(data->profile_filename, "-") != 0)
+      fclose(ifp);
 
     return (ST_SUCCESS);
 }
@@ -128,20 +132,18 @@ int st_noisered_start(eff_t effp)
  * due to overlapping windows. */
 static void reduce_noise(chandata_t* chan, float* window, float level)
 {
-    float *inr   = (float*)calloc(WINDOWSIZE, sizeof(float));
-    float *ini   = (float*)calloc(WINDOWSIZE, sizeof(float));
-    float *outr  = (float*)calloc(WINDOWSIZE, sizeof(float));
-    float *outi  = (float*)calloc(WINDOWSIZE, sizeof(float));
-    float *power = (float*)calloc(WINDOWSIZE, sizeof(float));
+    float *inr, *ini, *outr, *outi, *power;
     float *smoothing = chan->smoothing;
-    static int callnum = 0;
     int i;
 
-    callnum ++;
-
-    for (i = 0; i < FREQCOUNT; i ++) {
+    inr = (float*)xcalloc(WINDOWSIZE * 5, sizeof(float));
+    ini = inr + WINDOWSIZE;
+    outr = ini + WINDOWSIZE;
+    outi = outr + WINDOWSIZE;
+    power = outi + WINDOWSIZE;
+    
+    for (i = 0; i < FREQCOUNT; i ++)
         assert(smoothing[i] >= 0 && smoothing[i] <= 1);
-    }
 
     memcpy(inr, window, WINDOWSIZE*sizeof(float));
 
@@ -151,7 +153,7 @@ static void reduce_noise(chandata_t* chan, float* window, float level)
     WindowFunc(HANNING, WINDOWSIZE, inr);
     PowerSpectrum(WINDOWSIZE, inr, power);
 
-    for(i = 0; i < FREQCOUNT; i ++) {
+    for (i = 0; i < FREQCOUNT; i ++) {
         float smooth;
         float plog;
         plog = log(power[i]);
@@ -159,10 +161,10 @@ static void reduce_noise(chandata_t* chan, float* window, float level)
             smooth = 0.0;
         else
             smooth = 1.0;
-
+        
         smoothing[i] = smooth * 0.5 + smoothing[i] * 0.5;
     }
-
+    
     /* Audacity says this code will eliminate tinkle bells.
      * I have no idea what that means. */
     for (i = 2; i < FREQCOUNT - 2; i ++) {
@@ -174,7 +176,7 @@ static void reduce_noise(chandata_t* chan, float* window, float level)
             smoothing[i+2]<0.1)
             smoothing[i] = 0.0;
     }
-
+    
     outr[0] *= smoothing[0];
     outi[0] *= smoothing[0];
     outr[FREQCOUNT-1] *= smoothing[FREQCOUNT-1];
@@ -183,137 +185,122 @@ static void reduce_noise(chandata_t* chan, float* window, float level)
     for (i = 1; i < FREQCOUNT-1; i ++) {
         int j = WINDOWSIZE - i;
         float smooth = smoothing[i];
-
+        
         outr[i] *= smooth;
         outi[i] *= smooth;
         outr[j] *= smooth;
         outi[j] *= smooth;
     }
-
+    
     FFT(WINDOWSIZE, 1, outr, outi, inr, ini);
     WindowFunc(HANNING, WINDOWSIZE, inr);
-
+    
     memcpy(window, inr, WINDOWSIZE*sizeof(float));
 
-    free(inr);
-    free(ini);
-    free(outr);
-    free(outi);
-    free(power);
-
-    for (i = 0; i < FREQCOUNT; i ++) {
+    for (i = 0; i < FREQCOUNT; i ++)
         assert(smoothing[i] >= 0 && smoothing[i] <= 1);
-    }
+
+    free(inr);
 }
 
 /* Do window management once we have a complete window, including mangling
  * the current window. */
-static int process_window(reddata_t data, int chan_num, int num_chans,
+static int process_window(eff_t effp, reddata_t data, int chan_num, int num_chans,
                           st_sample_t *obuf, int len) {
     int j;
     float* nextwindow;
     int use = min(len, WINDOWSIZE)-min(len,(WINDOWSIZE/2));
     chandata_t *chan = &(data->chandata[chan_num]);
     int first = (chan->lastwindow == NULL);
-    int clipped = 0;
 
-    nextwindow = (float*)calloc(WINDOWSIZE, sizeof(float));
+    if ((nextwindow = (float*)xcalloc(WINDOWSIZE, sizeof(float))) == NULL)
+        return ST_EOF;
+    
     memcpy(nextwindow, chan->window+WINDOWSIZE/2,
            sizeof(float)*(WINDOWSIZE/2));
 
     reduce_noise(chan, chan->window, data->threshold);
-        
     if (!first) {
         for (j = 0; j < use; j ++) {
             float s = chan->window[j] + chan->lastwindow[WINDOWSIZE/2 + j];
-            ST_NORMALIZED_CLIP_COUNT(s, clipped);
-            if (clipped)
-            {
-                /* Reset for future tests. */
-                clipped = 0;
-                st_warn("noisered: Output clipped to %f.\n", s);
-            }
             obuf[chan_num + num_chans * j] =
-                ST_FLOAT_DWORD_TO_SAMPLE(s);
+                ST_FLOAT_DWORD_TO_SAMPLE(s, effp->clips);
         }
         free(chan->lastwindow);
     } else {
         for (j = 0; j < use; j ++) {
             assert(chan->window[j] >= -1 && chan->window[j] <= 1);
             obuf[chan_num + num_chans * j] =
-                ST_FLOAT_DWORD_TO_SAMPLE(chan->window[j]);
+                ST_FLOAT_DWORD_TO_SAMPLE(chan->window[j], effp->clips);
         }
     }
     chan->lastwindow = chan->window;
     chan->window = nextwindow;
-
+    
     return use;
 }
 
 /*
  * Read in windows, and call process_window once we get a whole one.
  */
-int st_noisered_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_noisered_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                     st_size_t *isamp, st_size_t *osamp)
 {
     reddata_t data = (reddata_t) effp->priv;
-    int samp = min(*isamp, *osamp);
-    int tracks = effp->ininfo.channels;
-    int track_samples = samp / tracks;
-    int ncopy = min(track_samples, WINDOWSIZE-data->bufdata);
-    int whole_window = (ncopy + data->bufdata == WINDOWSIZE);
+    st_size_t samp = min(*isamp, *osamp);
+    st_size_t tracks = effp->ininfo.channels;
+    st_size_t track_samples = samp / tracks;
+    st_size_t ncopy = min(track_samples, WINDOWSIZE-data->bufdata);
+    st_size_t whole_window = (ncopy + data->bufdata == WINDOWSIZE);
     int oldbuf = data->bufdata;
-    int i;
+    st_size_t i;
+
     assert(effp->ininfo.channels == effp->outinfo.channels);
 
-    if (whole_window) {
+    if (whole_window)
         data->bufdata = WINDOWSIZE/2;
-    } else {
+    else
         data->bufdata += ncopy;
-    }
 
     /* Reduce noise on every channel. */
     for (i = 0; i < tracks; i ++) {
         chandata_t* chan = &(data->chandata[i]);
-        int j;
-        if (chan->window == NULL) {
-            chan->window = (float*)calloc(WINDOWSIZE, sizeof(float));
-        }
+        st_size_t j;
+
+        if (chan->window == NULL)
+            chan->window = (float*)xcalloc(WINDOWSIZE, sizeof(float));
         
-        for (j = 0; j < ncopy; j ++) {
+        for (j = 0; j < ncopy; j ++)
             chan->window[oldbuf + j] =
-                ST_SAMPLE_TO_FLOAT_DWORD(ibuf[i + tracks * j]);
-        }
+                ST_SAMPLE_TO_FLOAT_DWORD(ibuf[i + tracks * j], effp->clips);
 
         if (!whole_window)
             continue;
-        else {
-            process_window(data, i, tracks, obuf, oldbuf + ncopy);
-        }
+        else
+            process_window(effp, data, i, tracks, obuf, oldbuf + ncopy);
     }
     
     *isamp = tracks*ncopy;
-    if (whole_window) {
+    if (whole_window)
         *osamp = tracks*(WINDOWSIZE/2);
-    } else {
+    else
         *osamp = 0;
-    }
-    
-    return (ST_SUCCESS);
+
+    return ST_SUCCESS;
 }
 
 /*
  * We have up to half a window left to dump.
  */
 
-int st_noisered_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_noisered_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
     reddata_t data = (reddata_t)effp->priv;
     int i;
     int tracks = effp->ininfo.channels;
-    for (i = 0; i < tracks; i ++) {
-        *osamp = process_window(data, i, tracks, obuf, data->bufdata);
-    }
+    for (i = 0; i < tracks; i ++)
+        *osamp = process_window(effp, data, i, tracks, obuf, data->bufdata);
+
     /* FIXME: This is very picky.  osamp needs to be big enough to get all
      * remaining data or it will be discarded.
      */
@@ -323,19 +310,15 @@ int st_noisered_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 /*
  * Clean up.
  */
-int st_noisered_stop(eff_t effp)
+static int st_noisered_stop(eff_t effp)
 {
     reddata_t data = (reddata_t) effp->priv;
-    int i;
+    st_size_t i;
 
     for (i = 0; i < effp->ininfo.channels; i ++) {
         chandata_t* chan = &(data->chandata[i]);
-        if (chan->lastwindow != NULL) {
-            free(chan->lastwindow);
-        }
-        if (chan->window != NULL) {
-            free(chan->window);
-        }
+        free(chan->lastwindow);
+        free(chan->window);
         free(chan->smoothing);
         free(chan->noisegate);
     }
@@ -353,15 +336,11 @@ static st_effect_t st_noisered_effect = {
   st_noisered_start,
   st_noisered_flow,
   st_noisered_drain,
-  st_noisered_stop
+  st_noisered_stop,
+  st_effect_nothing
 };
 
 const st_effect_t *st_noisered_effect_fn(void)
 {
     return &st_noisered_effect;
 }
-
-/* For Emacs:
-  Local Variables:
-  c-basic-offset: 4
-*/
