@@ -1,4 +1,6 @@
 /*
+ * Sound Tools rate change effect file.
+ *
  * July 14, 1998
  * Copyright 1998  K. Bradley, Carnegie Mellon University
  *
@@ -16,18 +18,11 @@
  *   License along with this library; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- */
-
-/*
  * October 29, 1999
  * Various changes, bugfixes, speedups, by Stan Brooks.
  *
  */
 
-/*
- * Sound Tools rate change effect file.
- */
-
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +34,7 @@
 #define MF 30
 
 typedef struct {
-  int up,down;    /* up/down conversion factors for this stage      */
+  int up, down;   /* up/down conversion factors for this stage      */
   int filt_len;   /* # coefficients in filter_array                 */
   Float *filt_array; /* filter coefficients                         */
   int held;       /* # samples held in input but not yet processed  */
@@ -49,7 +44,6 @@ typedef struct {
 } polystage;
 
 typedef struct polyphase {
-
   st_rate_t lcmrate;             /* least common multiple of rates */
   st_rate_t inskip, outskip;     /* LCM increments for I & O rates */
   double Factor;                 /* out_rate/in_rate               */
@@ -57,51 +51,33 @@ typedef struct polyphase {
   st_size_t oskip;               /* output samples to skip at start*/
   double inpipe;                 /* output samples 'in the pipe'   */
   polystage *stage[MF];          /* array of pointers to polystage structs */
-
+  int win_type;
+  int win_width;
+  Float cutoff;
+  int m1[MF], m2[MF], b1[MF], b2[MF]; /* arrays used in optimize_factors */
 } *poly_t;
 
+assert_static(sizeof(struct polyphase) <= ST_MAX_EFFECT_PRIVSIZE, 
+              /* else */ skeleff_PRIVSIZE_too_big);
+
 /*
  * Process options
  */
-
-/* Options:
-
-   -w <nut / ham>        :  window type
-   -width <short / long> :  window width
-                            short = 128 samples
-                            long  = 1024 samples
-   <num>                    num:  explicit number
-
-   -cutoff <float>       :  frequency cutoff for base bandwidth.
-                            Default = 0.95 = 95%
-*/
-
-static int win_type  = 0;
-static int win_width = 1024;
-static Float cutoff = 0.95;
-
-int st_poly_getopts(eff_t effp, int n, char **argv)
+static int st_poly_getopts(eff_t effp, int n, char **argv)
 {
-  /* 0: nuttall
-     1: hamming */
-  win_type = 0;
+  poly_t rate = (poly_t) effp->priv;
 
-  /* width:  short = 128
-             long = 1024 (default) */
-  win_width = 1024;
-
-  /* cutoff:  frequency cutoff of base bandwidth in percentage. */
-  cutoff = 0.95;
-
-  while(n >= 2) {
+  rate->win_type = 0;           /* 0: nuttall, 1: hamming */
+  rate->win_width = 1024;
+  rate->cutoff = 0.95;
 
+  while (n >= 2) {
     /* Window type check */
     if(!strcmp(argv[0], "-w")) {
       if(!strcmp(argv[1], "ham"))
-        win_type = 1;
+        rate->win_type = 1;
       if(!strcmp(argv[1], "nut"))
-        win_type = 0;
-
+        rate->win_type = 0;
       argv += 2;
       n -= 2;
       continue;
@@ -109,13 +85,7 @@ int st_poly_getopts(eff_t effp, int n, char **argv)
 
     /* Window width check */
     if(!strcmp(argv[0], "-width")) {
-      if(!strcmp(argv[1], "short"))
-        win_width = 128;
-      else if(!strcmp(argv[1], "long"))
-        win_width = 1024;
-      else
-        win_width = atoi(argv[1]);
-
+        rate->win_width = atoi(argv[1]);
       argv += 2;
       n -= 2;
       continue;
@@ -123,16 +93,17 @@ int st_poly_getopts(eff_t effp, int n, char **argv)
 
     /* Cutoff frequency check */
     if(!strcmp(argv[0], "-cutoff")) {
-      cutoff = atof(argv[1]);
+      rate->cutoff = atof(argv[1]);
       argv += 2;
       n -= 2;
       continue;
     }
 
     st_fail("Polyphase: unknown argument (%s %s)!", argv[0], argv[1]);
-    return (ST_EOF);
+    return ST_EOF;
   }
-  return (ST_SUCCESS);
+  
+  return ST_SUCCESS;
 }
 
 /*
@@ -156,14 +127,6 @@ static const unsigned short primes[] = {
   0
 };
 
-#ifndef max
-#define max(x,y) ((x > y) ? x : y)
-#endif
-
-#ifndef min
-#define min(x,y) ((x < y) ? x : y)
-#endif
-
 static int prime(int n, int *q0)
 {
   const unsigned short *p;
@@ -171,25 +134,25 @@ static int prime(int n, int *q0)
 
   p = primes;
   q = q0;
-  st_report("factors(%d) =",n);
+  st_debug("factors(%d) =",n);
   while (n > 1) {
     while ((pr = *p) && (n % pr)) p++;
     if (!pr) {
-      st_fail("Number %d too large of a prime.\n",n);
+      st_fail("Number %d too large of a prime.",n);
       pr = n;
     }
     *q++ = pr;
     n /= pr;
   }
   *q = 0;
-  for (pr=0; pr<q-q0; pr++) st_report(" %d",q0[pr]);
-  st_report("\n");
+  for (pr=0; pr<q-q0; pr++) st_debug(" %d",q0[pr]);
+  st_debug("");
   return (q-q0);
 }
 
-static int permute(int *m, int *l, int ct, int ct1, int amalg)
+static int permute(int *m, int *l, int ct, int ct1, size_t amalg)
 {
-  int k, n;
+  size_t k, n;
   int *p;
   int *q;
 
@@ -201,7 +164,7 @@ static int permute(int *m, int *l, int ct, int ct1, int amalg)
   for (k=ct; k>1; ) {
     int tmp;
     unsigned long j;
-    j = (rand()%32768L) + ((rand()%32768L)<<13); /* reasonably big */
+    j = (rand()%32768) + ((rand()%32768)<<13); /* reasonably big */
     j = j % k; /* non-negative! */
     k--;
     if (j != k) {
@@ -221,23 +184,21 @@ static int permute(int *m, int *l, int ct, int ct1, int amalg)
   }
   if (n) *p++=n;
   *p = 0;
-  /*for (k=0; k<p-m; k++) st_report(" %d",m[k]);*/
-  /*st_report("\n");*/
+  /*for (k=0; k<p-m; k++) st_debug(" %d",m[k]);*/
+  /*st_debug("");*/
   return (p-m);
 }
 
-static int optimize_factors(int numer, int denom, int *l1, int *l2)
+static int optimize_factors(poly_t rate, int numer, int denom, int *l1, int *l2)
 {
   int f_min,c_min,u_min,ct1,ct2;
-  int amalg;
+  size_t amalg;
   int k;
-  static int m1[MF],m2[MF];
-  static int b1[MF],b2[MF];
 
-  memset(m1,0,sizeof(int)*MF);
-  memset(m2,0,sizeof(int)*MF);
-  memset(b1,0,sizeof(int)*MF);
-  memset(b2,0,sizeof(int)*MF);
+  memset(rate->m1,0,sizeof(int)*MF);
+  memset(rate->m2,0,sizeof(int)*MF);
+  memset(rate->b1,0,sizeof(int)*MF);
+  memset(rate->b2,0,sizeof(int)*MF);
 
   f_min = numer; if (f_min>denom) f_min = denom;
   c_min = 1<<30;
@@ -247,35 +208,35 @@ static int optimize_factors(int numer, int denom, int *l1, int *l2)
   ct1 = prime(numer,l1);
   ct2 = prime(denom,l2);
 
-  for (amalg = max(9,l2[0]); amalg<= 9+l2[ct2-1]; amalg++) {
+  for (amalg = max(9,l2[0]); amalg <= (size_t)(9+l2[ct2-1]); amalg++) {
     for (k = 0; k<100000; k++) {
       int u,u1,u2,j,f,cost;
       cost = 0;
       f = denom;
       u = min(ct1,ct2) + 1;
-      /*st_report("pfacts(%d): ", numer);*/
-      u1 = permute(m1,l1,ct1,u,amalg);
-      /*st_report("pfacts(%d): ", denom);*/
-      u2 = permute(m2,l2,ct2,u,amalg);
+      /*st_debug("pfacts(%d): ", numer);*/
+      u1 = permute(rate->m1,l1,ct1,u,amalg);
+      /*st_debug("pfacts(%d): ", denom);*/
+      u2 = permute(rate->m2,l2,ct2,u,amalg);
       u = max(u1,u2);
       for (j=0; j<u; j++) {
-        if (j>=u1) m1[j]=1;
-        if (j>=u2) m2[j]=1;
-        f = (f * m1[j])/m2[j];
+        if (j>=u1) rate->m1[j]=1;
+        if (j>=u2) rate->m2[j]=1;
+        f = (f * rate->m1[j])/rate->m2[j];
         if (f < f_min) goto fail;
-        cost += f + m1[j]*m2[j];
+        cost += f + rate->m1[j]*rate->m2[j];
       }
       if (c_min>cost) {
         c_min = cost;
         u_min = u;
-#       if 0
-        st_report("c_min %d, [%d-%d]:",c_min,numer,denom);
-        for (j=0; j<u; j++)
-          st_report(" (%d,%d)",m1[j],m2[j]);
-        st_report("\n");
-#       endif
-       memcpy(b1,m1,u*sizeof(int));
-       memcpy(b2,m2,u*sizeof(int));
+        if (st_output_verbosity_level >= 4) {
+          st_debug("c_min %d, [%d-%d]:",c_min,numer,denom);
+          for (j=0; j<u; j++)
+            st_debug(" (%d,%d)",rate->m1[j],rate->m2[j]);
+          st_debug("");
+        }
+        memcpy(rate->b1,rate->m1,u*sizeof(int));
+        memcpy(rate->b2,rate->m2,u*sizeof(int));
       }
      fail:
         ;;
@@ -283,18 +244,14 @@ static int optimize_factors(int numer, int denom, int *l1, int *l2)
     if (u_min) break;
   }
   if (u_min) {
-    memcpy(l1,b1,u_min*sizeof(int));
-    memcpy(l2,b2,u_min*sizeof(int));
+    memcpy(l1,rate->b1,u_min*sizeof(int));
+    memcpy(l2,rate->b2,u_min*sizeof(int));
   }
   l1[u_min] = 0;
   l2[u_min] = 0;
   return u_min;
 }
 
-#ifndef PI
-#define PI 3.14159265358979
-#endif
-
 /* Calculate a Nuttall window of a given length.
    Buffer must already be allocated to appropriate size.
    */
@@ -306,7 +263,7 @@ static void nuttall(Float *buffer, int length)
   int N1;
 
   if(buffer == NULL || length <= 0)
-    st_fail("Illegal buffer %p or length %d to nuttall.\n", buffer, length);
+    st_fail("Illegal buffer %p or length %d to nuttall.", buffer, length);
 
   /* Initial variable setups. */
   N = length;
@@ -314,9 +271,9 @@ static void nuttall(Float *buffer, int length)
 
   for(j = 0; j < length; j++) {
     buffer[j] = 0.3635819 +
-      0.4891775 * cos(2*PI*1*(j - N1) / N) +
-      0.1365995 * cos(2*PI*2*(j - N1) / N) +
-      0.0106411 * cos(2*PI*3*(j - N1) / N);
+      0.4891775 * cos(2*M_PI*1*(j - N1) / N) +
+      0.1365995 * cos(2*M_PI*2*(j - N1) / N) +
+      0.0106411 * cos(2*M_PI*3*(j - N1) / N);
   }
 }
 /* Calculate a Hamming window of given length.
@@ -329,11 +286,11 @@ static void hamming(Float *buffer, int length)
     int N1;
 
     if(buffer == NULL || length <= 0)
-      st_fail("Illegal buffer %p or length %d to hamming.\n",buffer,length);
+      st_fail("Illegal buffer %p or length %d to hamming.",buffer,length);
 
     N1 = length/2;
     for(j=0;j<length;j++)
-      buffer[j] = 0.5 - 0.46 * cos(PI*j/N1);
+      buffer[j] = 0.5 - 0.46 * cos(M_PI*j/N1);
 }
 
 /* Calculate the sinc function properly */
@@ -349,26 +306,26 @@ static Float sinc(Float value)
 
    buffer must already be allocated.
 */
-static void fir_design(Float *buffer, int length, Float cutoff)
+static void fir_design(poly_t rate, Float *buffer, int length, Float cutoff)
 {
     int j;
     double sum;
 
-    if(buffer == NULL || length < 0 || cutoff < 0 || cutoff > PI)
-      st_fail("Illegal buffer %p, length %d, or cutoff %f.\n",buffer,length,cutoff);
+    if(buffer == NULL || length < 0 || cutoff < 0 || cutoff > M_PI)
+      st_fail("Illegal buffer %p, length %d, or cutoff %f.",buffer,length,cutoff);
 
     /* Use the user-option of window type */
-    if(win_type == 0)
+    if (rate->win_type == 0)
       nuttall(buffer, length); /* Design Nuttall window:  ** dB cutoff */
     else
       hamming(buffer,length);  /* Design Hamming window:  43 dB cutoff */
 
-    /* st_report("# fir_design length=%d, cutoff=%8.4f\n",length,cutoff); */
+    /* st_debug("# fir_design length=%d, cutoff=%8.4f",length,cutoff); */
     /* Design filter:  windowed sinc function */
     sum = 0.0;
     for(j=0;j<length;j++) {
-      buffer[j] *= sinc(PI*cutoff*(j-length/2)); /* center at length/2 */
-      /* st_report("%.1f %.6f\n",(float)j,buffer[j]); */
+      buffer[j] *= sinc(M_PI*cutoff*(j-length/2)); /* center at length/2 */
+      /* st_debug("%.1f %.6f",(float)j,buffer[j]); */
       sum += buffer[j];
     }
     sum = (double)1.0/sum;
@@ -376,12 +333,12 @@ static void fir_design(Float *buffer, int length, Float cutoff)
     for(j=0;j<length;j++) {
       buffer[j] *= sum;
     }
-    /* st_report("# end\n\n"); */
+    /* st_debug("# end"); */
 }
 
 #define RIBLEN 2048
 
-int st_poly_start(eff_t effp)
+static int st_poly_start(eff_t effp)
 {
     poly_t rate = (poly_t) effp->priv;
     static int l1[MF], l2[MF];
@@ -390,18 +347,13 @@ int st_poly_start(eff_t effp)
     int k;
 
     if (effp->ininfo.rate == effp->outinfo.rate)
-    {
-        st_fail("Input and Output rate must not be the same to use polyphase effect");
-        return(ST_EOF);
-    }
-
-    st_initrand();
+      return ST_EFF_NULL;
 
     rate->lcmrate = st_lcm((st_sample_t)effp->ininfo.rate,
                            (st_sample_t)effp->outinfo.rate);
 
     /* Cursory check for LCM overflow.
-     * If both rate are below 65k, there should be no problem.
+     * If both rates are below 65k, there should be no problem.
      * 16 bits x 16 bits = 32 bits, which we can handle.
      */
 
@@ -416,14 +368,14 @@ int st_poly_start(eff_t effp)
     }
 
     /* Find the prime factors of inskip and outskip */
-    total = optimize_factors(rate->inskip, rate->outskip, l1, l2);
+    total = optimize_factors(rate, rate->inskip, rate->outskip, l1, l2);
     rate->total = total;
     /* l1 and l2 are now lists of the up/down factors for conversion */
 
-    st_report("Poly:  input rate %d, output rate %d.  %d stages.",
+    st_debug("Poly:  input rate %d, output rate %d.  %d stages.",
             effp->ininfo.rate, effp->outinfo.rate,total);
-    st_report("Poly:  window: %s  size: %d  cutoff: %f.",
-            (win_type == 0) ? ("nut") : ("ham"), win_width, cutoff);
+    st_debug("Poly:  window: %s  size: %d  cutoff: %f.",
+            (rate->win_type == 0) ? ("nut") : ("ham"), rate->win_width, rate->cutoff);
 
     /* Create an array of filters and past history */
     uprate = effp->ininfo.rate;
@@ -431,52 +383,51 @@ int st_poly_start(eff_t effp)
       int j, prod, f_cutoff, f_len;
       polystage *s;
 
-      rate->stage[k] = s = (polystage*) malloc(sizeof(polystage));
+      rate->stage[k] = s = (polystage*) xmalloc(sizeof(polystage));
       s->up = l1[k];
       s->down = l2[k];
       f_cutoff = max(s->up, s->down);
-      f_len = max(20 * f_cutoff, win_width);
+      f_len = max(20 * f_cutoff, rate->win_width);
       prod = s->up * s->down;
       if (prod > 2*f_len) prod = s->up;
       f_len = ((f_len+prod-1)/prod) * prod; /* reduces rounding-errors in polyphase() */
       s->size = size;
       s->hsize = f_len/s->up; /* this much of window is past-history */
       s->held = 0;
-      st_report("Poly:  stage %d:  Up by %d, down by %d,  i_samps %d, hsize %d",
+      st_debug("Poly:  stage %d:  Up by %d, down by %d,  i_samps %d, hsize %d",
               k+1,s->up,s->down,size, s->hsize);
       s->filt_len = f_len;
-      s->filt_array = (Float *) malloc(sizeof(Float) * f_len);
-      s->window = (Float *) malloc(sizeof(Float) * (s->hsize+size));
+      s->filt_array = (Float *) xmalloc(sizeof(Float) * f_len);
+      s->window = (Float *) xmalloc(sizeof(Float) * (s->hsize+size));
       /* zero past_history section of window */
       for(j = 0; j < s->hsize; j++)
         s->window[j] = 0.0;
 
       uprate *= s->up;
-      st_report("Poly:         :  filt_len %d, cutoff freq %.1f",
-              f_len, uprate*cutoff/f_cutoff);
+      st_debug("Poly:         :  filt_len %d, cutoff freq %.1f",
+              f_len, uprate * rate->cutoff / f_cutoff);
       uprate /= s->down;
-      fir_design(s->filt_array, f_len, cutoff/f_cutoff);
-      /* s->filt_array[f_len-1]=0; */
+      fir_design(rate, s->filt_array, f_len, rate->cutoff / f_cutoff);
 
-                        skip *= s->up;
-                        skip += f_len;
-                        skip /= s->down;
-
-                        size = (size * s->up) / s->down;  /* this is integer */
+      skip *= s->up;
+      skip += f_len;
+      skip /= s->down;
+      
+      size = (size * s->up) / s->down;  /* this is integer */
     }
     rate->oskip = skip/2;
-                { /* bogus last stage is for output buffering */
+    { /* bogus last stage is for output buffering */
       polystage *s;
-      rate->stage[k] = s = (polystage*) malloc(sizeof(polystage));
+      rate->stage[k] = s = (polystage*) xmalloc(sizeof(polystage));
       s->up = s->down = 0;
       s->size = size;
       s->hsize = 0;
       s->held = 0;
       s->filt_len = 0;
       s->filt_array = NULL;
-      s->window = (Float *) malloc(sizeof(Float) * size);
+      s->window = (Float *) xmalloc(sizeof(Float) * size);
     }
-    st_report("Poly:  output samples %d, oskip %d",size, rate->oskip);
+    st_debug("Poly:  output samples %d, oskip %d",size, rate->oskip);
     return (ST_SUCCESS);
 }
 
@@ -510,10 +461,10 @@ static void polyphase(Float *output, polystage *s)
   Float *o_top;
 
   in = s->window + s->hsize;
-  /*for (mm=0; mm<s->filt_len; mm++) st_report("cf_%d %f\n",mm,s->filt_array[mm]);*/
+  /*for (mm=0; mm<s->filt_len; mm++) st_debug("cf_%d %f",mm,s->filt_array[mm]);*/
   /* assumes s->size divisible by down (now true) */
   o_top = output + (s->size * up) / down;
-  /*st_report(" isize %d, osize %d, up %d, down %d, N %d", s->size, o_top-output, up, down, f_len);*/
+  /*st_debug(" isize %d, osize %d, up %d, down %d, N %d", s->size, o_top-output, up, down, f_len);*/
   for (mm=0, o=output; o < o_top; mm+=down, o++) {
     double sum;
     const Float *p, *q;
@@ -535,18 +486,18 @@ static void update_hist(Float *hist, int hist_size, int in_size)
 
 }
 
-int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
+static int st_poly_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf,
                  st_size_t *isamp, st_size_t *osamp)
 {
   poly_t rate = (poly_t) effp->priv;
   polystage *s0,*s1;
 
   /* Sanity check:  how much can we tolerate? */
-  /* st_report("*isamp=%d *osamp=%d\n",*isamp,*osamp); fflush(stderr); */
+  /* st_debug("*isamp=%d *osamp=%d",*isamp,*osamp); */
   s0 = rate->stage[0];            /* the first stage */
   s1 = rate->stage[rate->total];  /* the 'last' stage is output buffer */
   {
-    int in_size, gap, k;
+    st_size_t in_size, gap, k;
 
     in_size = *isamp;
     gap = s0->size - s0->held; /* space available in this 'input' buffer */
@@ -570,7 +521,7 @@ int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
   }
 
   if (s0->held == s0->size && s1->held == 0) {
-    int k;
+    st_size_t k;
     /* input buffer full, output buffer empty, so do process */
 
     for(k=0; k<rate->total; k++) {
@@ -581,7 +532,7 @@ int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 
       out = rate->stage[k+1]->window + rate->stage[k+1]->hsize;
 
-      /* st_report("k=%d  insize=%d\n",k,in_size); fflush(stderr); */
+      /* st_debug("k=%d  insize=%d",k,in_size); */
       polyphase(out, s);
 
       /* copy input history into lower portion of rate->window[k] */
@@ -599,7 +550,7 @@ int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
     st_size_t out_size;
     st_size_t oskip;
     Float *out_buf;
-    int k;
+    st_size_t k;
 
     oskip = rate->oskip;
                 out_size = s1->held;
@@ -615,7 +566,7 @@ int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
     {
         float f;
         f = out_buf[k] * ISCALE; /* should clip-limit */
-        ST_SAMPLE_CLIP(f);
+        ST_SAMPLE_CLIP_COUNT(f, effp->clips);
         *q++ = f;
     }
 
@@ -638,7 +589,7 @@ int st_poly_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 /*
  * Process tail of input samples.
  */
-int st_poly_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_poly_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
   st_size_t in_size;
   /* Call "flow" with NULL input. */
@@ -650,37 +601,33 @@ int st_poly_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
  * Do anything required when you stop reading samples.
  * Don't close input file!
  */
-int st_poly_stop(eff_t effp)
+static int st_poly_stop(eff_t effp)
 {
-    poly_t rate = (poly_t) effp->priv;
-    polystage *s;
-    int k;
+    poly_t rate = (poly_t)effp->priv;
+    st_size_t k;
 
-    for(k = 0; k <= rate->total; k++) {
-      s = rate->stage[k];
-      free((void *) s->window);
-      if (s->filt_array) free((void *) s->filt_array);
-      free((void *) s);
+    for (k = 0; k <= rate->total; k++) {
+      free(rate->stage[k]->window);
+      free(rate->stage[k]->filt_array);
+      free(rate->stage[k]);
     }
-    return (ST_SUCCESS);
+
+    return ST_SUCCESS;
 }
 
 static st_effect_t st_polyphase_effect = {
   "polyphase",
-  "Usage: -w <nut / ham>        :  window type\n"
-  "       -width <short / long> :  window width\n"
-  "                                short = 128 samples\n"
-  "                                long  = 1024 samples\n"
-  "       <num>                    num:  explicit number\n"
+  "Usage: -w {nut|ham}   window type\n"
+  "       -width n       window width in samples [default 1024]\n"
   "\n"
-  "       -cutoff <float>       :  frequency cutoff for base bandwidth.\n"
-  "                                Default = 0.95 = 95%",
+  "       -cutoff float  frequency cutoff for base bandwidth [default 0.95]",
   ST_EFF_RATE,
   st_poly_getopts,
   st_poly_start,
   st_poly_flow,
   st_poly_drain,
-  st_poly_stop
+  st_poly_stop,
+  st_effect_nothing
 };
 
 const st_effect_t *st_polyphase_effect_fn(void)
