@@ -1,25 +1,32 @@
 /*
- * Psion record.app format (format of files needed for Revo,Revo+,Mako in 
- * System/Alarms to provide new alarm sounds. Note that the file normally
+ * Psion Record format (format of files used for Revo,Revo+,Mako in 
+ * System/Alarms to provide alarm sounds. Note that the file normally
  * has no extension, so I've made it .prc for now (Psion ReCord), until
- * somebody can come up with a better one. Also, I have absolutely no idea
- * what the header format is, I just looked at a few files, saw they all
- * had identical headers except for two places where the length words go,
- * so its very likely that most of this is wrong. It has worked for me,
- * however.
+ * somebody can come up with a better one.
  * Based (heavily) on the wve.c format file. 
  * Hacked by Bert van Leeuwen (bert@e.co.za)
- */
+ * Header check truncated to first 16 bytes (i.e. EPOC file header)
+ * and other improvements by Reuben Thomas <rrt@sc3d.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, write to the Free Software
+ * Foundation, Fifth Floor, 51 Franklin Street, Boston, MA 02111-1301,
+ * USA.  */
 
 #include "st_i.h"
 #include "g72x.h"
 #include <string.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
-
-/* Magic numbers used in Psion audio files */
-#define PSION_PRC_HDRSIZE   72
 
 typedef struct prcpriv
     {
@@ -30,26 +37,29 @@ typedef struct prcpriv
         st_size_t dataStart;
     } *prc_t;
 
-char header[]={
-  0x37,0x00,0x00,0x10,0x6d,0x00,0x00,0x10,
-  0x7e,0x00,0x00,0x10,0xcf,0xac,0x08,0x55,
-  0x14,0x00,0x00,0x00,0x04,0x52,0x00,0x00,
-  0x10,0x34,0x00,0x00,0x00,0x89,0x00,0x00,
-  0x10,0x25,0x00,0x00,0x00,0x7e,0x00,0x00,
-  0x10,0x2a,0x52,0x65,0x63,0x6f,0x72,0x64,
-  0x2e,0x61,0x70,0x70
+/* 16 bytes header = 3 UIDs plus checksum, standard Symbian/EPOC file
+   header */
+static const char prc_header[]={
+  '\x37','\x00','\x00','\x10','\x6d','\x00','\x00','\x10',
+  '\x7e','\x00','\x00','\x10','\xcf','\xac','\x08','\x55'
 };
 
+int prc_checkheader(ft_t ft, char *head)
+{
+  st_readbuf(ft, head, 1, sizeof(prc_header));
+  return memcmp(head, prc_header, sizeof(prc_header)) == 0;
+}
+
 static void prcwriteheader(ft_t ft);
 
-int st_prcseek(ft_t ft, st_size_t offset)
+static int st_prcseek(ft_t ft, st_size_t offset)
 {
     prc_t prc = (prc_t ) ft->priv;
     st_size_t new_offset, channel_block, alignment;
 
-    new_offset = offset * ft->info.size;
-    /* Make sure request aligns to a channel block (ie left+right) */
-    channel_block = ft->info.channels * ft->info.size;
+    new_offset = offset * ft->signal.size;
+    /* Make sure request aligns to a channel block (i.e. left+right) */
+    channel_block = ft->signal.channels * ft->signal.size;
     alignment = new_offset % channel_block;
     /* Most common mistaken is to compute something like
      * "skip everthing upto and including this sample" so
@@ -62,10 +72,10 @@ int st_prcseek(ft_t ft, st_size_t offset)
     return st_seeki(ft, new_offset, SEEK_SET);
 }
 
-int st_prcstartread(ft_t ft)
+static int st_prcstartread(ft_t ft)
 {
         prc_t p = (prc_t ) ft->priv;
-        char head[sizeof(header)];
+        char head[sizeof(prc_header)];
         int rc;
 
         uint16_t len;
@@ -75,19 +85,9 @@ int st_prcstartread(ft_t ft)
         if (rc)
             return rc;
 
-        /* PRC is in little endian format.  Swap whats read in
-         * on big endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 1 : 0;
-        }
-
         /* Check the header */
-        st_readbuf(ft, head,1, sizeof(header));
-        if (memcmp(head, header, sizeof(header))==0) {
-                st_report("Found Psion record.app header");
-        }
+        if (prc_checkheader(ft, head))
+                st_debug("Found Psion Record header");
         else
         {
                 st_fail_errno(ft,ST_EHDR,"Psion header doesn't start with the correct bytes\nTry the '.al' (A-law raw) file type with '-t al -r 8000 filename'");
@@ -96,24 +96,24 @@ int st_prcstartread(ft_t ft)
 
         st_readw(ft, &(len));
         p->length=len;
-        st_report("Found length=%d",len);
+        st_debug("Found length=%d",len);
 
         /* dummy read rest */
         st_readbuf(ft, head,1,14+2+2);
 
-        ft->info.encoding = ST_ENCODING_ALAW;
-        ft->info.size = ST_SIZE_BYTE;
+        ft->signal.encoding = ST_ENCODING_ALAW;
+        ft->signal.size = ST_SIZE_BYTE;
 
-        if (ft->info.rate != 0)
+        if (ft->signal.rate != 0)
             st_report("PRC must use 8000 sample rate.  Overriding");
-        ft->info.rate = 8000;
+        ft->signal.rate = 8000;
 
-        if (ft->info.channels != -1 && ft->info.channels != 1)
+        if (ft->signal.channels != ST_ENCODING_UNKNOWN && ft->signal.channels != 0)
             st_report("PRC must only supports 1 channel.  Overriding");
-        ft->info.channels = 1;
+        ft->signal.channels = 1;
 
         p->dataStart = st_tell(ft);
-        ft->length = p->length/ft->info.size;
+        ft->length = p->length/ft->signal.size;
 
         return (ST_SUCCESS);
 }
@@ -127,7 +127,7 @@ int st_prcstartread(ft_t ft)
    if it is not, the unspecified size remains in the header
    (this is illegal). */
 
-int st_prcstartwrite(ft_t ft)
+static int st_prcstartwrite(ft_t ft)
 {
         prc_t p = (prc_t ) ft->priv;
         int rc;
@@ -137,46 +137,33 @@ int st_prcstartwrite(ft_t ft)
         if (rc)
             return ST_EOF;
 
-        /* prc is in little endian format.  Swap whats read in
-         * on big endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 1 : 0;
-        }
-
         p->length = 0;
         if (p->repeats == 0)
             p->repeats = 1;
 
-        if (ft->info.rate != 0)
+        if (ft->signal.rate != 0)
             st_report("PRC must use 8000 sample rate.  Overriding");
 
-        if (ft->info.channels != -1 && ft->info.channels != 1)
+        if (ft->signal.channels != ST_ENCODING_UNKNOWN && ft->signal.channels != 0)
             st_report("PRC must only supports 1 channel.  Overriding");
 
-        ft->info.encoding = ST_ENCODING_ALAW;
-        ft->info.size = ST_SIZE_BYTE;
-        ft->info.rate = 8000;
+        ft->signal.encoding = ST_ENCODING_ALAW;
+        ft->signal.size = ST_SIZE_BYTE;
+        ft->signal.rate = 8000;
 
         prcwriteheader(ft);
         return ST_SUCCESS;
 }
 
-st_ssize_t st_prcread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
-{
-        return st_rawread(ft, buf, samp);
-}
-
-st_ssize_t st_prcwrite(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_prcwrite(ft_t ft, const st_sample_t *buf, st_size_t samp)
 {
         prc_t p = (prc_t ) ft->priv;
-        p->length += samp * ft->info.size;
-        st_report("length now = %d", p->length);
+        p->length += samp * ft->signal.size;
+        st_debug("length now = %d", p->length);
         return st_rawwrite(ft, buf, samp);
 }
 
-int st_prcstopwrite(ft_t ft)
+static int st_prcstopwrite(ft_t ft)
 {
         /* Call before seeking to flush buffer */
         st_rawstopwrite(ft);
@@ -187,7 +174,7 @@ int st_prcstopwrite(ft_t ft)
             return ST_SUCCESS;
         }
 
-        if (st_seeki(ft, 0L, 0) != 0)
+        if (st_seeki(ft, 0, 0) != 0)
         {
                 st_fail_errno(ft,errno,"Can't rewind output file to rewrite Psion header.");
                 return(ST_EOF);
@@ -201,9 +188,9 @@ static void prcwriteheader(ft_t ft)
   char nullbuf[15];
   prc_t p = (prc_t ) ft->priv;
 
-  st_report("Final length=%d",p->length);
+  st_debug("Final length=%d",p->length);
   memset(nullbuf,0,14);
-  st_writebuf(ft, header, 1, sizeof(header));
+  st_writebuf(ft, prc_header, 1, sizeof(prc_header));
   st_writew(ft, p->length);
   st_writebuf(ft, nullbuf,1,14);
   st_writew(ft, p->length);
@@ -211,7 +198,7 @@ static void prcwriteheader(ft_t ft)
 }
 
 /* Psion .prc */
-static char *prcnames[] = {
+static const char *prcnames[] = {
   "prc",
   NULL
 };
@@ -219,9 +206,9 @@ static char *prcnames[] = {
 static st_format_t st_prc_format = {
   prcnames,
   NULL,
-  ST_FILE_SEEK,
+  ST_FILE_SEEK | ST_FILE_BIG_END,
   st_prcstartread,
-  st_prcread,
+  st_rawread,
   st_rawstopread,
   st_prcstartwrite,
   st_prcwrite,
