@@ -1,12 +1,3 @@
-/*
- * July 5, 1991
- * Copyright 1991 Lance Norskog And Sundry Contributors
- * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Lance Norskog And Sundry Contributors are not responsible for 
- * the consequences of using this software.
- */
-
 /*
  * Sound Tools rate change effect file.
  * Spiffy rate changer using Smith & Wesson Bandwidth-Limited Interpolation.
@@ -19,35 +10,31 @@
  * at ftp://ccrma-ftp.stanford.edu/pub/NeXT/
  * under the name of resample-version.number.tar.Z
  *
- * NOTE: There is a newer version of the resample routine then what
- * this file was originally based on.  Those adventurous might be
- * interested in reviewing its improvesments and porting it to this
- * version.
- */
-
-/* Fixed bug: roll off frequency was wrong, too high by 2 when upsampling,
- * too low by 2 when downsampling.
- * Andreas Wilde, 12. Feb. 1999, andreas@eakaw2.et.tu-dresden.de
-*/
-
-/*
- * October 29, 1999
- * Various changes, bugfixes(?), increased precision, by Stan Brooks.
+ *
+ * FILE: resample.h
+ *   BY: Julius Smith (at CCRMA, Stanford U)
+ * C BY: translated from SAIL to C by Christopher Lee Fraley
+ *          (cf0v@andrew.cmu.edu)
+ * DATE: 7-JUN-88
+ * VERS: 2.0  (17-JUN-88, 3:00pm)
+ *
+ * July 5, 1991
+ * Copyright 1991 Lance Norskog And Sundry Contributors
+ * This source code is freely redistributable and may be used for
+ * any purpose.  This copyright notice must be maintained. 
+ * Lance Norskog And Sundry Contributors are not responsible for 
+ * the consequences of using this software.
  *
  * This source code is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- */
-/*
- * SJB: [11/25/99]
- * TODO: another idea for improvement...
- * note that upsampling usually doesn't require interpolation,
- * therefore is faster and more accurate than downsampling.
- * Downsampling by an integer factor is also simple, since
- * it just involves decimation if the input is already 
- * lowpass-filtered to the output Nyquist freqency.
- * Get the idea? :)
+ * Fixed bug: roll off frequency was wrong, too high by 2 when upsampling,
+ * too low by 2 when downsampling.
+ * Andreas Wilde, 12. Feb. 1999, andreas@eakaw2.et.tu-dresden.de
+ *
+ * October 29, 1999
+ * Various changes, bugfixes(?), increased precision, by Stan Brooks.
  */
 
 #include <math.h>
@@ -55,13 +42,52 @@
 #include <string.h>
 #include "st_i.h"
 
-/* resample includes */
-#include "resampl.h"
+
+/* Conversion constants */
+#define Lc        7
+#define Nc       (1<<Lc)
+#define La        16
+#define Na       (1<<La)
+#define Lp       (Lc+La)
+#define Np       (1<<Lp)
+#define Amask    (Na-1)
+#define Pmask    (Np-1)
+
+#define MAXNWING  (80<<Lc)
+/* Description of constants:
+ *
+ * Nc - is the number of look-up values available for the lowpass filter
+ *    between the beginning of its impulse response and the "cutoff time"
+ *    of the filter.  The cutoff time is defined as the reciprocal of the
+ *    lowpass-filter cut off frequence in Hz.  For example, if the
+ *    lowpass filter were a sinc function, Nc would be the index of the
+ *    impulse-response lookup-table corresponding to the first zero-
+ *    crossing of the sinc function.  (The inverse first zero-crossing
+ *    time of a sinc function equals its nominal cutoff frequency in Hz.)
+ *    Nc must be a power of 2 due to the details of the current
+ *    implementation. The default value of 128 is sufficiently high that
+ *    using linear interpolation to fill in between the table entries
+ *    gives approximately 16-bit precision, and quadratic interpolation
+ *    gives about 23-bit (float) precision in filter coefficients.
+ *
+ * Lc - is log base 2 of Nc.
+ *
+ * La - is the number of bits devoted to linear interpolation of the
+ *    filter coefficients.
+ *
+ * Lp - is La + Lc, the number of bits to the right of the binary point
+ *    in the integer "time" variable. To the left of the point, it indexes
+ *    the input array (X), and to the right, it is interpreted as a number
+ *    between 0 and 1 sample of the input X.  The default value of 23 is
+ *    about right.  There is a constraint that the filter window must be
+ *    "addressable" in a int32_t, more precisely, if Nmult is the number
+ *    of sinc zero-crossings in the right wing of the filter window, then
+ *    (Nwing<<Lp) must be expressible in 31 bits.
+ *
+ */
 
 static st_effect_t st_resample_effect;
 
-/* this Float MUST match that in filter.c */
-#define Float double/*float*/
 #define ISCALE 0x10000
 
 /* largest factor for which exact-coefficients upsampling will be used */
@@ -78,7 +104,7 @@ typedef struct resamplestuff {
    long Nmult;
    long Nwing;
    long Nq;
-   Float *Imp;        /* impulse [Nwing+1] Filter coefficients */
+   double *Imp;        /* impulse [Nwing+1] Filter coefficients */
 
    double Time;       /* Current time/pos in input sample */
    long dhb;
@@ -90,8 +116,8 @@ typedef struct resamplestuff {
    long Xoff;         /* Xh plus some room for creep  */
    long Xread;        /* X[Xread] is start-position to enter new samples */
    long Xp;           /* X[Xp] is position to start filter application   */
-   unsigned long Xsize,Ysize; /* size (Floats) of X[],Y[]         */
-   Float *X, *Y;      /* I/O buffers */
+   unsigned long Xsize,Ysize; /* size (doubles) of X[],Y[]         */
+   double *X, *Y;      /* I/O buffers */
 } *resample_t;
 
 static void LpFilter(double c[],
@@ -101,7 +127,7 @@ static void LpFilter(double c[],
                      long Num);
 
 /* makeFilter is used by filter.c */
-int makeFilter(Float Imp[],
+int makeFilter(double Imp[],
                long Nwing,
                double Froll,
                double Beta,
@@ -115,7 +141,7 @@ static long SrcEX(resample_t r, long Nx);
 /*
  * Process options
  */
-int st_resample_getopts(eff_t effp, int n, char **argv) 
+int st_resample_getopts(eff_t effp, int n, char **argv)
 {
         resample_t r = (resample_t) effp->priv;
 
@@ -125,8 +151,7 @@ int st_resample_getopts(eff_t effp, int n, char **argv)
         r->quadr = 0;
         r->Nmult = 45;
 
-        /* This used to fail, but with sox-12.15 it works. AW */
-        if ((n >= 1)) {
+        if (n >= 1) {
                 if (!strcmp(argv[0], "-qs")) {
                         r->quadr = 1;
                         n--; argv++;
@@ -145,28 +170,22 @@ int st_resample_getopts(eff_t effp, int n, char **argv)
                 }
         }
 
-        if ((n >= 1) && (sscanf(argv[0], "%lf", &r->rolloff) != 1))
-        {
+        if ((n >= 1) && (sscanf(argv[0], "%lf", &r->rolloff) != 1)) {
           st_fail(st_resample_effect.usage);
           return (ST_EOF);
-        }
-        else if ((r->rolloff <= 0.01) || (r->rolloff >= 1.0))
-        {
+        } else if ((r->rolloff <= 0.01) || (r->rolloff >= 1.0)) {
           st_fail("resample: rolloff factor (%f) no good, should be 0.01<x<1.0", r->rolloff);
           return(ST_EOF);
         }
 
-        if ((n >= 2) && !sscanf(argv[1], "%lf", &r->beta))
-        {
-          st_fail(st_resample_effect.usage);
-          return (ST_EOF);
-        }
-        else if (r->beta <= 2.0) {
-          r->beta = 0;
-                st_report("resample opts: Nuttall window, cutoff %f\n", r->rolloff);
-        } else {
-                st_report("resample opts: Kaiser window, cutoff %f, beta %f\n", r->rolloff, r->beta);
-        }
+        if ((n >= 2) && !sscanf(argv[1], "%lf", &r->beta)) {
+        	st_fail(st_resample_effect.usage);
+          	return (ST_EOF);
+        } else if (r->beta <= 2.0) {
+        	r->beta = 0;
+                st_debug("resample opts: Nuttall window, cutoff %f", r->rolloff);
+        } else
+                st_debug("resample opts: Kaiser window, cutoff %f, beta %f", r->rolloff, r->beta);
         return (ST_SUCCESS);
 }
 
@@ -175,107 +194,101 @@ int st_resample_getopts(eff_t effp, int n, char **argv)
  */
 int st_resample_start(eff_t effp)
 {
-        resample_t r = (resample_t) effp->priv;
-        long Xoff, gcdrate;
-        int i;
+  resample_t r = (resample_t) effp->priv;
+  long Xoff, gcdrate;
+  int i;
 
-        if (effp->ininfo.rate == effp->outinfo.rate)
-        {
-            st_fail("Input and Output rates must be different to use resample effect");
-            return(ST_EOF);
-        }
-                
-        r->Factor = (double)effp->outinfo.rate / (double)effp->ininfo.rate;
+  /* The next line makes the "speed" effect accurate; it's needed because
+   * ininfo.rate (st_rate_t) isn't floating point (but it's probably not worth
+   * changing st_rate_t just because of this): */
+  double in_rate = floor(effp->ininfo.rate / effp->globalinfo->speed + .5)
+    * effp->globalinfo->speed;
 
-        gcdrate = st_gcd((long)effp->ininfo.rate, (long)effp->outinfo.rate);
-        r->a = effp->ininfo.rate / gcdrate;
-        r->b = effp->outinfo.rate / gcdrate;
+  if (in_rate == effp->outinfo.rate)
+    return ST_EFF_NULL;
+          
+  r->Factor = (double) effp->outinfo.rate / in_rate;
 
-        if (r->a <= r->b && r->b <= NQMAX) {
-                r->quadr = -1; /* exact coeff's   */
-                r->Nq = r->b;  /* MAX(r->a,r->b);       */
-        } else {
-                r->Nq = Nc; /* for now */
-        }
+  gcdrate = st_gcd((long) effp->ininfo.rate, (long) effp->outinfo.rate);
+  r->a = effp->ininfo.rate / gcdrate;
+  r->b = effp->outinfo.rate / gcdrate;
 
-        /* Check for illegal constants */
-# if 0
-        if (Lp >= 16) st_fail("Error: Lp>=16");
-        if (Nb+Nhg+NLpScl >= 32) st_fail("Error: Nb+Nhg+NLpScl>=32");
-        if (Nh+Nb > 32) st_fail("Error: Nh+Nb>32");
-# endif
+  if (r->a <= r->b && r->b <= NQMAX) {
+    r->quadr = -1;      /* exact coeffs */
+    r->Nq = r->b;       /* max(r->a,r->b) */
+  } else
+    r->Nq = Nc; /* for now */
 
-        /* Nwing: # of filter coeffs in right wing */
-        r->Nwing = r->Nq * (r->Nmult/2+1) + 1;
+  /* Nwing: # of filter coeffs in right wing */
+  r->Nwing = r->Nq * (r->Nmult / 2 + 1) + 1;
 
-        r->Imp = (Float *)malloc(sizeof(Float) * (r->Nwing+2)) + 1;
-        /* need Imp[-1] and Imp[Nwing] for quadratic interpolation */
-        /* returns error # <=0, or adjusted wing-len > 0 */
-        i = makeFilter(r->Imp, r->Nwing, r->rolloff, r->beta, r->Nq, 1);
-        if (i <= 0)
-        {
-                st_fail("resample: Unable to make filter\n");
-                return (ST_EOF);
-        }
+  r->Imp = (double *) xmalloc(sizeof(double) * (r->Nwing + 2)) + 1;
+  /* need Imp[-1] and Imp[Nwing] for quadratic interpolation */
+  /* returns error # <=0, or adjusted wing-len > 0 */
+  i = makeFilter(r->Imp, r->Nwing, r->rolloff, r->beta, r->Nq, 1);
+  if (i <= 0) {
+    st_fail("resample: Unable to make filter");
+    return (ST_EOF);
+  }
 
-        /*st_report("Nmult: %ld, Nwing: %ld, Nq: %ld\n",r->Nmult,r->Nwing,r->Nq);*/
+  st_debug("Nmult: %ld, Nwing: %ld, Nq: %ld", r->Nmult, r->Nwing, r->Nq);
 
-        if (r->quadr < 0) { /* exact coeff's method */
-                r->Xh = r->Nwing/r->b;
-          st_report("resample: rate ratio %ld:%ld, coeff interpolation not needed\n", r->a, r->b);
-        } else {
-          r->dhb = Np;  /* Fixed-point Filter sampling-time-increment */
-          if (r->Factor<1.0) r->dhb = r->Factor*Np + 0.5;
-          r->Xh = (r->Nwing<<La)/r->dhb;
-          /* (Xh * dhb)>>La is max index into Imp[] */
-        }
+  if (r->quadr < 0) {     /* exact coeff's method */
+    r->Xh = r->Nwing / r->b;
+    st_debug("resample: rate ratio %ld:%ld, coeff interpolation not needed", r->a, r->b);
+  } else {
+    r->dhb = Np;        /* Fixed-point Filter sampling-time-increment */
+    if (r->Factor < 1.0)
+      r->dhb = r->Factor * Np + 0.5;
+    r->Xh = (r->Nwing << La) / r->dhb;
+    /* (Xh * dhb)>>La is max index into Imp[] */
+  }
 
-        /* reach of LP filter wings + some creeping room */
-        Xoff = r->Xh + 10;
-        r->Xoff = Xoff;
+  /* reach of LP filter wings + some creeping room */
+  Xoff = r->Xh + 10;
+  r->Xoff = Xoff;
 
-        /* Current "now"-sample pointer for input to filter */
-        r->Xp = Xoff;
-        /* Position in input array to read into */
-        r->Xread = Xoff;
-        /* Current-time pointer for converter */
-        r->Time = Xoff;
-        if (r->quadr < 0) { /* exact coeff's method */
-                r->t = Xoff*r->Nq;
-        }
-        i = BUFFSIZE - 2*Xoff;
-        if (i < r->Factor + 1.0/r->Factor)      /* Check input buffer size */
-        {
-                st_fail("Factor is too small or large for BUFFSIZE");
-                return (ST_EOF);
-        }
-        
-        r->Xsize = 2*Xoff + i/(1.0+r->Factor);
-        r->Ysize = BUFFSIZE - r->Xsize;
-        /* st_report("Xsize %d, Ysize %d, Xoff %d",r->Xsize,r->Ysize,r->Xoff); */
+  /* Current "now"-sample pointer for input to filter */
+  r->Xp = Xoff;
+  /* Position in input array to read into */
+  r->Xread = Xoff;
+  /* Current-time pointer for converter */
+  r->Time = Xoff;
+  if (r->quadr < 0) {     /* exact coeff's method */
+    r->t = Xoff * r->Nq;
+  }
+  i = BUFFSIZE - 2 * Xoff;
+  if (i < r->Factor + 1.0 / r->Factor) {  /* Check input buffer size */
+    st_fail("Factor is too small or large for BUFFSIZE");
+    return (ST_EOF);
+  }
 
-        r->X = (Float *) malloc(sizeof(Float) * (BUFFSIZE));
-        r->Y = r->X + r->Xsize;
+  r->Xsize = 2 * Xoff + i / (1.0 + r->Factor);
+  r->Ysize = BUFFSIZE - r->Xsize;
+  st_debug("Xsize %d, Ysize %d, Xoff %d", r->Xsize, r->Ysize, r->Xoff);
 
-        /* Need Xoff zeros at beginning of sample */
-        for (i=0; i<Xoff; i++)
-                r->X[i] = 0;
-        return (ST_SUCCESS);
+  r->X = (double *) xmalloc(sizeof(double) * (BUFFSIZE));
+  r->Y = r->X + r->Xsize;
+
+  /* Need Xoff zeros at beginning of sample */
+  for (i = 0; i < Xoff; i++)
+    r->X[i] = 0;
+  return (ST_SUCCESS);
 }
 
 /*
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+int st_resample_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                      st_size_t *isamp, st_size_t *osamp)
 {
         resample_t r = (resample_t) effp->priv;
         long i, last, Nout, Nx, Nproc;
 
-        /* constrain amount we actually process */
-        /*fprintf(stderr,"Xp %d, Xread %d, isamp %d, ",r->Xp, r->Xread,*isamp);*/
+        st_debug("Xp %d, Xread %d, isamp %d, ",r->Xp, r->Xread,*isamp);
 
+        /* constrain amount we actually process */
         Nproc = r->Xsize - r->Xp;
 
         i = (r->Ysize < *osamp)? r->Ysize : *osamp;
@@ -290,14 +303,14 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         }
         if ((unsigned long)Nx > *isamp)
                 Nx = *isamp;
-        /*fprintf(stderr,"Nx %d\n",Nx);*/
+        st_debug("Nx %d",Nx);
 
         if (ibuf == NULL) {
                 for(i = r->Xread; i < Nx + r->Xread  ; i++) 
                         r->X[i] = 0;
         } else {
                 for(i = r->Xread; i < Nx + r->Xread  ; i++) 
-                        r->X[i] = (Float)(*ibuf++)/ISCALE;
+                        r->X[i] = (double)(*ibuf++)/ISCALE;
         }
         last = i;
         Nproc = last - r->Xoff - r->Xp;
@@ -312,7 +325,7 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         if (r->quadr < 0) { /* exact coeff's method */
                 long creep; 
                 Nout = SrcEX(r, Nproc);
-                /*fprintf(stderr,"Nproc %d --> %d\n",Nproc,Nout);*/
+                st_debug("Nproc %d --> %d",Nproc,Nout);
                 /* Move converter Nproc samples back in time */
                 r->t -= Nproc * r->b;
                 /* Advance by number of samples processed */
@@ -323,12 +336,12 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
                 {
                   r->t -= creep * r->b;  /* Remove time accumulation   */
                   r->Xp += creep;        /* and add it to read pointer */
-                  /*fprintf(stderr,"Nproc %ld, creep %ld\n",Nproc,creep);*/
+                  st_debug("Nproc %ld, creep %ld",Nproc,creep);
                 }
         } else { /* approx coeff's method */
                 long creep; 
                 Nout = SrcUD(r, Nproc);
-                /*fprintf(stderr,"Nproc %d --> %d\n",Nproc,Nout);*/
+                st_debug("Nproc %d --> %d",Nproc,Nout);
                 /* Move converter Nproc samples back in time */
                 r->Time -= Nproc;
                 /* Advance by number of samples processed */
@@ -339,7 +352,7 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
                 {
                   r->Time -= creep;   /* Remove time accumulation   */
                   r->Xp += creep;     /* and add it to read pointer */
-                  /* fprintf(stderr,"Nproc %ld, creep %ld\n",Nproc,creep); */
+                  st_debug("Nproc %ld, creep %ld",Nproc,creep);
                 }
         }
 
@@ -347,7 +360,7 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         long i,k;
         /* Copy back portion of input signal that must be re-used */
         k = r->Xp - r->Xoff;
-        /*fprintf(stderr,"k %d, last %d\n",k,last);*/
+        st_debug("k %d, last %d",k,last);
         for (i=0; i<last - k; i++) 
             r->X[i] = r->X[i+k];
 
@@ -356,10 +369,9 @@ int st_resample_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         r->Xp = r->Xoff;
 
         for(i=0; i < Nout; i++) { 
-                // orig: *obuf++ = r->Y[i] * ISCALE;
-                Float ftemp = r->Y[i] * ISCALE;
+                double ftemp = r->Y[i] * ISCALE;
 
-                ST_SAMPLE_CLIP(ftemp);
+                ST_SAMPLE_CLIP_COUNT(ftemp, effp->clips);
                 *obuf++ = ftemp;
         }
 
@@ -380,7 +392,7 @@ int st_resample_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
         st_sample_t *Obuf;
         int rc;
 
-        /* fprintf(stderr,"Xoff %d  <--- DRAIN\n",r->Xoff); */
+        st_debug("Xoff %d  <--- DRAIN",r->Xoff);
 
         /* stuff end with Xoff zeros */
         isamp_res = r->Xoff;
@@ -393,16 +405,16 @@ int st_resample_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
                 rc = st_resample_flow(effp, NULL, Obuf, (st_size_t *)&Isamp, (st_size_t *)&Osamp);
                 if (rc)
                     return rc;
-                /* fprintf(stderr,"DRAIN isamp,osamp  (%d,%d) -> (%d,%d)\n",
-                        isamp_res,osamp_res,Isamp,Osamp); */
+                st_debug("DRAIN isamp,osamp  (%d,%d) -> (%d,%d)",
+                         isamp_res,osamp_res,Isamp,Osamp);
                 Obuf += Osamp;
                 osamp_res -= Osamp;
                 isamp_res -= Isamp;
         }
         *osamp -= osamp_res;
-        /* fprintf(stderr,"DRAIN osamp %d\n", *osamp); */
+        st_debug("DRAIN osamp %d", *osamp);
         if (isamp_res)
-                st_warn("drain overran obuf by %d\n", isamp_res); fflush(stderr);
+                st_warn("drain overran obuf by %d", isamp_res);
         /* FIXME: This is very picky.  IF obuf is not big enough to
          * drain remaining samples, they will be lost.
          */
@@ -425,7 +437,7 @@ int st_resample_stop(eff_t effp)
 
 /* over 90% of CPU time spent in this iprodUD() function */
 /* quadratic interpolation */
-static double qprodUD(const Float Imp[], const Float *Xp, long Inc, double T0, 
+static double qprodUD(const double Imp[], const double *Xp, long Inc, double T0, 
                       long dhb, long ct)
 {
   const double f = 1.0/(1<<La);
@@ -433,16 +445,16 @@ static double qprodUD(const Float Imp[], const Float *Xp, long Inc, double T0,
   long Ho;
 
   Ho = T0 * dhb;
-  Ho += (ct-1)*dhb; /* so Float sum starts with smallest coef's */
+  Ho += (ct-1)*dhb; /* so double sum starts with smallest coef's */
   Xp += (ct-1)*Inc;
   v = 0;
   do {
-    Float coef;
+    double coef;
     long Hoh;
     Hoh = Ho>>La;
     coef = Imp[Hoh];
     {
-      Float dm,dp,t;
+      double dm,dp,t;
       dm = coef - Imp[Hoh-1];
       dp = Imp[Hoh+1] - coef;
       t =(Ho & Amask) * f;
@@ -457,7 +469,7 @@ static double qprodUD(const Float Imp[], const Float *Xp, long Inc, double T0,
 }
 
 /* linear interpolation */
-static double iprodUD(const Float Imp[], const Float *Xp, long Inc, 
+static double iprodUD(const double Imp[], const double *Xp, long Inc, 
                       double T0, long dhb, long ct)
 {
   const double f = 1.0/(1<<La);
@@ -465,11 +477,11 @@ static double iprodUD(const Float Imp[], const Float *Xp, long Inc,
   long Ho;
 
   Ho = T0 * dhb;
-  Ho += (ct-1)*dhb; /* so Float sum starts with smallest coef's */
+  Ho += (ct-1)*dhb; /* so double sum starts with smallest coef's */
   Xp += (ct-1)*Inc;
   v = 0;
   do {
-    Float coef;
+    double coef;
     long Hoh;
     Hoh = Ho>>La;
     /* if (Hoh >= End) break; */
@@ -487,56 +499,55 @@ static double iprodUD(const Float Imp[], const Float *Xp, long Inc,
 
 static long SrcUD(resample_t r, long Nx)
 {
-   Float *Ystart, *Y;
+   double *Ystart, *Y;
    double Factor;
    double dt;                  /* Step through input signal */
    double time;
-   double (*prodUD)(const Float[], const Float *, long, double, long, long);
+   double (*prodUD)(const double[], const double *, long, double, long, long);
    int n;
 
    prodUD = (r->quadr)? qprodUD:iprodUD; /* quadratic or linear interp */
    Factor = r->Factor;
    time = r->Time;
    dt = 1.0/Factor;        /* Output sampling period */
-   /*fprintf(stderr,"Factor %f, dt %f, ",Factor,dt); */
-   /*fprintf(stderr,"Time %f, ",r->Time);*/
+   st_debug("Factor %f, dt %f, ",Factor,dt);
+   st_debug("Time %f, ",r->Time);
    /* (Xh * dhb)>>La is max index into Imp[] */
-   /*fprintf(stderr,"ct=%d\n",ct);*/
-   /*fprintf(stderr,"ct=%.2f %d\n",(double)r->Nwing*Na/r->dhb, r->Xh);*/
-   /*fprintf(stderr,"ct=%ld, T=%.6f, dhb=%6f, dt=%.6f\n",
-                         r->Xh, time-floor(time),(double)r->dhb/Na,dt);*/
+   st_debug("ct=%.2f %d",(double)r->Nwing*Na/r->dhb, r->Xh);
+   st_debug("ct=%ld, T=%.6f, dhb=%6f, dt=%.6f",
+                         r->Xh, time-floor(time),(double)r->dhb/Na,dt);
    Ystart = Y = r->Y;
    n = (int)ceil((double)Nx/dt);
    while(n--)
       {
-      Float *Xp;
+      double *Xp;
       double v;
       double T;
       T = time-floor(time);        /* fractional part of Time */
       Xp = r->X + (long)time;      /* Ptr to current input sample */
 
       /* Past  inner product: */
-      v = (*prodUD)(r->Imp, Xp, -1L, T, r->dhb, r->Xh); /* needs Np*Nmult in 31 bits */
+      v = (*prodUD)(r->Imp, Xp, -1, T, r->dhb, r->Xh); /* needs Np*Nmult in 31 bits */
       /* Future inner product: */
-      v += (*prodUD)(r->Imp, Xp+1, 1L, (1.0-T), r->dhb, r->Xh); /* prefer even total */
+      v += (*prodUD)(r->Imp, Xp+1, 1, (1.0-T), r->dhb, r->Xh); /* prefer even total */
 
       if (Factor < 1) v *= Factor;
       *Y++ = v;              /* Deposit output */
       time += dt;            /* Move to next sample by time increment */
       }
    r->Time = time;
-   /*fprintf(stderr,"Time %f\n",r->Time);*/
+   st_debug("Time %f",r->Time);
    return (Y - Ystart);        /* Return the number of output samples */
 }
 
 /* exact coeff's */
-static double prodEX(const Float Imp[], const Float *Xp, 
+static double prodEX(const double Imp[], const double *Xp, 
                      long Inc, long T0, long dhb, long ct)
 {
   double v;
-  const Float *Cp;
+  const double *Cp;
 
-  Cp  = Imp + (ct-1)*dhb + T0; /* so Float sum starts with smallest coef's */
+  Cp  = Imp + (ct-1)*dhb + T0; /* so double sum starts with smallest coef's */
   Xp += (ct-1)*Inc;
   v = 0;
   do {
@@ -549,7 +560,7 @@ static double prodEX(const Float Imp[], const Float *Xp,
 
 static long SrcEX(resample_t r, long Nx)
 {
-   Float *Ystart, *Y;
+   double *Ystart, *Y;
    double Factor;
    long a,b;
    long time;
@@ -563,7 +574,7 @@ static long SrcEX(resample_t r, long Nx)
    n = (Nx*b + (a-1))/a;
    while(n--)
       {
-        Float *Xp;
+        double *Xp;
         double v;
         long T;
         T = time % b;              /* fractional part of Time */
@@ -582,7 +593,7 @@ static long SrcEX(resample_t r, long Nx)
    return (Y - Ystart);        /* Return the number of output samples */
 }
 
-int makeFilter(Float Imp[], long Nwing, double Froll, double Beta, 
+int makeFilter(double Imp[], long Nwing, double Froll, double Beta, 
                long Num, int Normalize)
 {
    double *ImpR;
@@ -599,7 +610,7 @@ int makeFilter(Float Imp[], long Nwing, double Froll, double Beta,
    if (Mwing==0)
       return(-4);
 
-   ImpR = (double *) malloc(sizeof(double) * Mwing);
+   ImpR = (double *) xmalloc(sizeof(double) * Mwing);
 
    /* Design a Nuttall or Kaiser windowed Sinc low-pass filter */
    LpFilter(ImpR, Mwing, Froll, Beta, Num);
@@ -612,7 +623,7 @@ int makeFilter(Float Imp[], long Nwing, double Froll, double Beta,
       for (i=Dh; i<Mwing; i+=Dh)
          DCgain += ImpR[i];
       DCgain = 2*DCgain + ImpR[0];    /* DC gain of real coefficients */
-      /*st_report("DCgain err=%.12f",DCgain-1.0);*/
+      st_debug("DCgain err=%.12f",DCgain-1.0);
   
       DCgain = 1.0/DCgain;
       for (i=0; i<Mwing; i++)
@@ -714,13 +725,14 @@ static void LpFilter(double *c, long N, double frq, double Beta, long Num)
 
 static st_effect_t st_resample_effect = {
    "resample",
-   "Usage: resample [ rolloff [ beta ] ]",
+   "Usage: resample [ -qs | -q | -ql ] [ rolloff [ beta ] ]",
    ST_EFF_RATE,
    st_resample_getopts,
    st_resample_start,
    st_resample_flow,
    st_resample_drain,
-   st_resample_stop
+   st_resample_stop,
+  st_effect_nothing
 };
 
 const st_effect_t *st_resample_effect_fn(void)
