@@ -30,7 +30,7 @@
  * the difference in its defintion.
  * The idea of the echoplexer is modified and enhanceb by an automatic
  * setting of each decay for realistic reverb.
- * Some bugs are fixed concerning malloc and fade-outs.
+ * Some bugs are fixed concerning xmalloc and fade-outs.
  * Added an output volume (gain-out) avoiding saturation or clipping.
  *
  *
@@ -98,24 +98,24 @@
 static st_effect_t st_reverb_effect;
 
 #define REVERB_FADE_THRESH 10
-#define DELAY_BUFSIZ ( 50L * ST_MAXRATE )
+#define DELAY_BUFSIZ ( 50 * ST_MAXRATE )
 #define MAXREVERBS 8
 
 /* Private data for SKEL file */
 typedef struct reverbstuff {
         int     counter;                        
-        int     numdelays;
+        size_t  numdelays;
         float   *reverbbuf;
         float   in_gain, out_gain, time;
         float   delay[MAXREVERBS], decay[MAXREVERBS];
-        long    samples[MAXREVERBS], maxsamples;
+        size_t  samples[MAXREVERBS], maxsamples;
         st_sample_t pl, ppl, pppl;
 } *reverb_t;
 
 /*
  * Process options
  */
-int st_reverb_getopts(eff_t effp, int n, char **argv) 
+static int st_reverb_getopts(eff_t effp, int n, char **argv) 
 {
         reverb_t reverb = (reverb_t) effp->priv;
         int i;
@@ -150,10 +150,10 @@ int st_reverb_getopts(eff_t effp, int n, char **argv)
 /*
  * Prepare for processing.
  */
-int st_reverb_start(eff_t effp)
+static int st_reverb_start(eff_t effp)
 {
         reverb_t reverb = (reverb_t) effp->priv;
-        int i;
+        size_t i;
 
         reverb->in_gain = 1.0;
 
@@ -173,12 +173,12 @@ int st_reverb_start(eff_t effp)
                 reverb->samples[i] = reverb->delay[i] * effp->ininfo.rate / 1000.0;
                 if ( reverb->samples[i] < 1 )
                 {
-                    st_fail("reverb: delay must be positive!\n");
+                    st_fail("reverb: delay must be positive!");
                     return (ST_EOF);
                 }
                 if ( reverb->samples[i] > DELAY_BUFSIZ )
                 {
-                        st_fail("reverb: delay must be less than %g seconds!\n",
+                        st_fail("reverb: delay must be less than %g seconds!",
                                 DELAY_BUFSIZ / (float) effp->ininfo.rate );
                         return(ST_EOF);
                 }
@@ -187,12 +187,7 @@ int st_reverb_start(eff_t effp)
                 if ( reverb->samples[i] > reverb->maxsamples )
                     reverb->maxsamples = reverb->samples[i];
         }
-        if (! (reverb->reverbbuf = (float *) malloc(sizeof (float) * reverb->maxsamples)))
-        {
-                st_fail("reverb: Cannot malloc %d bytes!\n", 
-                        sizeof(float) * reverb->maxsamples);
-                return(ST_EOF);
-        }
+        reverb->reverbbuf = (float *) xmalloc(sizeof (float) * reverb->maxsamples);
         for ( i = 0; i < reverb->maxsamples; ++i )
                 reverb->reverbbuf[i] = 0.0;
         reverb->pppl = reverb->ppl = reverb->pl = 0x7fffff;             /* fade-outs */
@@ -208,12 +203,12 @@ int st_reverb_start(eff_t effp)
  * Processed signed long samples from ibuf to obuf.
  * Return number of samples processed.
  */
-int st_reverb_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_reverb_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                    st_size_t *isamp, st_size_t *osamp)
 {
         reverb_t reverb = (reverb_t) effp->priv;
-        int len, done;
-        int i, j;
+        size_t len, done;
+        size_t i, j;
         
         float d_in, d_out;
         st_sample_t out;
@@ -229,7 +224,7 @@ int st_reverb_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
                         d_in +=
 reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsamples] * reverb->decay[j];
                 d_out = d_in * reverb->out_gain;
-                out = st_clip24((st_sample_t) d_out);
+                out = ST_24BIT_CLIP_COUNT((st_sample_t) d_out, effp->clips);
                 *obuf++ = out * 256;
                 reverb->reverbbuf[i] = d_in;
                 i++;            /* XXX need a % maxsamples here ? */
@@ -243,12 +238,12 @@ reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsam
 /*
  * Drain out reverb lines. 
  */
-int st_reverb_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
+static int st_reverb_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
 {
         reverb_t reverb = (reverb_t) effp->priv;
         float d_in, d_out;
         st_sample_t out, l;
-        int i, j;
+        size_t i, j;
         st_size_t done;
 
         i = reverb->counter;
@@ -261,10 +256,10 @@ int st_reverb_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
                         d_in += 
 reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsamples] * reverb->decay[j];
                 d_out = d_in * reverb->out_gain;
-                out = st_clip24((st_sample_t) d_out);
+                out = ST_24BIT_CLIP_COUNT((st_sample_t) d_out, effp->clips);
                 obuf[done++] = out * 256;
                 reverb->reverbbuf[i] = d_in;
-                l = st_clip24((st_sample_t) d_in);
+                l = ST_24BIT_CLIP_COUNT((st_sample_t) d_in, effp->clips);
                 reverb->pppl = reverb->ppl;
                 reverb->ppl = reverb->pl;
                 reverb->pl = l;
@@ -280,7 +275,7 @@ reverb->reverbbuf[(i + reverb->maxsamples - reverb->samples[j]) % reverb->maxsam
 /*
  * Clean up reverb effect.
  */
-int st_reverb_stop(eff_t effp)
+static int st_reverb_stop(eff_t effp)
 {
         reverb_t reverb = (reverb_t) effp->priv;
 
@@ -297,7 +292,8 @@ static st_effect_t st_reverb_effect = {
   st_reverb_start,
   st_reverb_flow,
   st_reverb_drain,
-  st_reverb_stop
+  st_reverb_stop,
+  st_effect_nothing
 };
 
 const st_effect_t *st_reverb_effect_fn(void)
