@@ -1,22 +1,19 @@
 /*
+ * NIST Sphere file format handler.
+ *
  * August 7, 2000
  *
  * Copyright (C) 2000 Chris Bagwell (cbagwell@sprynet.com)
  *
  */
 
-/*
- * NIST Sphere file format handler.
- */
-
 #include "st_i.h"
 
 #include <math.h>
 #include <string.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
-/* Private data for SKEL file */
+
+/* Private data for sphere file */
 typedef struct spherestuff {
         char      shorten_check[4];
         st_size_t numSamples;
@@ -29,7 +26,7 @@ typedef struct spherestuff {
  *      size and encoding of samples, 
  *      mono/stereo/quad.
  */
-int st_spherestartread(ft_t ft) 
+static int st_spherestartread(ft_t ft) 
 {
         sphere_t sphere = (sphere_t) ft->priv;
         int rc;
@@ -59,12 +56,7 @@ int st_spherestartread(ft_t ft)
 
         /* Determine header size, and allocate a buffer large enough to hold it. */
         sscanf(fldsval, "%d", &header_size);
-        buf = (char *)malloc(header_size);
-        if (buf == NULL)
-        {
-            st_fail_errno(ft,ST_ENOMEM,"Unable to allocate memory");
-            return(ST_ENOMEM);
-        }
+        buf = (char *)xmalloc(header_size);
 
         /* Skip what we have read so far */
         header_size -= 16;
@@ -80,16 +72,16 @@ int st_spherestartread(ft_t ft)
 
         while (strncmp(buf, "end_head", 8) != 0)
         {
-            if (strncmp(buf, "sample_n_bytes", 14) == 0 && ft->info.size == -1)
+            if (strncmp(buf, "sample_n_bytes", 14) == 0 && ft->signal.size == -1)
             {
                 sscanf(buf, "%63s %15s %d", fldname, fldtype, &i);
-                ft->info.size = i;
+                ft->signal.size = i;
             }
             if (strncmp(buf, "channel_count", 13) == 0 && 
-                ft->info.channels == -1)
+                ft->signal.channels == 0)
             {
                 sscanf(buf, "%63s %15s %d", fldname, fldtype, &i);
-                ft->info.channels = i;
+                ft->signal.channels = i;
             }
             if (strncmp(buf, "sample_coding", 13) == 0)
             {
@@ -97,37 +89,25 @@ int st_spherestartread(ft_t ft)
                 /* Only bother looking for ulaw flag.  All others
                  * should be caught below by default PCM check
                  */
-                if (ft->info.encoding == -1 && 
+                if (ft->signal.encoding == ST_ENCODING_UNKNOWN && 
                     strncmp(fldsval,"ulaw",4) == 0)
                 {
-                    ft->info.encoding = ST_ENCODING_ULAW;
+                    ft->signal.encoding = ST_ENCODING_ULAW;
                 }
             }
             if (strncmp(buf, "sample_rate ", 12) == 0 &&
-                ft->info.rate == 0)
+                ft->signal.rate == 0)
             {
                 sscanf(buf, "%53s %15s %ld", fldname, fldtype, &rate);
-                ft->info.rate = rate;
+                ft->signal.rate = rate;
             }
             if (strncmp(buf, "sample_byte_format", 18) == 0)
             {
                 sscanf(buf, "%53s %15s %127s", fldname, fldtype, fldsval);
                 if (strncmp(fldsval,"01",2) == 0)
-                {
-                    /* Data is in little endian. */
-                    if (ST_IS_BIGENDIAN)
-                    {
-                        ft->swap = ft->swap ? 0 : 1;
-                    }
-                }
+                  ft->signal.reverse_bytes = ST_IS_BIGENDIAN; /* Data is little endian. */
                 else if (strncmp(fldsval,"10",2) == 0)
-                {
-                    /* Data is in big endian. */
-                    if (ST_IS_LITTLEENDIAN)
-                    {
-                        ft->swap = ft->swap ? 0 : 1;
-                    }
-                }
+                  ft->signal.reverse_bytes = ST_IS_LITTLEENDIAN; /* Data is big endian. */
             }
 
             if (st_reads(ft, buf, header_size) == ST_EOF)
@@ -140,19 +120,19 @@ int st_spherestartread(ft_t ft)
             header_size -= (strlen(buf) + 1);
         }
 
-        if (ft->info.size == -1)
-            ft->info.size = ST_SIZE_BYTE;
+        if (ft->signal.size == -1)
+            ft->signal.size = ST_SIZE_BYTE;
 
         /* sample_coding is optional and is PCM if missing.
          * This means encoding is signed if size = word or
          * unsigned if size = byte.
          */
-        if (ft->info.encoding == -1)
+        if (ft->signal.encoding == ST_ENCODING_UNKNOWN)
         {
-            if (ft->info.size == 1)
-                ft->info.encoding = ST_ENCODING_UNSIGNED;
+            if (ft->signal.size == 1)
+                ft->signal.encoding = ST_ENCODING_UNSIGNED;
             else
-                ft->info.encoding = ST_ENCODING_SIGN2;
+                ft->signal.encoding = ST_ENCODING_SIGN2;
         }
 
         while (header_size)
@@ -168,20 +148,17 @@ int st_spherestartread(ft_t ft)
 
         sphere->shorten_check[0] = 0;
 
-        /* TODO: Check first four bytes of data to see if its shorten
-         * compressed or not.  This data will need to be written to
-         * buffer during first st_sphereread().
+        /* Check first four bytes of data to see if it's shorten
+         * compressed or not.
          */
-#if 0
         st_reads(ft, sphere->shorten_check, 4);
 
         if (!strcmp(sphere->shorten_check,"ajkg"))
         {
-            st_fail_errno(ft,ST_EFMT,"File uses shorten compression, can not handle this.\n");
+            st_fail_errno(ft,ST_EFMT,"File uses shorten compression, cannot handle this.");
             free(buf);
             return(ST_EOF);
         }
-#endif
 
         free(buf);
         return (ST_SUCCESS);
@@ -194,7 +171,7 @@ int st_spherestartread(ft_t ft)
  * Return number of samples read.
  */
 
-st_ssize_t st_sphereread(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_sphereread(ft_t ft, st_sample_t *buf, st_size_t len) 
 {
     sphere_t sphere = (sphere_t) ft->priv;
 
@@ -208,7 +185,7 @@ st_ssize_t st_sphereread(ft_t ft, st_sample_t *buf, st_ssize_t len)
     return st_rawread(ft, buf, len);
 }
 
-int st_spherestartwrite(ft_t ft) 
+static int st_spherestartwrite(ft_t ft) 
 {
     int rc;
     int x;
@@ -220,7 +197,7 @@ int st_spherestartwrite(ft_t ft)
         return (ST_EOF);
     }
 
-    switch (ft->info.encoding)
+    switch (ft->signal.encoding)
     {
         case ST_ENCODING_ULAW:
         case ST_ENCODING_SIGN2:
@@ -247,7 +224,7 @@ int st_spherestartwrite(ft_t ft)
         
 }
 
-st_ssize_t st_spherewrite(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_spherewrite(ft_t ft, const st_sample_t *buf, st_size_t len) 
 {
     sphere_t sphere = (sphere_t) ft->priv;
 
@@ -255,7 +232,7 @@ st_ssize_t st_spherewrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
     return st_rawwrite(ft, buf, len);
 }
 
-int st_spherestopwrite(ft_t ft) 
+static int st_spherestopwrite(ft_t ft) 
 {
     int rc;
     char buf[128];
@@ -266,40 +243,34 @@ int st_spherestopwrite(ft_t ft)
     if (rc)
         return rc;
 
-    if (st_seeki(ft, 0L, 0) != 0)
+    if (st_seeki(ft, 0, 0) != 0)
     {
-        st_fail_errno(ft,errno,"Could not rewird output file to rewrite sphere header.\n");
+        st_fail_errno(ft,errno,"Could not rewird output file to rewrite sphere header.");
         return (ST_EOF);
     }
 
     st_writes(ft, "NIST_1A\n");
     st_writes(ft, "   1024\n");
 
-    samples = sphere->numSamples/ft->info.channels;
+    samples = sphere->numSamples/ft->signal.channels;
     sprintf(buf, "sample_count -i %ld\n", samples);
     st_writes(ft, buf);
 
-    sprintf(buf, "sample_n_bytes -i %d\n", ft->info.size);
+    sprintf(buf, "sample_n_bytes -i %d\n", ft->signal.size);
     st_writes(ft, buf);
 
-    sprintf(buf, "channel_count -i %d\n", ft->info.channels);
+    sprintf(buf, "channel_count -i %d\n", ft->signal.channels);
     st_writes(ft, buf);
 
-    if (ft->swap)
-    {
-        sprintf(buf, "sample_byte_format -s2 %s\n", ST_IS_BIGENDIAN ? "01" : "10");
-    }
-    else
-    {
-        sprintf(buf, "sample_byte_format -s2 %s\n", ST_IS_BIGENDIAN ? "10" : "01");
-    }
+    sprintf(buf, "sample_byte_format -s2 %s\n",
+        ft->signal.reverse_bytes != ST_IS_BIGENDIAN ? "10" : "01");
     st_writes(ft, buf);
 
-    rate = ft->info.rate;
+    rate = ft->signal.rate;
     sprintf(buf, "sample_rate -i %ld\n", rate);
     st_writes(ft, buf);
 
-    if (ft->info.encoding == ST_ENCODING_ULAW)
+    if (ft->signal.encoding == ST_ENCODING_ULAW)
         st_writes(ft, "sample_coding -s4 ulaw\n");
     else
         st_writes(ft, "sample_coding -s3 pcm\n");
@@ -310,15 +281,16 @@ int st_spherestopwrite(ft_t ft)
 }
 
 /* NIST Sphere File */
-static char *spherenames[] = {
+static const char *spherenames[] = {
   "sph",
+  "nist",
   NULL
 };
 
 static st_format_t st_sphere_format = {
   spherenames,
   NULL,
-  ST_FILE_STEREO,
+  0,
   st_spherestartread,
   st_sphereread,
   st_rawstopread,
