@@ -1,11 +1,3 @@
-/*
- * Copyright 1997 Chris Bagwell And Sundry Contributors
- * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained.
- * Rick Richardson, Lance Norskog And Sundry Contributors are not
- * responsible for the consequences of using this software.
- */
-
 /* Direct to Sun Audio Driver
  *
  * Added by Chris Bagwell (cbagwell@sprynet.com) on 2/26/96
@@ -15,31 +7,36 @@
  *
  */
 
+/*
+ * Copyright 1997 Chris Bagwell And Sundry Contributors
+ * This source code is freely redistributable and may be used for
+ * any purpose.  This copyright notice must be maintained.
+ * Rick Richardson, Lance Norskog And Sundry Contributors are not
+ * responsible for the consequences of using this software.
+ */
+
 #include "st_i.h"
 
-#if     defined(HAVE_SUNAUDIO)
+#ifdef HAVE_SUN_AUDIO
 
 #include <sys/ioctl.h>
-#if defined(__SVR4) || defined(__NetBSD__) || defined(__OpenBSD__)
-#if defined(__NetBSD__) || defined(__OpenBSD__)
-#include <sys/types.h>  /* This should be in audioio.h itself but its not */
-#endif
-#include <sys/audioio.h>
-#else
+#include <sys/types.h>
+#ifdef HAVE_SUN_AUDIOIO_H
 #include <sun/audioio.h>
+#else
+#ifdef HAVE_SYS_AUDIOIO_H
+#include <sys/audioio.h>
+#endif
 #endif
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 #if !defined(__NetBSD__) && !defined(__OpenBSD__)
 #include <stropts.h>
 #endif
-#ifdef _HAVE_MALLOC_H
-#include <malloc.h>
-#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <string.h>
+
 /*
  * Do anything required before you start reading samples.
  * Read file header.
@@ -47,28 +44,26 @@
  *      size and encoding of samples,
  *      mono/stereo/quad.
  */
-int st_sunstartread(ft_t ft)
+static int st_sunstartread(ft_t ft)
 {
-    int samplesize, encoding;
+    st_fileinfo_t *file = (st_fileinfo_t *)ft->priv;
+    st_size_t samplesize, encoding;
     audio_info_t audio_if;
 #ifdef __SVR4
     audio_device_t audio_dev;
 #endif
     char simple_hw=0;
 
-    /* Hard code for now. */
-    ft->file.count = 0;
-    ft->file.pos = 0;
-    ft->file.eof = 0;
-    ft->file.size = 1024;
-    if ((ft->file.buf = malloc (ft->file.size)) == NULL) {
-        st_fail_errno(ft,ST_ENOMEM,"unable to allocate input buffer of size %d", ft->file.size);
-        return ST_EOF;
-    }
+    /* Hard-code for now. */
+    file->count = 0;
+    file->pos = 0;
+    file->eof = 0;
+    file->size = 1024;
+    file->buf = xmalloc (file->size);
 
-    if (ft->info.rate == 0.0) ft->info.rate = 8000;
-    if (ft->info.size == -1) ft->info.size = ST_SIZE_BYTE;
-    if (ft->info.encoding == -1) ft->info.encoding = ST_ENCODING_ULAW;
+    if (ft->signal.rate == 0.0) ft->signal.rate = 8000;
+    if (ft->signal.size == -1) ft->signal.size = ST_SIZE_BYTE;
+    if (ft->signal.encoding == ST_ENCODING_UNKNOWN) ft->signal.encoding = ST_ENCODING_ULAW;
 
 #ifdef __SVR4
     /* Read in old values, change to what we need and then send back */
@@ -76,68 +71,69 @@ int st_sunstartread(ft_t ft)
         st_fail_errno(ft,errno,"Unable to get device information.");
         return(ST_EOF);
     }
-    st_report("Hardware detected:  %s\n",audio_dev.name);
+    st_report("Hardware detected:  %s",audio_dev.name);
     if (strcmp("SUNW,am79c30",audio_dev.name) == 0)
     {
         simple_hw = 1;
     }
 #endif
 
-    // If simple hardware detected in force data to ulaw.
+    /* If simple hardware detected in force data to ulaw. */
     if (simple_hw)
     {
-        if (ft->info.size == ST_SIZE_BYTE)
+        if (ft->signal.size == ST_SIZE_BYTE)
         {
-            if (ft->info.encoding != ST_ENCODING_ULAW &&
-                ft->info.encoding != ST_ENCODING_ALAW)
+            if (ft->signal.encoding != ST_ENCODING_ULAW &&
+                ft->signal.encoding != ST_ENCODING_ALAW)
             {
                 st_report("Warning: Detected simple hardware.  Forcing output to ULAW");
-                ft->info.encoding = ST_ENCODING_ULAW;
+                ft->signal.encoding = ST_ENCODING_ULAW;
             }
         }
-        else if (ft->info.size == ST_SIZE_WORD)
+        else if (ft->signal.size == ST_SIZE_16BIT)
         {
             st_report("Warning: Detected simple hardware.  Forcing output to ULAW");
-            ft->info.size = ST_SIZE_BYTE;
-            ft->info.encoding = ST_ENCODING_ULAW;
+            ft->signal.size = ST_SIZE_BYTE;
+            ft->signal.encoding = ST_ENCODING_ULAW;
         }
     }
 
-    if (ft->info.size == ST_SIZE_BYTE) {
+    if (ft->signal.size == ST_SIZE_BYTE) {
         samplesize = 8;
-        if (ft->info.encoding != ST_ENCODING_ULAW &&
-            ft->info.encoding != ST_ENCODING_ALAW &&
-            ft->info.encoding != ST_ENCODING_SIGN2) {
-            st_fail_errno(ft,ST_EFMT,"Sun Audio driver only supports ULAW, ALAW, and Signed Linear for bytes.");
+        if (ft->signal.encoding != ST_ENCODING_ULAW &&
+            ft->signal.encoding != ST_ENCODING_ALAW &&
+            ft->signal.encoding != ST_ENCODING_SIGN2) {
+            st_fail_errno(ft,ST_EFMT,"Sun audio driver only supports ULAW, ALAW, and signed linear for bytes.");
                 return (ST_EOF);
         }
-        if ((ft->info.encoding == ST_ENCODING_ULAW ||
-             ft->info.encoding == ST_ENCODING_ALAW) && ft->info.channels == 2)
+        if ((ft->signal.encoding == ST_ENCODING_ULAW ||
+             ft->signal.encoding == ST_ENCODING_ALAW) && 
+            ft->signal.channels == 2)
         {
-            st_report("Warning: only support mono for ULAW and ALAW data.  Forcing to mono");
-            ft->info.channels = 2;
+            st_report("Warning: only support mono for ULAW and ALAW data.  Forcing to mono.");
+            ft->signal.channels = 1;
         }
     }
-    else if (ft->info.size == ST_SIZE_WORD) {
+    else if (ft->signal.size == ST_SIZE_16BIT) {
         samplesize = 16;
-        if (ft->info.encoding != ST_ENCODING_SIGN2) {
-            st_fail_errno(ft,ST_EFMT,"Sun Audio driver only supports Signed Linear for words.");
+        if (ft->signal.encoding != ST_ENCODING_SIGN2) {
+            st_fail_errno(ft,ST_EFMT,"Sun audio driver only supports signed linear for words.");
             return(ST_EOF);
         }
     }
     else {
-        st_fail_errno(ft,ST_EFMT,"Sun Audio driver only supports bytes and words");
+        st_fail_errno(ft,ST_EFMT,"Sun audio driver only supports bytes and words");
         return(ST_EOF);
     }
 
-    if (ft->info.channels == -1) ft->info.channels = 1;
-    else if (ft->info.channels > 1) {
-        st_report("Warning: some sun audio devices can not play stereo");
-        st_report("at all or sometime only with signed words.  If the");
+    if (ft->signal.channels == 0) ft->signal.channels = 1;
+    else if (ft->signal.channels > 1) {
+        st_report("Warning: some Sun audio devices can not play stereo");
+        st_report("at all or sometimes only with signed words.  If the");
         st_report("sound seems sluggish then this is probably the case.");
         st_report("Try forcing output to signed words or use the avg");
         st_report("filter to reduce the number of channels.");
-        ft->info.channels = 2;
+        ft->signal.channels = 2;
     }
 
     /* Read in old values, change to what we need and then send back */
@@ -146,11 +142,11 @@ int st_sunstartread(ft_t ft)
         return(ST_EOF);
     }
     audio_if.record.precision = samplesize;
-    audio_if.record.channels = ft->info.channels;
-    audio_if.record.sample_rate = ft->info.rate;
-    if (ft->info.encoding == ST_ENCODING_ULAW)
+    audio_if.record.channels = ft->signal.channels;
+    audio_if.record.sample_rate = ft->signal.rate;
+    if (ft->signal.encoding == ST_ENCODING_ULAW)
         encoding = AUDIO_ENCODING_ULAW;
-    else if (ft->info.encoding == ST_ENCODING_ALAW)
+    else if (ft->signal.encoding == ST_ENCODING_ALAW)
         encoding = AUDIO_ENCODING_ALAW;
     else
         encoding = AUDIO_ENCODING_LINEAR;
@@ -161,11 +157,11 @@ int st_sunstartread(ft_t ft)
         st_fail_errno(ft,errno,"Unable to initialize sample size for /dev/audio");
         return(ST_EOF);
     }
-    if (audio_if.record.channels != ft->info.channels) {
+    if (audio_if.record.channels != ft->signal.channels) {
         st_fail_errno(ft,errno,"Unable to initialize number of channels for /dev/audio");
         return(ST_EOF);
     }
-    if (audio_if.record.sample_rate != ft->info.rate) {
+    if (audio_if.record.sample_rate != ft->signal.rate) {
         st_fail_errno(ft,errno,"Unable to initialize rate for /dev/audio");
         return(ST_EOF);
     }
@@ -180,29 +176,27 @@ int st_sunstartread(ft_t ft)
     ioctl(fileno(ft->fp), I_FLUSH, FLUSHR);
 #endif
     /* Change to non-buffered I/O*/
-    setvbuf(ft->fp, NULL, _IONBF, sizeof(char) * ft->file.size);
+    setvbuf(ft->fp, NULL, _IONBF, sizeof(char) * file->size);
 
     return (ST_SUCCESS);
 }
 
-int st_sunstartwrite(ft_t ft)
+static int st_sunstartwrite(ft_t ft)
 {
-    int samplesize, encoding;
+    st_fileinfo_t *file = (st_fileinfo_t *)ft->priv;
+    st_size_t samplesize, encoding;
     audio_info_t audio_if;
 #ifdef __SVR4
     audio_device_t audio_dev;
 #endif
     char simple_hw=0;
 
-    /* Hard code for now. */
-    ft->file.count = 0;
-    ft->file.pos = 0;
-    ft->file.eof = 0;
-    ft->file.size = 1024;
-    if ((ft->file.buf = malloc (ft->file.size)) == NULL) {
-        st_fail_errno(ft,ST_ENOMEM,"unable to allocate output buffer of size %d", ft->file.size);
-        return(ST_EOF);
-    }
+    /* Hard-code for now. */
+    file->count = 0;
+    file->pos = 0;
+    file->eof = 0;
+    file->size = 1024;
+    file->buf = xmalloc (file->size);
 
 #ifdef __SVR4
     /* Read in old values, change to what we need and then send back */
@@ -210,7 +204,7 @@ int st_sunstartwrite(ft_t ft)
         st_fail_errno(ft,errno,"Unable to get device information.");
         return(ST_EOF);
     }
-    st_report("Hardware detected:  %s\n",audio_dev.name);
+    st_report("Hardware detected:  %s",audio_dev.name);
     if (strcmp("SUNW,am79c30",audio_dev.name) == 0)
     {
         simple_hw = 1;
@@ -219,60 +213,63 @@ int st_sunstartwrite(ft_t ft)
 
     if (simple_hw)
     {
-        if (ft->info.size == ST_SIZE_BYTE)
+        if (ft->signal.size == ST_SIZE_BYTE)
         {
-            if (ft->info.encoding != ST_ENCODING_ULAW &&
-                ft->info.encoding != ST_ENCODING_ALAW)
+            if (ft->signal.encoding != ST_ENCODING_ULAW &&
+                ft->signal.encoding != ST_ENCODING_ALAW)
             {
                 st_report("Warning: Detected simple hardware.  Forcing output to ULAW");
-                ft->info.encoding = ST_ENCODING_ULAW;
+                ft->signal.encoding = ST_ENCODING_ULAW;
             }
         }
-        else if (ft->info.size == ST_SIZE_WORD)
+        else if (ft->signal.size == ST_SIZE_16BIT)
         {
             st_report("Warning: Detected simple hardware.  Forcing output to ULAW");
-            ft->info.size = ST_SIZE_BYTE;
-            ft->info.encoding = ST_ENCODING_ULAW;
+            ft->signal.size = ST_SIZE_BYTE;
+            ft->signal.encoding = ST_ENCODING_ULAW;
         }
     }
 
-    if (ft->info.rate == 0.0) ft->info.rate = 8000;
-    if (ft->info.size == -1) ft->info.size = ST_SIZE_BYTE;
-    if (ft->info.encoding == -1) ft->info.encoding = ST_ENCODING_ULAW;
+    if (ft->signal.rate == 0.0) ft->signal.rate = 8000;
+    if (ft->signal.size == -1) ft->signal.size = ST_SIZE_BYTE;
+    if (ft->signal.encoding == ST_ENCODING_UNKNOWN) 
+        ft->signal.encoding = ST_ENCODING_ULAW;
 
-    if (ft->info.size == ST_SIZE_BYTE) {
+    if (ft->signal.size == ST_SIZE_BYTE) 
+    {
         samplesize = 8;
-        if (ft->info.encoding != ST_ENCODING_ULAW &&
-            ft->info.encoding != ST_ENCODING_ALAW &&
-            ft->info.encoding != ST_ENCODING_SIGN2) {
+        if (ft->signal.encoding != ST_ENCODING_ULAW &&
+            ft->signal.encoding != ST_ENCODING_ALAW &&
+            ft->signal.encoding != ST_ENCODING_SIGN2) {
             st_report("Sun Audio driver only supports ULAW, ALAW, and Signed Linear for bytes.");
             st_report("Forcing to ULAW");
-            ft->info.encoding = ST_ENCODING_ULAW;
+            ft->signal.encoding = ST_ENCODING_ULAW;
         }
-        if ((ft->info.encoding == ST_ENCODING_ULAW ||
-             ft->info.encoding == ST_ENCODING_ALAW) && ft->info.channels == 2)
+        if ((ft->signal.encoding == ST_ENCODING_ULAW ||
+             ft->signal.encoding == ST_ENCODING_ALAW) && 
+            ft->signal.channels == 2)
         {
-            st_report("Warning: only support mono for ULAW and ALAW data.  Forcing to mono");
-            ft->info.channels = 2;
+            st_report("Warning: only support mono for ULAW and ALAW data.  Forcing to mono.");
+            ft->signal.channels = 1;
         }
 
     }
-    else if (ft->info.size == ST_SIZE_WORD) {
+    else if (ft->signal.size == ST_SIZE_16BIT) {
         samplesize = 16;
-        if (ft->info.encoding != ST_ENCODING_SIGN2) {
+        if (ft->signal.encoding != ST_ENCODING_SIGN2) {
             st_report("Sun Audio driver only supports Signed Linear for words.");
             st_report("Forcing to Signed Linear");
-            ft->info.encoding = ST_ENCODING_SIGN2;
+            ft->signal.encoding = ST_ENCODING_SIGN2;
         }
     }
     else {
         st_report("Sun Audio driver only supports bytes and words");
-        ft->info.size = ST_SIZE_WORD;
+        ft->signal.size = ST_SIZE_16BIT;
         samplesize = 16;
     }
 
-    if (ft->info.channels == -1) ft->info.channels = 1;
-    else if (ft->info.channels > 1) ft->info.channels = 2;
+    if (ft->signal.channels == 0) ft->signal.channels = 1;
+    else if (ft->signal.channels > 1) ft->signal.channels = 2;
 
     /* Read in old values, change to what we need and then send back */
     if (ioctl(fileno(ft->fp), AUDIO_GETINFO, &audio_if) < 0) {
@@ -280,11 +277,11 @@ int st_sunstartwrite(ft_t ft)
         return(ST_EOF);
     }
     audio_if.play.precision = samplesize;
-    audio_if.play.channels = ft->info.channels;
-    audio_if.play.sample_rate = ft->info.rate;
-    if (ft->info.encoding == ST_ENCODING_ULAW)
+    audio_if.play.channels = ft->signal.channels;
+    audio_if.play.sample_rate = ft->signal.rate;
+    if (ft->signal.encoding == ST_ENCODING_ULAW)
         encoding = AUDIO_ENCODING_ULAW;
-    else if (ft->info.encoding == ST_ENCODING_ALAW)
+    else if (ft->signal.encoding == ST_ENCODING_ALAW)
         encoding = AUDIO_ENCODING_ALAW;
     else
         encoding = AUDIO_ENCODING_LINEAR;
@@ -295,11 +292,11 @@ int st_sunstartwrite(ft_t ft)
         st_fail_errno(ft,errno,"Unable to initialize sample size for /dev/audio");
         return(ST_EOF);
     }
-    if (audio_if.play.channels != ft->info.channels) {
+    if (audio_if.play.channels != ft->signal.channels) {
         st_fail_errno(ft,errno,"Unable to initialize number of channels for /dev/audio");
         return(ST_EOF);
     }
-    if (audio_if.play.sample_rate != ft->info.rate) {
+    if (audio_if.play.sample_rate != ft->signal.rate) {
         st_fail_errno(ft,errno,"Unable to initialize rate for /dev/audio");
         return(ST_EOF);
     }
@@ -308,13 +305,13 @@ int st_sunstartwrite(ft_t ft)
         return(ST_EOF);
     }
     /* Change to non-buffered I/O */
-    setvbuf(ft->fp, NULL, _IONBF, sizeof(char) * ft->file.size);
+    setvbuf(ft->fp, NULL, _IONBF, sizeof(char) * file->size);
 
     return (ST_SUCCESS);
 }
 
 /* Sun /dev/audio player */
-static char *sunnames[] = {
+static const char *sunnames[] = {
   "sunau",
   NULL
 };
@@ -322,7 +319,7 @@ static char *sunnames[] = {
 static st_format_t st_sun_format = {
   sunnames,
   NULL,
-  ST_FILE_STEREO,
+  ST_FILE_DEVICE,
   st_sunstartread,
   st_rawread,
   st_rawstopread,
@@ -336,4 +333,5 @@ const st_format_t *st_sun_format_fn(void)
 {
     return &st_sun_format;
 }
+
 #endif
