@@ -14,7 +14,7 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
-#include "st_i.h"
+#include "synth.h"
 
 static st_effect_t st_synth_effect;
 
@@ -24,17 +24,40 @@ static st_effect_t st_synth_effect;
 #define SYNTH_SQUARE     1
 #define SYNTH_SAWTOOTH   2
 #define SYNTH_TRIANGLE   3
-#define SYNTH_TRAPETZ    4
+#define SYNTH_TRAPEZIUM  4
+#define SYNTH_TRAPETZ    SYNTH_TRAPEZIUM /* Deprecated name for trapezium */
 #define SYNTH_WHITENOISE 5
+#define SYNTH_NOISE      SYNTH_WHITENOISE /* Just a handy alias */
 #define SYNTH_PINKNOISE  6
 #define SYNTH_BROWNNOISE 7
-#define SYNTH_VOICENOISE 8
-#define SYNTH_EXP        9
+#define SYNTH_EXP        8
 
 #define SYNTH_CREATE    0x000
 #define SYNTH_MIX       0x100
 #define SYNTH_AMOD      0x200
 #define SYNTH_FMOD      0x400
+
+enum_item const synth_type[] = {
+  ENUM_ITEM(SYNTH_,SINE      )
+  ENUM_ITEM(SYNTH_,SQUARE    )
+  ENUM_ITEM(SYNTH_,SAWTOOTH  )
+  ENUM_ITEM(SYNTH_,TRIANGLE  )
+  ENUM_ITEM(SYNTH_,TRAPEZIUM )
+  ENUM_ITEM(SYNTH_,TRAPETZ   )
+  ENUM_ITEM(SYNTH_,WHITENOISE)
+  ENUM_ITEM(SYNTH_,NOISE     )
+  ENUM_ITEM(SYNTH_,PINKNOISE )
+  ENUM_ITEM(SYNTH_,BROWNNOISE)
+  ENUM_ITEM(SYNTH_,EXP       )
+  {0, 0}};
+
+enum_item const combine_type[] = {
+  ENUM_ITEM(SYNTH_,CREATE)
+  ENUM_ITEM(SYNTH_,MIX   )
+  ENUM_ITEM(SYNTH_,AMOD  )
+  ENUM_ITEM(SYNTH_,FMOD  )
+  {0, 0}};
+
 /* do not ask me for the colored noise, i copied the 
  * algorithm somewhere...
  */
@@ -42,7 +65,6 @@ static st_effect_t st_synth_effect;
 #define PINKNOISE_FAC   (5000.0/32768.0)
 #define LOG_10_20     0.1151292546497022842009e0
 
-/*#define TIMERES 1000*/
 #define MAXCHAN 4
 
 
@@ -74,7 +96,7 @@ typedef struct{
 } PinkNoise;
 
 /* Setup PinkNoise structure for N rows of generators. */
-void InitializePinkNoise( PinkNoise *pink, int numRows )
+static void InitializePinkNoise( PinkNoise *pink, int numRows )
 {
         int i;
         long pmax;
@@ -89,7 +111,7 @@ void InitializePinkNoise( PinkNoise *pink, int numRows )
 }
 
 /* Generate Pink noise values between -1.0 and +1.0 */
-float GeneratePinkNoise( PinkNoise *pink )
+static float GeneratePinkNoise( PinkNoise *pink )
 {
         long newRandom;
         long sum;
@@ -227,7 +249,6 @@ int st_synth_getopts(eff_t effp, int n, char **argv)
     /* set default parameters */
     synth->length = 0; /* use length of input file */
     synth->length_str = 0;
-    synth->max = ST_SAMPLE_MAX;
     for(c=0;c<MAXCHAN;c++){
         synth->freq[c] = 440.0;
         synth->freq2[c] = 440.0;
@@ -253,17 +274,11 @@ int st_synth_getopts(eff_t effp, int n, char **argv)
     
 
     /* read length if given ( if first par starts with digit )*/
-    if( isdigit((int)argv[argn][0])) {
-        synth->length_str = (char *)malloc(strlen(argv[argn])+1);
-        if (!synth->length_str)
-        {
-            st_fail("Could not allocate memeory");
-            return(ST_EOF);
-        }
+    if( isdigit((int)argv[argn][0]) || argv[argn][0] == '.') {
+        synth->length_str = (char *)xmalloc(strlen(argv[argn])+1);
         strcpy(synth->length_str,argv[argn]);
         /* Do a dummy parse of to see if it will fail */
-        if (st_parsesamples(0, synth->length_str, &synth->length, 't') !=
-                ST_SUCCESS)
+        if (st_parsesamples(0, synth->length_str, &synth->length, 't') == NULL)
         {
             st_fail(st_synth_effect.usage);
             return (ST_EOF);
@@ -271,121 +286,64 @@ int st_synth_getopts(eff_t effp, int n, char **argv)
         argn++;
     }
     /* for one or more channel */
-    /* type [mix] [f1[-f2]] [p0] [p1] [p2] [p3] [p4] */
-    for(c=0;c<MAXCHAN;c++){
-        if(n > argn){
-            /* next par must be type */
-            if( strcasecmp(argv[argn],"sine")==0){
-                synth->type[c]=SYNTH_SINE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"square")==0){
-                synth->type[c]=SYNTH_SQUARE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"sawtooth")==0){
-                synth->type[c]=SYNTH_SAWTOOTH;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"triangle")==0){
-                synth->type[c]=SYNTH_TRIANGLE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"exp")==0){
-                synth->type[c]=SYNTH_EXP;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"trapetz")==0){
-                synth->type[c]=SYNTH_TRAPETZ;
-                argn++;
-            }else if( strcasecmp(argv[argn],"whitenoise")==0){
-                synth->type[c]=SYNTH_WHITENOISE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"noise")==0){
-                synth->type[c]=SYNTH_WHITENOISE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"pinknoise")==0){
-                synth->type[c]=SYNTH_PINKNOISE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"brownnoise")==0){
-                synth->type[c]=SYNTH_BROWNNOISE;
-                argn++; /* 1 */
-            }else if( strcasecmp(argv[argn],"voicenoise")==0){
-                synth->type[c]=SYNTH_VOICENOISE;
-                argn++; /* 1 */
-            }else{
-                /* type not given, error */
-                st_warn("synth: no type given");
-                st_fail(st_synth_effect.usage);
-                return(ST_EOF);
-            }
-            if(n > argn){
-                /* maybe there is a mix-type in next arg */
-                if(strcasecmp(argv[argn],"create")==0){
-                    synth->mix[c]=SYNTH_CREATE;
-                    argn++;
-                }else if(strcasecmp(argv[argn],"mix")==0){
-                    synth->mix[c]=SYNTH_MIX;
-                argn++;
-                }else if(strcasecmp(argv[argn],"amod")==0){
-                    synth->mix[c]=SYNTH_AMOD;
-                    argn++;
-                }else if(strcasecmp(argv[argn],"fmod")==0){
-                    synth->mix[c]=SYNTH_FMOD;
-                    argn++;
-                }
-                if(n > argn){
-                    /* read frequency's if given */
-                    synth->freq[c]= StringToFreq(argv[argn],&hlp);
-                    synth->freq2[c] = synth->freq[c];
-                    if(synth->freq[c] < 0.0){
-                        st_warn("synth: illegal freq");
-                        st_fail(st_synth_effect.usage);
-                        return(ST_EOF);
-                    }
-                    if(*hlp=='-') {
-                        /* freq2 given ! */
-                        char *hlp2;
-                        synth->freq2[c]=StringToFreq(hlp+1,&hlp2);
-                        if(synth->freq2[c] < 0.0){
-                            st_warn("synth: illegal freq2");
-                            st_fail(st_synth_effect.usage);
-                            return(ST_EOF);
-                        }
-                    }
-                    argn++;
-                    i=0; 
-                    /* read rest of parameters */
-                    while(n > argn){
-                        if( ! isdigit((int)argv[argn][0]) ){
-                            /* not a digit, must be type of next channel */
-                            break;
-                        }
-                        if( i >= PCOUNT) {
-                            st_warn("synth: too many parameters");
-                            st_fail(st_synth_effect.usage);
-                            return(ST_EOF);
-                            
-                        }
-                        synth->par[c][i]=strtod(argv[argn],&hlp);
-                        if(hlp==argv[argn]){
-                                /* error in number */
-                            st_warn("synth: parameter error");
-                            st_fail(st_synth_effect.usage);
-                            return(ST_EOF);
-                        } 
-                        i++;
-                        argn++;
-                    }/* .. while */
-                    if(n > argn){
-                        /* got here by 'break', scan parms for next chan */
-                    }else{
-                        break;
-                    }
-                }
-            }
-        } /* if n > argn */
-    }/* for .. */
+    /* type [combine] [f1[-f2]] [p0] [p1] [p2] [p3] [p4] */
+    for (c = 0; c < MAXCHAN && n > argn; c++) {
+      enum_item const * p = find_enum_text(argv[argn], synth_type);
+      if (p == NULL) {
+        st_fail("no type given");
+        return ST_EOF;
+      }
+      synth->type[c] = p->value;
+      if (++argn == n) break;
+
+      /* maybe there is a combine-type in next arg */
+      p = find_enum_text(argv[argn], combine_type);
+      if (p != NULL) {
+        synth->mix[c] = p->value;
+        if (++argn == n) break;
+      }
+
+      /* read frequencies if given */
+      if (isdigit((int)argv[argn][0]) || argv[argn][0] == '%') {
+        synth->freq2[c] = synth->freq[c] = StringToFreq(argv[argn], &hlp);
+        if (synth->freq[c] < 0) {
+          st_fail("invalid freq");
+          return ST_EOF;
+        }
+        if (*hlp == '-') { /* freq2 given? */
+          char * hlp2;
+          synth->freq2[c] = StringToFreq(hlp + 1, &hlp2);
+          if (synth->freq2[c] < 0) {
+            st_fail("invalid freq2");
+            return ST_EOF;
+          }
+          if (synth->length_str == NULL) {
+            st_fail("length must be given when using freq2");
+            return ST_EOF;
+          }
+        }
+        if (++argn == n) break;
+      }
+
+      /* read rest of parameters */
+      for (i = 0; argn < n && isdigit((int)argv[argn][0]); ++i, ++argn) {
+        if (i == PCOUNT) {
+          st_fail("too many parameters");
+          return ST_EOF;
+        }
+        synth->par[c][i] = strtod(argv[argn], &hlp);
+        if (hlp == argv[argn]) {
+          st_fail("parameter error");
+          return ST_EOF;
+        }
+      }
+      if (argn == n) break;
+    }
 
     /* make some intelligent parameter initialization for channels
      * where no parameters were given
      *
-     * - of only parms for one channel were given, copy to ther channels
+     * - if only parms for one channel were given, copy to other channels
      * - if parm for 2 channels were given, copy to channel 1->3, 2->4
      * - if parm for 3 channels were given, copy 2->4
      */
@@ -412,13 +370,14 @@ int st_synth_start(eff_t effp)
     int i;
     int c;
     synth_t synth = (synth_t) effp->priv;
+    int shift_for_max = (4 - min(effp->outinfo.size, 4)) << 3;
 
-    st_initrand();
+    synth->max = (ST_SAMPLE_MAX >> shift_for_max) << shift_for_max;
 
     if (synth->length_str)
     {
         if (st_parsesamples(effp->ininfo.rate, synth->length_str,
-                            &synth->length, 't') != ST_SUCCESS)
+                            &synth->length, 't') == NULL)
         {
             st_fail(st_synth_effect.usage);
             return(ST_EOF);
@@ -491,10 +450,28 @@ int st_synth_start(eff_t effp)
                 /* Initialize pink noise signals with different numbers of rows. */
                 InitializePinkNoise( &(synth->pinkn[c]),10+2*c);
                 break;
+            case SYNTH_EXP:
+                /* p2 is position of maximum*/
+                if (synth->par[c][2] < 0)
+                  synth->par[c][2] = 0.5;
+
+                /* p2 is amplitude */
+                if (synth->par[c][3] < 0)
+                  synth->par[c][3] = 1;
+                break;
+
             default:
                 break;
         }
+
+        st_debug("type=%i, mix=%i, length=%u, f1=%g, f2=%g",
+                synth->type[c], synth->mix[c], 
+                synth->length, synth->freq[c], synth->freq2[c]);
+        st_debug("p0=%g, p1=%g, p2=%g, p3=%g, p4=%g",
+                synth->par[c][0], synth->par[c][1],
+                synth->par[c][2], synth->par[c][3], synth->par[c][4]);
     }
+    st_debug("inchan=%i, rate=%i", (int)effp->ininfo.channels,synth->rate);
     return (ST_SUCCESS);
 }
 
@@ -666,7 +643,7 @@ static st_sample_t do_synth(st_sample_t iv, synth_t synth, int c){
             ov = iv * r ;
             break;
         default:
-            st_fail("synth: internel error 2");
+            st_fail("synth: internal error 2");
             break;
     }
 
@@ -678,14 +655,15 @@ static st_sample_t do_synth(st_sample_t iv, synth_t synth, int c){
 /*
  * Processed signed long samples from ibuf to obuf.
  */
-int st_synth_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+int st_synth_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                   st_size_t *isamp, st_size_t *osamp)
 {
     synth_t synth = (synth_t) effp->priv;
     int len; /* number of input samples */
-    int done;
+    int done = 0;
     int c;
     int chan=effp->ininfo.channels;
+    int result = ST_SUCCESS;
 
     if(chan > MAXCHAN ){
         st_fail("synth: can not operate with more than %d channels",MAXCHAN);
@@ -694,7 +672,8 @@ int st_synth_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
 
     len = ((*isamp > *osamp) ? *osamp : *isamp) / chan;
 
-    for(done = 0; done < len ; done++){
+    while (done < len && result == ST_SUCCESS)
+    {
         for(c=0;c<chan;c++){
             /* each channel is independent, but the algorithm is the same */
 
@@ -702,62 +681,27 @@ int st_synth_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         }
         ibuf+=chan;
         obuf+=chan;
+        ++done;
         synth->samples_done++;
-        if(synth->length > 0 ){
-            if( synth->samples_done > synth->length){
-                *osamp = done*chan;
-                return ST_EOF;
-                break;
-
-            }
+        if (synth->length > 0 && synth->samples_done == synth->length)
+        {
+            result = ST_EOF;
+        }
     }
-        
-    }
-    return (ST_SUCCESS);
-}
-
-/*
- * Drain out remaining samples if the effect generates any.
- */
-
-int st_synth_drain(eff_t effp, st_sample_t *obuf, st_size_t *osamp)
-{
-    *osamp = 0;
-    return ST_EOF;
-}
-
-/*
- * Do anything required when you stop reading samples.  
- *      (free allocated memory, etc.)
- */
-int st_synth_stop(eff_t effp)
-{
-    /* nothing to do */
-    return (ST_SUCCESS);
+    *isamp = *osamp = done * chan;
+    return result;
 }
 
 static st_effect_t st_synth_effect = {
   "synth",
-  "Usage: synth [length] type mix [freq[-freq2]] [off] [ph] [p1] [p2] [p3]\n"
-  "       <length> length in sec or hh:mm:ss.frac, 0=inputlength, default=0\n"
-  "       <type>   is sine, square, triangle, sawtooth, trapetz, exp,\n"
-  "                whitenoise, pinknoise, brownnoise, default=sine\n"
-  "       <mix>    is create, mix, amod, default=create\n"
-  "       <freq>   frequency at beginning in Hz, not used  for noise..\n"
-  "       <freq2>  frequency at end in Hz, not used for noise..\n"
-  "                <freq/2> can be given as %%n, where 'n' is the number of\n"
-  "                half notes in respect to A (440Hz)\n"
-  "       <off>    Bias (DC-offset)  of signal in percent, default=0\n"
-  "       <ph>     phase shift 0..100 shift phase 0..2*Pi, not used for noise..\n"
-  "       <p1>     square: Ton/Toff, triangle+trapetz: rising slope time (0..100)\n"
-  "       <p2>     trapetz: ON time (0..100)\n"
-  "       <p3>     trapetz: falling slope position (0..100)",
+  "Usage: synth [len] {[type] [combine] [freq[-freq2]] [off] [ph] [p1] [p2] [p3]}",
   ST_EFF_MCHAN,
   st_synth_getopts,
   st_synth_start,
   st_synth_flow,
-  st_synth_drain,
-  st_synth_stop
+  st_effect_nothing_drain,
+  st_effect_nothing,
+  st_effect_nothing
 };
 
 const st_effect_t *st_synth_effect_fn(void)
