@@ -37,7 +37,7 @@ typedef struct
 /*
  * Process options
  */
-int st_trim_getopts(eff_t effp, int n, char **argv) 
+static int st_trim_getopts(eff_t effp, int n, char **argv) 
 {
     trim_t trim = (trim_t) effp->priv;
 
@@ -46,31 +46,19 @@ int st_trim_getopts(eff_t effp, int n, char **argv)
      */
     switch (n) {
         case 2:
-            trim->length_str = (char *)malloc(strlen(argv[1])+1);
-            if (!trim->length_str)
-            {
-                st_fail("Could not allocate memory");
-                return(ST_EOF);
-            }
+            trim->length_str = (char *)xmalloc(strlen(argv[1])+1);
             strcpy(trim->length_str,argv[1]);
             /* Do a dummy parse to see if it will fail */
-            if (st_parsesamples(0, trim->length_str,
-                                &trim->length, 't') != ST_SUCCESS)
+            if (st_parsesamples(0, trim->length_str, &trim->length, 't') == NULL)
             {
                 st_fail(st_trim_effect.usage);
                 return(ST_EOF);
             }
         case 1:
-            trim->start_str = (char *)malloc(strlen(argv[0])+1);
-            if (!trim->start_str)
-            {
-                st_fail("Could not allocate memory");
-                return(ST_EOF);
-            }
+            trim->start_str = (char *)xmalloc(strlen(argv[0])+1);
             strcpy(trim->start_str,argv[0]);
             /* Do a dummy parse to see if it will fail */
-            if (st_parsesamples(0, trim->start_str,
-                                &trim->start, 't') != ST_SUCCESS)
+            if (st_parsesamples(0, trim->start_str, &trim->start, 't') == NULL)
             {
                 st_fail(st_trim_effect.usage);
                 return(ST_EOF);
@@ -88,12 +76,12 @@ int st_trim_getopts(eff_t effp, int n, char **argv)
 /*
  * Start processing
  */
-int st_trim_start(eff_t effp)
+static int st_trim_start(eff_t effp)
 {
     trim_t trim = (trim_t) effp->priv;
 
     if (st_parsesamples(effp->ininfo.rate, trim->start_str,
-                        &trim->start, 't') != ST_SUCCESS)
+                        &trim->start, 't') == NULL)
     {
         st_fail(st_trim_effect.usage);
         return(ST_EOF);
@@ -104,7 +92,7 @@ int st_trim_start(eff_t effp)
     if (trim->length_str)
     {
         if (st_parsesamples(effp->ininfo.rate, trim->length_str,
-                    &trim->length, 't') != ST_SUCCESS)
+                    &trim->length, 't') == NULL)
         {
             st_fail(st_trim_effect.usage);
             return(ST_EOF);
@@ -128,7 +116,7 @@ int st_trim_start(eff_t effp)
  * Place in buf[].
  * Return number of samples read.
  */
-int st_trim_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf, 
+static int st_trim_flow(eff_t effp, const st_sample_t *ibuf, st_sample_t *obuf, 
                  st_size_t *isamp, st_size_t *osamp)
 {
     int finished = 0;
@@ -191,32 +179,26 @@ int st_trim_flow(eff_t effp, st_sample_t *ibuf, st_sample_t *obuf,
         return (ST_SUCCESS);
 }
 
-/*
- * Do anything required when you stop reading samples.  
- * Don't close input file! 
- */
-int st_trim_stop(eff_t effp)
+static int delete(eff_t effp)
 {
     trim_t trim = (trim_t) effp->priv;
 
-    if (trim->start_str)
-        free(trim->start_str);
-    if (trim->length_str)
-        free(trim->length_str);
+    free(trim->start_str);
+    free(trim->length_str);
 
     return (ST_SUCCESS);
 }
 
-st_size_t st_trim_get_start(eff_t effp)
-{
-    trim_t trim = (trim_t)effp->priv;
-    return trim->start;
-}
+st_size_t st_trim_get_start(eff_t effp)          
+{        
+    trim_t trim = (trim_t)effp->priv;    
+    return trim->start;          
+}        
 
-void st_trim_clear_start(eff_t effp)
-{
-    trim_t trim = (trim_t)effp->priv;
-    trim->start = 0;
+void st_trim_clear_start(eff_t effp)     
+{        
+    trim_t trim = (trim_t)effp->priv;    
+    trim->start = 0;     
 }
 
 static st_effect_t st_trim_effect = {
@@ -227,7 +209,8 @@ static st_effect_t st_trim_effect = {
   st_trim_start,
   st_trim_flow,
   st_effect_nothing_drain,
-  st_effect_nothing
+  st_effect_nothing,
+  delete
 };
 
 const st_effect_t *st_trim_effect_fn(void)
