@@ -1,3 +1,11 @@
+/*
+ * util.c.
+ * Incorporate Jimen Ching's fixes for real library operation: Aug 3, 1994.
+ * Redo all work from scratch, unfortunately.
+ * Separate out all common variables used by effects & handlers,
+ * and utility routines for other main programs to use.
+ */
+
 /*
  * July 5, 1991
  * Copyright 1991 Lance Norskog And Sundry Contributors
@@ -8,79 +16,87 @@
  */
 
 #include "st_i.h"
+#include <stddef.h>
 #include <string.h>
 #include <ctype.h>
-
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
-/*
- * util.c.
- * Incorporate Jimen Ching's fixes for real library operation: Aug 3, 1994.
- * Redo all work from scratch, unfortunately.
- * Separate out all common variables used by effects & handlers,
- * and utility routines for other main programs to use.
- */
+st_output_message_handler_t st_output_message_handler = NULL;
+int st_output_verbosity_level = 2;
 
-/* export flags */
-/* FIXME: To be moved inside of fileop structure per handler. */
-int verbose = 0;        /* be noisy on stderr */
-
-/* FIXME:  These functions are user level concepts.  Move them outside
- * the ST library.
- */
-char *myname = 0;
-
-void st_report(const char *fmt, ...)
+void st_output_message(FILE *file, const char *filename, const char *fmt, va_list ap)
 {
-        va_list args;
+  char buffer[10];
+  char const * drivername;
+  char const * dot_pos;
+ 
+  drivername = strrchr(filename, '/');
+  if (drivername != NULL) {
+    ++drivername;
+  } else {
+    drivername = strrchr(filename, '\\');
+    if (drivername != NULL)
+      ++drivername;
+    else
+      drivername = filename;
+  }
 
-        if (! verbose)
-                return;
+  dot_pos = strrchr(drivername, '.');
+  if (dot_pos != NULL && dot_pos - drivername <= (ptrdiff_t)(sizeof(buffer) - 1)) {
+    strncpy(buffer, drivername, (size_t)(dot_pos - drivername));
+    buffer[dot_pos - drivername] = '\0';
+    drivername = buffer;
+  }
 
-        fprintf(stderr, "%s: ", myname);
-        va_start(args, fmt);
-        vfprintf(stderr, fmt, args);
-        va_end(args);
-        fprintf(stderr, "\n");
-}
-
-void st_warn(const char *fmt, ...)
-{
-        va_list args;
-
-        fprintf(stderr, "%s: ", myname);
-        va_start(args, fmt);
-
-        vfprintf(stderr, fmt, args);
-        va_end(args);
-        fprintf(stderr, "\n");
-}
-
-/* Warning: This function is depricated.  st_fail_errno() is
- * the preferred way so that applications can control printing
- * to their choice; not just stderr.
- */
-void st_fail(const char *fmt, ...)
-{
-        va_list args;
-
-        fprintf(stderr, "%s: ", myname);
-
-        va_start(args, fmt);
-        vfprintf(stderr, fmt, args);
-        va_end(args);
-        fprintf(stderr, "\n");
+  fprintf(file, "%s: ", drivername);
+  vfprintf(file, fmt, ap);
 }
 
 
-/* Warning: no error checking is done with errstr.  Be sure not to
- * go over the array limit ourself!  vsnprint does not seem to be
- * on all platforms so not using that.
+
+/* This is a bit of a hack.  It's useful to have the ST library
+ * report which driver (i.e. format or effect handler) is outputing
+ * the message.  Using the filename for this purpose is only an
+ * approximation, but it saves a lot of work. ;)
  */
+char const * st_message_filename = 0;
+
+
+
+static void st_emit_message(int level, char const *fmt, va_list ap)
+{
+  if (st_output_message_handler != NULL)
+    (*st_output_message_handler)(level, st_message_filename, fmt, ap);
+}
+
+
+
+#undef st_fail
+#undef st_warn
+#undef st_report
+#undef st_debug
+#undef st_debug_more
+#undef st_debug_most
+
+#define ST_MESSAGE_FUNCTION(name,level) \
+void name(char const * fmt, ...) \
+{ \
+  va_list args; \
+\
+  va_start(args, fmt); \
+  st_emit_message(level, fmt, args); \
+  va_end(args); \
+}
+
+ST_MESSAGE_FUNCTION(st_fail  , 1)
+ST_MESSAGE_FUNCTION(st_warn  , 2)
+ST_MESSAGE_FUNCTION(st_report, 3)
+ST_MESSAGE_FUNCTION(st_debug , 4)
+ST_MESSAGE_FUNCTION(st_debug_more , 5)
+ST_MESSAGE_FUNCTION(st_debug_most , 6)
+
+#undef ST_MESSAGE_FUNCTION
+
 void st_fail_errno(ft_t ft, int st_errno, const char *fmt, ...)
 {
         va_list args;
@@ -88,73 +104,44 @@ void st_fail_errno(ft_t ft, int st_errno, const char *fmt, ...)
         ft->st_errno = st_errno;
 
         va_start(args, fmt);
+#ifdef _MSC_VER
         vsprintf(ft->st_errstr, fmt, args);
+#else
+        vsnprintf(ft->st_errstr, sizeof(ft->st_errstr), fmt, args);
+#endif
         va_end(args);
         ft->st_errstr[255] = '\0';
 }
 
-int st_is_bigendian(void)
-{
-    int b;
-    char *p;
-
-    b = 1;
-    p = (char *) &b;
-    if (!*p)
-        return 1;
-    else
-        return 0;
-}
-
-int st_is_littleendian(void)
-{
-    int b;
-    char *p;
-
-    b = 1;
-    p = (char *) &b;
-    if (*p)
-        return 1;
-    else
-        return 0;
-}
-
-int strcmpcase(char *s1, char *s2)
-{
-        while(*s1 && *s2 && (tolower(*s1) == tolower(*s2)))
-                s1++, s2++;
-        return *s1 - *s2;
-}
-
 /*
  * Check that we have a known format suffix string.
  */
-int st_gettype(ft_t formp)
+int st_gettype(ft_t formp, st_bool is_file_extension)
 {
-    char **list;
+    const char * const *list;
     int i;
     const st_format_t *f;
 
-    if (! formp->filetype){
-        st_fail_errno(formp,
-                      ST_EFMT,
-                      "Filetype was not specified");
-        return(ST_EFMT);
+    if (!formp->filetype) {
+        st_fail_errno(formp, ST_EFMT, "Filetype was not specified");
+        return ST_EFMT;
     }
-    for(i = 0; st_format_fns[i]; i++) {
+    for (i = 0; st_format_fns[i]; i++) {
         f = st_format_fns[i]();
-        for(list = f->names; *list; list++) {
-            char *s1 = *list, *s2 = formp->filetype;
-            if (! strcmpcase(s1, s2))
+        if (is_file_extension && (f->flags & ST_FILE_DEVICE))
+          continue; /* don't match device names in real file names */
+        for (list = f->names; *list; list++) {
+            const char *s1 = *list, *s2 = formp->filetype;
+            if (!strcasecmp(s1, s2))
                 break;  /* not a match */
         }
-        if (! *list)
+        if (!*list)
             continue;
         /* Found it! */
         formp->h = f;
         return ST_SUCCESS;
     }
-    st_fail_errno(formp, ST_EFMT, "File type '%s' is not known",
+    st_fail_errno(formp, ST_EFMT, "File type `%s' is not known",
                   formp->filetype);
     return ST_EFMT;
 }
@@ -163,56 +150,35 @@ int st_gettype(ft_t formp)
  * Check that we have a known effect name.  If found, copy name of
  * effect into structure and place a pointer to internal data.
  * Returns -1 on error else it turns the total number of arguments
- * that should be passed to this effects getopt() function.
+ * that should be passed to this effect's getopt() function.
  */
 int st_geteffect_opt(eff_t effp, int argc, char **argv)
 {
     int i, optind;
 
-    for(i = 0; st_effect_fns[i]; i++)
+    for (i = 0; st_effect_fns[i]; i++)
     {
-        char *s1, *s2;
         const st_effect_t *e = st_effect_fns[i]();
 
-        if (!e || !e->name)
-            continue;
-
-        s1 = e->name;
-        s2 = argv[0];
-
-        while(*s1 && *s2 && (tolower(*s1) == tolower(*s2)))
-            s1++, s2++;
-        if (*s1 || *s2)
-            continue;       /* not a match */
-
-        /* Found it! */
-        effp->name = e->name;
-        effp->h = e;
-
-        optind = 1;
-
-        while (optind < argc)
-        {
-            for (i = 0; st_effect_fns[i]; i++)
-            {
-                const st_effect_t *e = st_effect_fns[i]();
-                char *s1 = e->name, *s2 = argv[optind];
-                while (*s1 && *s2 && (tolower(*s1) == tolower(*s2)))
-                    s1++, s2++;
-                if (*s1 || *s2)
-                    continue;
-
-                /* Found it! */
-                return (optind - 1);
-            }
-            /* Didn't find a match, try the next argument. */
-            optind++;
+        if (e && e->name && strcasecmp(e->name, argv[0]) == 0) {
+          effp->name = e->name;
+          effp->h = e;
+          for (optind = 1; optind < argc; optind++)
+          {
+              for (i = 0; st_effect_fns[i]; i++)
+              {
+                  const st_effect_t *e = st_effect_fns[i]();
+                  if (e && e->name && strcasecmp(e->name, argv[optind]) == 0)
+                    return (optind - 1);
+              }
+              /* Didn't find a match, try the next argument. */
+          }
+          /*
+           * No matches found, all the following arguments are
+           * for this effect passed in.
+           */
+          return (optind - 1);
         }
-        /*
-         * No matches found, all the following arguments are
-         * for this effect passed in.
-         */
-        return (optind - 1);
     }
 
     return (ST_EOF);
@@ -224,61 +190,38 @@ int st_geteffect_opt(eff_t effp, int argc, char **argv)
  * Returns -1 on on failure.
  */
 
-int st_geteffect(eff_t effp, char *effect_name)
+int st_geteffect(eff_t effp, const char *effect_name)
 {
     int i;
 
     for(i = 0; st_effect_fns[i]; i++) {
-        char *s1, *s2;
         const st_effect_t *e = st_effect_fns[i]();
 
-        if (!e || !e->name)
-            continue;
-
-        s1 = e->name;
-        s2 = effect_name;
-
-        while(*s1 && *s2 && (tolower(*s1) == tolower(*s2)))
-            s1++, s2++;
-        if (*s1 || *s2)
-            continue;       /* not a match */
-
-        /* Found it! */
-        effp->name = e->name;
-        effp->h = e;
-
-        return ST_SUCCESS;
+        if (e && e->name && strcasecmp(e->name, effect_name) == 0) {
+          effp->name = e->name;
+          effp->h = e;
+          return ST_SUCCESS;
+        }
     }
 
     return (ST_EOF);
 }
 
 /*
- * Check that we have a known effect name.  Return ST_SUCESS if found, else
- * return ST_EOF.
+ * Check if we have a known effect name.
  */
-int st_checkeffect(char *effect_name)
+st_bool is_effect_name(char const * text)
 {
     int i;
 
     for(i = 0; st_effect_fns[i]; i++) {
-        char *s1, *s2;
         const st_effect_t *e = st_effect_fns[i]();
 
-        if (!e || !e->name)
-            continue;
-
-        s1 = e->name;
-        s2 = effect_name;
-        while(*s1 && *s2 && (tolower(*s1) == tolower(*s2)))
-            s1++, s2++;
-        if (*s1 || *s2)
-            continue;       /* not a match */
-
-        return ST_SUCCESS;
+        if (e && e->name && strcasecmp(e->name, text) == 0)
+          return st_true;
     }
 
-    return (ST_EOF);
+    return st_false;
 }
 
 /*
@@ -293,7 +236,7 @@ int st_checkeffect(char *effect_name)
  * calls.
  */
 
-int st_updateeffect(eff_t effp, st_signalinfo_t *in, st_signalinfo_t *out, 
+int st_updateeffect(eff_t effp, const st_signalinfo_t *in, const st_signalinfo_t *out, 
                     int effect_mask)
 {
     effp->ininfo = *in;
@@ -346,23 +289,36 @@ int st_updateeffect(eff_t effp, st_signalinfo_t *in, st_signalinfo_t *out,
  * st_parsesamples
  *
  * Parse a string for # of samples.  If string ends with a 's'
- * then string is interrepted as a user calculated # of samples.
+ * then the string is interpreted as a user calculated # of samples.
  * If string contains ':' or '.' or if it ends with a 't' then its
  * treated as an amount of time.  This is converted into seconds and
  * fraction of seconds and then use the sample rate to calculate
  * # of samples.
- * Returns ST_EOF on error.
+ * Returns NULL on error, pointer to next char to parse otherwise.
  */
-int st_parsesamples(st_rate_t rate, char *str, st_size_t *samples, char def)
+char const * st_parsesamples(st_rate_t rate, const char *str, st_size_t *samples, char def)
 {
     int found_samples = 0, found_time = 0;
-    int time;
+    int time = 0;
     long long_samples;
     float frac = 0;
+    char const * end;
+    char const * pos;
+    st_bool found_colon, found_dot;
 
-    if (strchr(str, ':') || strchr(str, '.') || str[strlen(str)-1] == 't')
+    for (end = str; *end && strchr("0123456789:.ts", *end); ++end);
+    if (end == str)
+      return NULL;
+
+    pos = strchr(str, ':');
+    found_colon = pos && pos < end;
+    
+    pos = strchr(str, '.');
+    found_dot = pos && pos < end;
+
+    if (found_colon || found_dot || *(end-1) == 't')
         found_time = 1;
-    else if (str[strlen(str)-1] == 's')
+    else if (*(end-1) == 's')
         found_samples = 1;
 
     if (found_time || (def == 't' && !found_samples))
@@ -371,8 +327,8 @@ int st_parsesamples(st_rate_t rate, char *str, st_size_t *samples, char def)
 
         while(1)
         {
-            if (sscanf(str, "%d", &time) != 1)
-                return ST_EOF;
+            if (str[0] != '.' && sscanf(str, "%d", &time) != 1)
+                return NULL;
             *samples += time;
 
             while (*str != ':' && *str != '.' && *str != 0)
@@ -389,19 +345,19 @@ int st_parsesamples(st_rate_t rate, char *str, st_size_t *samples, char def)
         if (*str == '.')
         {
             if (sscanf(str, "%f", &frac) != 1)
-                return ST_EOF;
+                return NULL;
         }
 
         *samples *= rate;
-        *samples += (rate * frac);
-        return ST_SUCCESS;
+        *samples += (rate * frac) + 0.5;
+        return end;
     }
     if (found_samples || (def == 's' && !found_time))
     {
         if (sscanf(str, "%ld", &long_samples) != 1)
-            return ST_EOF;
+            return NULL;
         *samples = long_samples;
-        return ST_SUCCESS;
+        return end;
     }
-    return ST_EOF;
+    return NULL;
 }
