@@ -197,12 +197,9 @@ typedef struct vocstuff {
 #define VOC_FMT_MU255          7   /* CCITT u-Law 8-bit PCM */
 #define VOC_FMT_CRLADPCM4A 0x200   /* Creative 16-bit to 4-bit ADPCM */
 
-#define min(a, b)       (((a) < (b)) ? (a) : (b))
-
 /* Prototypes for internal functions */
 static int getblock(ft_t);
 static void blockstart(ft_t);
-static void blockstop(ft_t);
 
 /* Conversion macros (from raw.c) */
 #define ST_ALAW_BYTE_TO_SAMPLE(d) ((st_sample_t)(st_alaw2linear16(d)) << 16)
@@ -212,7 +209,7 @@ static void blockstop(ft_t);
 /*-----------------------------------------------------------------
  * st_vocstartread() -- start reading a VOC file
  *-----------------------------------------------------------------*/
-int st_vocstartread(ft_t ft)
+static int st_vocstartread(ft_t ft)
 {
         int rtn = ST_SUCCESS;
         char header[20];
@@ -222,14 +219,6 @@ int st_vocstartread(ft_t ft)
         int ii;  /* for getting rid of lseek */
         unsigned char uc;
 
-        /* VOC is in Little Endian format.  Swap bytes read in on */
-        /* Big Endian mahcines.                                   */
-        if (ST_IS_BIGENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
-
         if (st_readbuf(ft, header, 1, 20) != 20)
         {
                 st_fail_errno(ft,ST_EHDR,"unexpected EOF in VOC header");
@@ -269,47 +258,47 @@ int st_vocstartread(ft_t ft)
         }
 
         /* setup word length of data */
-        ft->info.size = v->size;
+        ft->signal.size = v->size;
 
         /* ANN:  Check VOC format and map to the proper ST format value */
         switch (v->format) {
         case VOC_FMT_LIN8U:      /*     0    8 bit unsigned linear PCM */
-            ft->info.encoding = ST_ENCODING_UNSIGNED;
+            ft->signal.encoding = ST_ENCODING_UNSIGNED;
             break;
         case VOC_FMT_CRLADPCM4:  /*     1    Creative 8-bit to 4-bit ADPCM */
-            st_warn ("Unsupported VOC format CRLADPCM4 %d", v->format);
+            st_fail ("Unsupported VOC format CRLADPCM4 %d", v->format);
             rtn=ST_EOF;
             break;
         case VOC_FMT_CRLADPCM3:  /*     2    Creative 8-bit to 3-bit ADPCM */
-            st_warn ("Unsupported VOC format CRLADPCM3 %d", v->format);
+            st_fail ("Unsupported VOC format CRLADPCM3 %d", v->format);
             rtn=ST_EOF;
             break;
         case VOC_FMT_CRLADPCM2:  /*     3    Creative 8-bit to 2-bit ADPCM */
-            st_warn ("Unsupported VOC format CRLADPCM2 %d", v->format);
+            st_fail ("Unsupported VOC format CRLADPCM2 %d", v->format);
             rtn=ST_EOF;
             break;
         case VOC_FMT_LIN16:      /*     4    16-bit signed PCM */
-            ft->info.encoding = ST_ENCODING_SIGN2;
+            ft->signal.encoding = ST_ENCODING_SIGN2;
             break;
         case VOC_FMT_ALAW:       /*     6    CCITT a-Law 8-bit PCM */
-            ft->info.encoding = ST_ENCODING_ALAW;
+            ft->signal.encoding = ST_ENCODING_ALAW;
             break;
         case VOC_FMT_MU255:      /*     7    CCITT u-Law 8-bit PCM */
-            ft->info.encoding = ST_ENCODING_ULAW;
+            ft->signal.encoding = ST_ENCODING_ULAW;
             break;
         case VOC_FMT_CRLADPCM4A: /*0x200    Creative 16-bit to 4-bit ADPCM */
-            printf ("Unsupported VOC format CRLADPCM4A %d", v->format);
+            st_fail ("Unsupported VOC format CRLADPCM4A %d", v->format);
             rtn=ST_EOF;
             break;
         default:
-            printf ("Unknown VOC format %d", v->format);
+            st_fail ("Unknown VOC format %d", v->format);
             rtn=ST_EOF;
             break;
         }
 
         /* setup number of channels */
-        if (ft->info.channels == -1)
-                ft->info.channels = v->channels;
+        if (ft->signal.channels == 0)
+                ft->signal.channels = v->channels;
 
         return(ST_SUCCESS);
 }
@@ -319,10 +308,10 @@ int st_vocstartread(ft_t ft)
  * ANN:  Major changes here to support multi-part files and files
  *       that do not have audio in block 9's.
  *-----------------------------------------------------------------*/
-st_ssize_t st_vocread(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_vocread(ft_t ft, st_sample_t *buf, st_size_t len)
 {
         vs_t v = (vs_t) ft->priv;
-        int done = 0;
+        st_size_t done = 0;
         int rc = 0;
         int16_t sw;
         unsigned char  uc;
@@ -343,7 +332,7 @@ st_ssize_t st_vocread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         if (v->silent) {
                 /* Fill in silence */
                 for(;v->rest && (done < len); v->rest--, done++)
-                        *buf++ = 0x80000000L;
+                        *buf++ = 0x80000000;
         }
         /* else, not silence, read the block */
         else {
@@ -384,10 +373,10 @@ st_ssize_t st_vocread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                     } else if (v->format == VOC_FMT_ALAW) {
                         *buf++ =  ST_ALAW_BYTE_TO_SAMPLE(uc);
                     } else {
-                        *buf++ = ST_UNSIGNED_BYTE_TO_SAMPLE(uc);
+                        *buf++ = ST_UNSIGNED_BYTE_TO_SAMPLE(uc,);
                     }
                     break;
-                case ST_SIZE_WORD:
+                case ST_SIZE_16BIT:
                     st_readw(ft, (unsigned short *)&sw);
                     if (st_eof(ft))
                         {
@@ -395,7 +384,7 @@ st_ssize_t st_vocread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                             v->rest = 0;
                             return done;
                         }
-                    *buf++ = ST_SIGNED_WORD_TO_SAMPLE(sw);
+                    *buf++ = ST_SIGNED_WORD_TO_SAMPLE(sw,);
                     v->rest--; /* Processed 2 bytes so update */
                     break;
                 }
@@ -407,15 +396,6 @@ st_ssize_t st_vocread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return done;
 }
 
-/*-----------------------------------------------------------------
- * st_vocstartread() -- start reading a VOC file
- * nothing to do
- *-----------------------------------------------------------------*/
-int st_vocstopread(ft_t ft)
-{
-    return(ST_SUCCESS);
-}
-
 /* When saving samples in VOC format the following outline is followed:
  * If an 8-bit mono sample then use a VOC_DATA header.
  * If an 8-bit stereo sample then use a VOC_EXTENDED header followed
@@ -429,17 +409,10 @@ int st_vocstopread(ft_t ft)
  * which will work with the oldest software (eg. an 8-bit mono sample
  * will be able to be played with a really old SB VOC player.)
  */
-int st_vocstartwrite(ft_t ft)
+static int st_vocstartwrite(ft_t ft)
 {
         vs_t v = (vs_t) ft->priv;
 
-        /* VOC is in Little Endian format.  Swap whats read */
-        /* in on Big Endian machines.                       */
-        if (ST_IS_BIGENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         if (! ft->seekable)
         {
                 st_fail_errno(ft,ST_EOF,
@@ -455,25 +428,25 @@ int st_vocstartwrite(ft_t ft)
         st_writew(ft, 0x10a);              /* major/minor version number */
         st_writew(ft, 0x1129);          /* checksum of version number */
 
-        if (ft->info.size == ST_SIZE_BYTE)
-          ft->info.encoding = ST_ENCODING_UNSIGNED;
+        if (ft->signal.size == ST_SIZE_BYTE)
+          ft->signal.encoding = ST_ENCODING_UNSIGNED;
         else
-          ft->info.encoding = ST_ENCODING_SIGN2;
-        if (ft->info.channels == -1)
-                ft->info.channels = 1;
+          ft->signal.encoding = ST_ENCODING_SIGN2;
+        if (ft->signal.channels == 0)
+                ft->signal.channels = 1;
 
         return(ST_SUCCESS);
 }
 
 /*-----------------------------------------------------------------
- * st_vocstartread() -- start reading a VOC file
+ * st_vocwrite() -- write a VOC file
  *-----------------------------------------------------------------*/
-st_ssize_t st_vocwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
+static st_size_t st_vocwrite(ft_t ft, const st_sample_t *buf, st_size_t len)
 {
         vs_t v = (vs_t) ft->priv;
         unsigned char uc;
         int16_t sw;
-        st_ssize_t done = 0;
+        st_size_t done = 0;
 
         if (v->samples == 0) {
           /* No silence packing yet. */
@@ -482,11 +455,11 @@ st_ssize_t st_vocwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
         }
         v->samples += len;
         while(done < len) {
-          if (ft->info.size == ST_SIZE_BYTE) {
-            uc = ST_SAMPLE_TO_UNSIGNED_BYTE(*buf++);
+          if (ft->signal.size == ST_SIZE_BYTE) {
+            uc = ST_SAMPLE_TO_UNSIGNED_BYTE(*buf++, ft->clips);
             st_writeb(ft, uc);
           } else {
-            sw = (int) ST_SAMPLE_TO_SIGNED_WORD(*buf++);
+            sw = (int) ST_SAMPLE_TO_SIGNED_WORD(*buf++, ft->clips);
             st_writew(ft,sw);
           }
           done++;
@@ -494,10 +467,40 @@ st_ssize_t st_vocwrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
         return done;
 }
 
+/*-----------------------------------------------------------------
+ * blockstop() -- stop an output block
+ * End the current data or silence block.
+ *-----------------------------------------------------------------*/
+static void blockstop(ft_t ft)
+{
+        vs_t v = (vs_t) ft->priv;
+        st_sample_t datum;
+
+        st_writeb(ft, 0);                     /* End of file block code */
+        st_seeki(ft, v->blockseek, 0);         /* seek back to block length */
+        st_seeki(ft, 1, 1);                    /* seek forward one */
+        if (v->silent) {
+                st_writew(ft, v->samples);
+        } else {
+          if (ft->signal.size == ST_SIZE_BYTE) {
+            if (ft->signal.channels > 1) {
+              st_seeki(ft, 8, 1); /* forward 7 + 1 for new block header */
+            }
+          }
+                v->samples += 2;                /* adjustment: SBDK pp. 3-5 */
+                datum = (v->samples * ft->signal.size) & 0xff;
+                st_writeb(ft, (int)datum);       /* low byte of length */
+                datum = ((v->samples * ft->signal.size) >> 8) & 0xff;
+                st_writeb(ft, (int)datum);  /* middle byte of length */
+                datum = ((v->samples  * ft->signal.size)>> 16) & 0xff;
+                st_writeb(ft, (int)datum); /* high byte of length */
+        }
+}
+
 /*-----------------------------------------------------------------
  * st_vocstopwrite() -- stop writing a VOC file
  *-----------------------------------------------------------------*/
-int st_vocstopwrite(ft_t ft)
+static int st_vocstopwrite(ft_t ft)
 {
         blockstop(ft);
         return(ST_SUCCESS);
@@ -574,7 +577,7 @@ static int getblock(ft_t ft)
                             return(ST_EOF);
                           }
                           v->rate = uc;
-                          ft->info.rate = 1000000.0/(256 - v->rate);
+                          ft->signal.rate = 1000000.0/(256 - v->rate);
                           v->channels = 1;
                         }
                         st_readb(ft, &uc);
@@ -605,12 +608,12 @@ static int getblock(ft_t ft)
                             return(ST_EOF);
                         }
                         v->rate = new_rate_32;
-                        ft->info.rate = new_rate_32;
+                        ft->signal.rate = new_rate_32;
                         st_readb(ft, &uc);
                         switch (uc)
                         {
                             case 8:     v->size = ST_SIZE_BYTE; break;
-                            case 16:    v->size = ST_SIZE_WORD; break;
+                            case 16:    v->size = ST_SIZE_16BIT; break;
                             default:
                                 st_fail_errno(ft,ST_EFMT,
                                               "Don't understand size %d", uc);
@@ -657,27 +660,37 @@ static int getblock(ft_t ft)
                         st_readb(ft, &uc);
                         /* Falling! Falling! */
                 case VOC_TEXT:
-                        {
-                            uint32_t i;
-                            /* Could add to comment in SF? */
-                            for(i = 0; i < sblen; i++) {
-                                st_readb(ft, (unsigned char *)&trash);
-                                /* uncomment lines below to display text */
-                                /* Note, if this is uncommented, studio */
-                                /* will not be able to read the VOC file */
-                                /* ANN:  added verbose dump of text info */
-                                /* */
-                                if (verbose) {
-                                    if ((trash != '\0') && (trash != '\r'))
-                                        putc (trash, stderr);
-                                }
-                                /* */
+                        { 
+                          /* TODO: Could add to comment in SF? */
+
+                          /* Note, if this is sent to stderr, studio */
+                          /* will not be able to read the VOC file */
+
+                          uint32_t i = sblen;
+                          char c/*, line_buf[80];
+                          int len = 0*/;
+
+                          while (i--)
+                          {
+                            st_readb(ft, (unsigned char *)&c);
+                            /* FIXME: this needs to be tested but I couldn't
+                             * find a voc file with a VOC_TEXT chunk :(
+                            if (c != '\0' && c != '\r')
+                              line_buf[len++] = c;
+                            if (len && (c == '\0' || c == '\r' ||
+                                i == 0 || len == sizeof(line_buf) - 1))
+                            {
+                              st_report("%s", line_buf);
+                              line_buf[len] = '\0';
+                              len = 0;
                             }
+                            */
+                          }
                         }
                         continue;       /* get next block */
                 case VOC_LOOP:
                 case VOC_LOOPEND:
-                        st_report("File %s: skipping repeat loop");
+                        st_debug("File %s: skipping repeat loop");
                         for(i = 0; i < sblen; i++)
                             st_readb(ft, (unsigned char *)&trash);
                         break;
@@ -712,17 +725,17 @@ static int getblock(ft_t ft)
                         }
                         st_readb(ft, &uc);
                         if (uc)
-                                ft->info.channels = 2;  /* Stereo */
+                                ft->signal.channels = 2;  /* Stereo */
                         /* Needed number of channels before finishing
                            compute for rate */
-                        ft->info.rate = (256000000L/(65536L - v->rate))/
-                            ft->info.channels;
+                        ft->signal.rate = (256000000/(65536 - v->rate))/
+                            ft->signal.channels;
                         /* An extended block must be followed by a data */
                         /* block to be valid so loop back to top so it  */
                         /* can be grabed.                               */
                         continue;
                 default:
-                        st_report("File %s: skipping unknown block code %d",
+                        st_debug("File %s: skipping unknown block code %d",
                                 ft->filename, block);
                         for(i = 0; i < sblen; i++)
                             st_readb(ft, (unsigned char *)&trash);
@@ -745,19 +758,19 @@ static void blockstart(ft_t ft)
                 st_writeb(ft, 0);               /* Period length */
                 st_writeb(ft, v->rate);         /* Rate code */
         } else {
-          if (ft->info.size == ST_SIZE_BYTE) {
+          if (ft->signal.size == ST_SIZE_BYTE) {
             /* 8-bit sample section.  By always setting the correct     */
             /* rate value in the DATA block (even when its preceeded    */
             /* by an EXTENDED block) old software can still play stereo */
             /* files in mono by just skipping over the EXTENDED block.  */
             /* Prehaps the rate should be doubled though to make up for */
             /* double amount of samples for a given time????            */
-            if (ft->info.channels > 1) {
+            if (ft->signal.channels > 1) {
               st_writeb(ft, VOC_EXTENDED);      /* Voice Extended block code */
               st_writeb(ft, 4);                /* block length = 4 */
               st_writeb(ft, 0);                /* block length = 4 */
               st_writeb(ft, 0);                /* block length = 4 */
-                  v->rate = 65536L - (256000000.0/(2*(float)ft->info.rate));
+                  v->rate = 65536 - (256000000.0/(2*(float)ft->signal.rate));
               st_writew(ft,v->rate);    /* Rate code */
               st_writeb(ft, 0);         /* File is not packed */
               st_writeb(ft, 1);         /* samples are in stereo */
@@ -766,7 +779,7 @@ static void blockstart(ft_t ft)
             st_writeb(ft, 0);           /* block length (for now) */
             st_writeb(ft, 0);           /* block length (for now) */
             st_writeb(ft, 0);           /* block length (for now) */
-            v->rate = 256 - (1000000.0/(float)ft->info.rate);
+            v->rate = 256 - (1000000.0/(float)ft->signal.rate);
             st_writeb(ft, (int) v->rate);/* Rate code */
             st_writeb(ft, 0);           /* 8-bit raw data */
         } else {
@@ -774,10 +787,10 @@ static void blockstart(ft_t ft)
             st_writeb(ft, 0);           /* block length (for now) */
             st_writeb(ft, 0);           /* block length (for now) */
             st_writeb(ft, 0);           /* block length (for now) */
-            v->rate = ft->info.rate;
+            v->rate = ft->signal.rate;
             st_writedw(ft, v->rate);    /* Rate code */
             st_writeb(ft, 16);          /* Sample Size */
-            st_writeb(ft, ft->info.channels);   /* Sample Size */
+            st_writeb(ft, ft->signal.channels);   /* Sample Size */
             st_writew(ft, 0x0004);      /* Encoding */
             st_writeb(ft, 0);           /* Unused */
             st_writeb(ft, 0);           /* Unused */
@@ -787,38 +800,8 @@ static void blockstart(ft_t ft)
         }
 }
 
-/*-----------------------------------------------------------------
- * blockstop() -- stop an output block
- * End the current data or silence block.
- *-----------------------------------------------------------------*/
-static void blockstop(ft_t ft)
-{
-        vs_t v = (vs_t) ft->priv;
-        st_sample_t datum;
-
-        st_writeb(ft, 0);                     /* End of file block code */
-        st_seeki(ft, v->blockseek, 0);         /* seek back to block length */
-        st_seeki(ft, 1, 1);                    /* seek forward one */
-        if (v->silent) {
-                st_writew(ft, v->samples);
-        } else {
-          if (ft->info.size == ST_SIZE_BYTE) {
-            if (ft->info.channels > 1) {
-              st_seeki(ft, 8, 1); /* forward 7 + 1 for new block header */
-            }
-          }
-                v->samples += 2;                /* adjustment: SBDK pp. 3-5 */
-                datum = (v->samples * ft->info.size) & 0xff;
-                st_writeb(ft, (int)datum);       /* low byte of length */
-                datum = ((v->samples * ft->info.size) >> 8) & 0xff;
-                st_writeb(ft, (int)datum);  /* middle byte of length */
-                datum = ((v->samples  * ft->info.size)>> 16) & 0xff;
-                st_writeb(ft, (int)datum); /* high byte of length */
-        }
-}
-
 /* Sound Blaster .VOC */
-static char *vocnames[] = {
+static const char *vocnames[] = {
   "voc",
   NULL
 };
@@ -826,10 +809,10 @@ static char *vocnames[] = {
 static st_format_t st_voc_format = {
   vocnames,
   NULL,
-  ST_FILE_STEREO,
+  ST_FILE_LIT_END,
   st_vocstartread,
   st_vocread,
-  st_vocstopread,
+  st_format_nothing,
   st_vocstartwrite,
   st_vocwrite,
   st_vocstopwrite,
