@@ -5,15 +5,15 @@
  * Portions from oggenc, (c) Michael Smith <msmith@labyrinth.net.au>,
  * ogg123, (c) Kenneth Arnold <kcarnold@yahoo.com>, and
  * libvorbisfile (c) Xiphophorus Company
- * 
+ *
  * May 9, 2001 - Stan Seibert (indigo@aztec.asu.edu)
  * Ogg Vorbis driver initially written.
  *
  * July 5, 1991 - Skeleton file
  * Copyright 1991 Lance Norskog And Sundry Contributors
  * This source code is freely redistributable and may be used for
- * any purpose.  This copyright notice must be maintained. 
- * Lance Norskog And Sundry Contributors are not responsible for 
+ * any purpose.  This copyright notice must be maintained.
+ * Lance Norskog And Sundry Contributors are not responsible for
  * the consequences of using this software.
  *
  * TODO: When reading in comments, it doesn't understand how to read
@@ -21,10 +21,11 @@
  */
 #include "st_i.h"
 
-#if defined(HAVE_LIBVORBIS)
+#if defined HAVE_LIBVORBISENC && defined HAVE_LIBVORBISFILE
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
+#include <errno.h>
 
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
@@ -45,7 +46,7 @@ typedef struct vorbis_enc {
         ogg_stream_state os;
         ogg_page         og;
         ogg_packet       op;
-        
+
         vorbis_dsp_state vd;
         vorbis_block     vb;
         vorbis_info      vi;
@@ -65,16 +66,17 @@ typedef struct vorbisstuff {
 } *vorbis_t;
 
 /******** Callback functions used in ov_open_callbacks ************/
-int myclose (void *datasource)
+static int myclose (void *datasource UNUSED)
 {
         /* Do nothing so sox can close the file for us */
         return 0;
 }
 
-/* Taken from vorbisfile.c in libvorbis source code */
-static int _fseek64_wrap(FILE *f,ogg_int64_t off,int whence){
-  if(f==NULL)return(-1);
-  return fseek(f,(int)off,whence);
+static int _fseeko64_wrap(FILE *f, ogg_int64_t off, int whence) {
+  int ret = fseeko(f, (long)off, whence);
+  if (ret == EBADF)
+    ret = -1;
+  return ret;
 }
 
 /********************* End callbacks *****************************/
@@ -82,12 +84,12 @@ static int _fseek64_wrap(FILE *f,ogg_int64_t off,int whence){
 
 /*
  * Do anything required before you start reading samples.
- * Read file header. 
- *      Find out sampling rate, 
- *      size and encoding of samples, 
+ * Read file header.
+ *      Find out sampling rate,
+ *      size and encoding of samples,
  *      mono/stereo/quad.
  */
-int st_vorbisstartread(ft_t ft) 
+static int st_vorbisstartread(ft_t ft)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
         vorbis_info *vi;
@@ -97,20 +99,13 @@ int st_vorbisstartread(ft_t ft)
 
         ov_callbacks callbacks = {
                 (size_t (*)(void *, size_t, size_t, void *))  fread,
-                (int (*)(void *, ogg_int64_t, int))           _fseek64_wrap,
+                (int (*)(void *, ogg_int64_t, int))           _fseeko64_wrap,
                 (int (*)(void *))                             myclose,
-                (long (*)(void *))                            ftell
+                (long (*)(void *))                            ftello
         };
 
-        
-        
         /* Allocate space for decoding structure */
-        vb->vf = (OggVorbis_File *)malloc(sizeof(OggVorbis_File));
-        if (vb->vf == NULL) 
-        {
-            st_fail_errno(ft, ST_ENOMEM, "Could not allocate memory");
-            return (ST_EOF);
-        }
+        vb->vf = (OggVorbis_File *)xmalloc(sizeof(OggVorbis_File));
 
         /* Init the decoder */
         if (ov_open_callbacks((void *)ft->fp,vb->vf,NULL,0,callbacks) < 0)
@@ -125,39 +120,30 @@ int st_vorbisstartread(ft_t ft)
         vc = ov_comment(vb->vf, -1);
 
         /* Record audio info */
-        ft->info.rate = vi->rate;
-        ft->info.size = ST_SIZE_16BIT;
-        ft->info.encoding = ST_ENCODING_VORBIS;
-        ft->info.channels = vi->channels;
+        ft->signal.rate = vi->rate;
+        ft->signal.size = ST_SIZE_16BIT;
+        ft->signal.encoding = ST_ENCODING_VORBIS;
+        ft->signal.channels = vi->channels;
 
         /* ov_pcm_total doesn't work on non-seekable files so
          * skip that step in that case.  Also, it reports
          * "frame"-ish results so we must * channels.
          */
         if (ft->seekable)
-            ft->length = ov_pcm_total(vb->vf, -1) * ft->info.channels;
-        
+            ft->length = ov_pcm_total(vb->vf, -1) * ft->signal.channels;
+
         /* Record comments */
         if (vc->comments == 0)
                 ft->comment = NULL;
-        else 
+        else
         {
                 comment_size = 0;
 
                 for (i = 0; i < vc->comments; i++)
                         comment_size += vc->comment_lengths[i] + 1;
 
-                if ((ft->comment = (char *)calloc(comment_size, sizeof(char))) 
-                     == NULL)
-                {
-                        ov_clear(vb->vf);
-                        free(vb->vf);
+                ft->comment = (char *)xcalloc(comment_size, sizeof(char));
 
-                        st_fail_errno(ft, ST_ENOMEM, 
-                                      "Could not allocate memory");
-                        return (ST_EOF);
-                }
-                
                 offset = 0;
                 for (i = 0; i < vc->comments; i++)
                 {
@@ -175,13 +161,7 @@ int st_vorbisstartread(ft_t ft)
 
         /* Setup buffer */
         vb->buf_len = DEF_BUF_LEN;
-        if ((vb->buf = (char *)calloc(vb->buf_len, sizeof(char))) == NULL )
-        {
-                ov_clear(vb->vf);
-                free(vb->vf);
-                st_fail_errno(ft, ST_ENOMEM, "Could not allocate memory");
-                return (ST_EOF);
-        }
+        vb->buf = (char *)xcalloc(vb->buf_len, sizeof(char));
         vb->start = vb->end = 0;
 
         /* Fill in other info */
@@ -195,7 +175,7 @@ int st_vorbisstartread(ft_t ft)
 /* Refill the buffer with samples.  Returns BUF_EOF if the end of the
    vorbis data was reached while the buffer was being filled,
    BUF_ERROR is something bad happens, and BUF_DATA otherwise */
-int refill_buffer (vorbis_t vb)
+static int refill_buffer (vorbis_t vb)
 {
         int num_read;
 
@@ -205,18 +185,18 @@ int refill_buffer (vorbis_t vb)
         while (vb->end < vb->buf_len)
         {
                 num_read = ov_read(vb->vf, vb->buf + vb->end,
-                                   vb->buf_len - vb->end, 0, 2, 1, 
+                                   vb->buf_len - vb->end, 0, 2, 1,
                                    &vb->current_section);
-                
+
                 if (num_read == 0)
                         return (BUF_EOF);
                 else if (num_read == OV_HOLE)
-                        st_warn("Warning: hole in stream; probably harmless\n");
+                        st_warn("Warning: hole in stream; probably harmless");
                 else if (num_read < 0)
                         return (BUF_ERROR);
                 else
                         vb->end += num_read;
-                        
+
         }
 
         return (BUF_DATA);
@@ -230,10 +210,10 @@ int refill_buffer (vorbis_t vb)
  * Return number of samples read.
  */
 
-st_ssize_t st_vorbisread(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_vorbisread(ft_t ft, st_sample_t *buf, st_size_t len)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
-        int i;
+        st_size_t i;
         int ret;
         st_sample_t l;
 
@@ -252,7 +232,7 @@ st_ssize_t st_vorbisread(ft_t ft, st_sample_t *buf, st_ssize_t len)
                         }
                 }
 
-                l = (vb->buf[vb->start+1]<<24) 
+                l = (vb->buf[vb->start+1]<<24)
                         | (0xffffff &  (vb->buf[vb->start]<<16));
                 *(buf + i) = l;
                 vb->start += 2;
@@ -262,10 +242,10 @@ st_ssize_t st_vorbisread(ft_t ft, st_sample_t *buf, st_ssize_t len)
 }
 
 /*
- * Do anything required when you stop reading samples.  
- * Don't close input file! 
+ * Do anything required when you stop reading samples.
+ * Don't close input file!
  */
-int st_vorbisstopread(ft_t ft) 
+static int st_vorbisstopread(ft_t ft)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
 
@@ -275,9 +255,9 @@ int st_vorbisstopread(ft_t ft)
         return (ST_SUCCESS);
 }
 
-/* Write a page of ogg data to a file.  Taken directly from encode.c in 
+/* Write a page of ogg data to a file.  Taken directly from encode.c in
    oggenc.   Returns the number of bytes written. */
-int oe_write_page(ogg_page *page, ft_t ft)
+static int oe_write_page(ogg_page *page, ft_t ft)
 {
         int written;
         written = st_writebuf(ft, page->header,1,page->header_len);
@@ -289,7 +269,7 @@ int oe_write_page(ogg_page *page, ft_t ft)
 /* Write out the header packets.  Derived mostly from encode.c in
    oggenc.  Returns HEADER_ERROR if the header cannot be written and
    HEADER_OK otherwise. */
-int write_vorbis_header(ft_t ft, vorbis_enc_t *ve)
+static int write_vorbis_header(ft_t ft, vorbis_enc_t *ve)
 {
         ogg_packet header_main;
         ogg_packet header_comments;
@@ -300,19 +280,22 @@ int write_vorbis_header(ft_t ft, vorbis_enc_t *ve)
         char *comment;
 
         /* Make the comment structure */
-        vc.user_comments = (char **)calloc(1, sizeof(char *));
-        vc.comment_lengths = (int *)calloc(1, sizeof(int));
+        vc.user_comments = (char **)xcalloc(1, sizeof(char *));
+        vc.comment_lengths = (int *)xcalloc(1, sizeof(int));
         vc.comments = 1;
 
         /* We check if there is a FIELD=value pair already in the comment
          * if not, add one */
-        if (strchr(ft->comment,'=') == NULL) 
+        if (strchr(ft->comment,'=') == NULL)
         {
-            comment = (char *)calloc(1,strlen(ft->comment)+strlen("COMMENT=")+1);
+            comment = (char *)xcalloc(1,strlen(ft->comment)+strlen("COMMENT=")+1);
             strncpy(comment,"COMMENT=",strlen("COMMENT="));
         }
         else
-            comment = (char *)calloc(1,strlen(ft->comment)+1);
+            comment = (char *)xcalloc(1,strlen(ft->comment)+1);
+        
+        if (!comment)
+            return HEADER_ERROR;
 
         strcat(comment,ft->comment);
 
@@ -324,17 +307,18 @@ int write_vorbis_header(ft_t ft, vorbis_enc_t *ve)
                                   &header_main,
                                   &header_comments,
                                   &header_codebooks);
-        
+
         /* And stream them out */
         ogg_stream_packetin(&ve->os,&header_main);
         ogg_stream_packetin(&ve->os,&header_comments);
         ogg_stream_packetin(&ve->os,&header_codebooks);
-        
+
         while((result = ogg_stream_flush(&ve->os, &ve->og)))
         {
-                if(!result) break;
+                if (!result)
+                  break;
                 ret = oe_write_page(&ve->og, ft);
-                if(!ret)
+                if (!ret)
                 {
                     free(comment);
                     return HEADER_ERROR;
@@ -345,70 +329,72 @@ int write_vorbis_header(ft_t ft, vorbis_enc_t *ve)
         return HEADER_OK;
 }
 
-int st_vorbisstartwrite(ft_t ft) 
+static int st_vorbisstartwrite(ft_t ft)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
         vorbis_enc_t *ve;
         long rate;
+        double quality = 3; /* Default compression quality gives ~112kbps */
 
-        ft->info.size = ST_SIZE_16BIT;
-        ft->info.encoding = ST_ENCODING_VORBIS;
+        ft->signal.size = ST_SIZE_16BIT;
+        ft->signal.encoding = ST_ENCODING_VORBIS;
 
         /* Allocate memory for all of the structures */
-        ve = vb->vorbis_enc_data = (vorbis_enc_t *)malloc(sizeof(vorbis_enc_t));
-        if (ve == NULL)
-        {
-            st_fail_errno(ft, ST_ENOMEM, "Could not allocate memory");
-            return (ST_EOF);
-        }
+        ve = vb->vorbis_enc_data = (vorbis_enc_t *)xmalloc(sizeof(vorbis_enc_t));
 
         vorbis_info_init(&ve->vi);
 
-        /* DEBUG */
-        rate = ft->info.rate;
-
+        /* TODO */
+        rate = ft->signal.rate;
         if (rate)
-        {
             st_fail_errno(ft, ST_EHDR, "Error setting up Ogg Vorbis encorder - make sure you've specied a sane rate and number of channels");
-        }
 
-        /* Set encoding to average bit rate of 112kbps VBR */
-        vorbis_encode_init_vbr(&ve->vi, ft->info.channels, ft->info.rate, 0.3f);
+        /* Use encoding to average bit rate of VBR as specified by the -C option */
+        if (ft->signal.compression != HUGE_VAL)
+        {
+            if (ft->signal.compression < -1 || ft->signal.compression > 10)
+            {
+                st_fail_errno(ft,ST_EINVAL,
+                              "Vorbis compression quality nust be between -1 and 10");
+                return ST_EOF;
+            }
+            quality = ft->signal.compression;
+        }
+        vorbis_encode_init_vbr(&ve->vi, ft->signal.channels, ft->signal.rate, quality / 10);
 
         vorbis_analysis_init(&ve->vd, &ve->vi);
         vorbis_block_init(&ve->vd, &ve->vb);
-       
-        st_initrand();
+
         ogg_stream_init(&ve->os, rand()); /* Random serial number */
-        
+
         if (write_vorbis_header(ft, ve) == HEADER_ERROR)
         {
             st_fail_errno(ft,ST_EHDR,
                           "Error writing header for Ogg Vorbis audio stream");
             return (ST_EOF);
         }
-        
-        return(ST_SUCCESS);     
+
+        return(ST_SUCCESS);
 }
 
-st_ssize_t st_vorbiswrite(ft_t ft, st_sample_t *buf, st_ssize_t len) 
+static st_size_t st_vorbiswrite(ft_t ft, const st_sample_t *buf, st_size_t len)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
         vorbis_enc_t *ve = vb->vorbis_enc_data;
-        st_ssize_t samples = len / ft->info.channels;   
+        st_size_t samples = len / ft->signal.channels;
         float **buffer = vorbis_analysis_buffer(&ve->vd, samples);
-        st_ssize_t i, j;
+        st_size_t i, j;
         int ret;
         int eos = 0;
 
         /* Copy samples into vorbis buffer */
         for (i = 0; i < samples; i++)
-                for (j = 0; j < ft->info.channels; j++)
-                        buffer[j][i] = buf[i*ft->info.channels + j] 
+                for (j = 0; j < ft->signal.channels; j++)
+                        buffer[j][i] = buf[i*ft->signal.channels + j]
                                 / ((float)ST_SAMPLE_MAX);
 
         vorbis_analysis_wrote(&ve->vd, samples);
-        
+
         while(vorbis_analysis_blockout(&ve->vd,&ve->vb)==1)
         {
                 /* Do the main analysis, creating a packet */
@@ -416,19 +402,19 @@ st_ssize_t st_vorbiswrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
                 vorbis_bitrate_addblock(&ve->vb);
 
                 /* Add packet to bitstream */
-                while (vorbis_bitrate_flushpacket(&ve->vd, &ve->op)) 
+                while (vorbis_bitrate_flushpacket(&ve->vd, &ve->op))
                 {
                     ogg_stream_packetin(&ve->os,&ve->op);
-                
+
                     /* If we've gone over a page boundary, we can do actual
-                     * output, so do so (for however many pages are available) 
+                     * output, so do so (for however many pages are available)
                      */
-                
+
                     while(!eos)
                     {
                         int result = ogg_stream_pageout(&ve->os,&ve->og);
                         if(!result) break;
-                        
+
                         ret = oe_write_page(&ve->og, ft);
                         if(!ret)
                             return (ST_EOF);
@@ -439,10 +425,10 @@ st_ssize_t st_vorbiswrite(ft_t ft, st_sample_t *buf, st_ssize_t len)
                 }
         }
 
-        return (len);    
+        return (len);
 }
 
-int st_vorbisstopwrite(ft_t ft) 
+static int st_vorbisstopwrite(ft_t ft)
 {
         vorbis_t vb = (vorbis_t) ft->priv;
         vorbis_enc_t *ve = vb->vorbis_enc_data;
@@ -458,7 +444,7 @@ int st_vorbisstopwrite(ft_t ft)
         return (ST_SUCCESS);
 }
 
-static char *vorbisnames[] = {
+static const char *vorbisnames[] = {
   "vorbis",
   "ogg",
   NULL
@@ -467,7 +453,7 @@ static char *vorbisnames[] = {
 static st_format_t st_vorbis_format = {
   vorbisnames,
   NULL,
-  ST_FILE_STEREO,
+  0,
   st_vorbisstartread,
   st_vorbisread,
   st_vorbisstopread,
@@ -481,4 +467,4 @@ const st_format_t *st_vorbis_format_fn(void)
 {
     return &st_vorbis_format;
 }
-#endif /* HAVE_LIBVORBIS */
+#endif
