@@ -4,16 +4,12 @@
  */
 
 #include "st_i.h"
-#include "g72x.h"
 #include <string.h>
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#endif
 
 /* Magic numbers used in Psion audio files */
 #define PSION_MAGIC     "ALawSoundFile**"
 #define PSION_VERSION   ((short)3856)
-#define PSION_INV_VERSION   ((short)4111)
 #define PSION_HDRSIZE   32
 
 typedef struct wvepriv
@@ -27,17 +23,17 @@ typedef struct wvepriv
 
 static void wvewriteheader(ft_t ft);
 
-int st_wveseek(ft_t ft, st_size_t offset)
+static int st_wveseek(ft_t ft, st_size_t offset)
 {
     int new_offset, channel_block, alignment;
-    wve_t wve = (wve_t ) ft->priv;
+    wve_t wve = (wve_t)ft->priv;
 
-    new_offset = offset * ft->info.size;
-    /* Make sure request aligns to a channel block (ie left+right) */
-    channel_block = ft->info.channels * ft->info.size;
+    new_offset = offset * ft->signal.size;
+    /* Make sure request aligns to a channel block (i.e. left+right) */
+    channel_block = ft->signal.channels * ft->signal.size;
     alignment = new_offset % channel_block;
-    /* Most common mistaken is to compute something like
-     * "skip everthing upto and including this sample" so
+    /* Most common mistake is to compute something like
+     * "skip everthing up to and including this sample" so
      * advance to next sample block in this case.
      */
     if (alignment != 0)
@@ -47,9 +43,9 @@ int st_wveseek(ft_t ft, st_size_t offset)
     return st_seeki(ft, offset, SEEK_SET);
 }
 
-int st_wvestartread(ft_t ft)
+static int st_wvestartread(ft_t ft)
 {
-        wve_t p = (wve_t ) ft->priv;
+        wve_t p = (wve_t)ft->priv;
         char magic[16];
         short version;
         int rc;
@@ -61,18 +57,10 @@ int st_wvestartread(ft_t ft)
         if (rc)
             return rc;
 
-        /* WVE is in big endian format.  Swap whats read in
-         * on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         /* Check the magic word (null-terminated) */
         st_reads(ft, magic, 16);
         if (strncmp(magic, PSION_MAGIC, 15)==0) {
-                st_report("Found Psion magic word");
+                st_debug("Found Psion magic word");
         }
         else
         {
@@ -82,20 +70,9 @@ int st_wvestartread(ft_t ft)
 
         st_readw(ft, (unsigned short *)&version);
 
-        /* Check for what type endian machine its read on */
-        if (version == PSION_INV_VERSION)
-        {
-                /* This is probably left over from a time before
-                 * testing for endianess was standardized.  Leaving since
-                 * it doesn't hurt.
-                 */
-                ft->swap = ft->swap ? 0 : 1;
-                st_report("Found inverted PSION magic word.  Swapping bytes.");
-        }
-        else if (version == PSION_VERSION)
-        {
-            st_report("Found PSION magic word");
-        }
+        /* Check magic version */
+        if (version == PSION_VERSION)
+            st_debug("Found Psion magic word");
         else
         {
             st_fail_errno(ft,ST_EHDR,"Wrong version in Psion header");
@@ -108,23 +85,23 @@ int st_wvestartread(ft_t ft)
 
         st_readw(ft, (unsigned short *)&(p->repeats));
 
-        (void)st_readw(ft, (unsigned short *)&trash16);
-        (void)st_readw(ft, (unsigned short *)&trash16);
-        (void)st_readw(ft, (unsigned short *)&trash16);
+        st_readw(ft, (unsigned short *)&trash16);
+        st_readw(ft, (unsigned short *)&trash16);
+        st_readw(ft, (unsigned short *)&trash16);
 
-        ft->info.encoding = ST_ENCODING_ALAW;
-        ft->info.size = ST_SIZE_BYTE;
+        ft->signal.encoding = ST_ENCODING_ALAW;
+        ft->signal.size = ST_SIZE_BYTE;
 
-        if (ft->info.rate != 0)
+        if (ft->signal.rate != 0)
             st_report("WVE must use 8000 sample rate.  Overriding");
-        ft->info.rate = 8000;
+        ft->signal.rate = 8000;
 
-        if (ft->info.channels != -1 && ft->info.channels != 1)
+        if (ft->signal.channels != ST_ENCODING_UNKNOWN && ft->signal.channels != 1)
             st_report("WVE must only supports 1 channel.  Overriding");
-        ft->info.channels = 1;
+        ft->signal.channels = 1;
 
         p->dataStart = st_tell(ft);
-        ft->length = p->length/ft->info.size;
+        ft->length = p->length/ft->signal.size;
 
         return (ST_SUCCESS);
 }
@@ -138,9 +115,9 @@ int st_wvestartread(ft_t ft)
    if it is not, the unspecified size remains in the header
    (this is illegal). */
 
-int st_wvestartwrite(ft_t ft)
+static int st_wvestartwrite(ft_t ft)
 {
-        wve_t p = (wve_t ) ft->priv;
+        wve_t p = (wve_t)ft->priv;
         int rc;
 
         /* Needed for rawwrite() */
@@ -148,49 +125,36 @@ int st_wvestartwrite(ft_t ft)
         if (rc)
             return ST_EOF;
 
-        /* wve is in big endian format.  Swap whats read in
-         * on little endian machines.
-         */
-        if (ST_IS_LITTLEENDIAN)
-        {
-                ft->swap = ft->swap ? 0 : 1;
-        }
-
         p->length = 0;
         if (p->repeats == 0)
             p->repeats = 1;
 
-        if (ft->info.rate != 0)
+        if (ft->signal.rate != 0)
             st_report("WVE must use 8000 sample rate.  Overriding");
 
-        if (ft->info.channels != -1 && ft->info.channels != 1)
+        if (ft->signal.channels != 0 && ft->signal.channels != 1)
             st_report("WVE must only supports 1 channel.  Overriding");
 
-        ft->info.encoding = ST_ENCODING_ALAW;
-        ft->info.size = ST_SIZE_BYTE;
-        ft->info.rate = 8000;
+        ft->signal.encoding = ST_ENCODING_ALAW;
+        ft->signal.size = ST_SIZE_BYTE;
+        ft->signal.rate = 8000;
 
         wvewriteheader(ft);
         return ST_SUCCESS;
 }
 
-st_ssize_t st_wveread(ft_t ft, st_sample_t *buf, st_ssize_t samp)
+static st_size_t st_wvewrite(ft_t ft, const st_sample_t *buf, st_size_t samp)
 {
-        return st_rawread(ft, buf, samp);
-}
-
-st_ssize_t st_wvewrite(ft_t ft, st_sample_t *buf, st_ssize_t samp)
-{
-        wve_t p = (wve_t ) ft->priv;
-        p->length += samp * ft->info.size;
+        wve_t p = (wve_t)ft->priv;
+        p->length += samp * ft->signal.size;
         return st_rawwrite(ft, buf, samp);
 }
 
-int st_wvestopwrite(ft_t ft)
+static int st_wvestopwrite(ft_t ft)
 {
 
         /* Call before seeking to flush buffer */
-        return st_rawstopwrite(ft);
+        st_rawstopwrite(ft);
 
         if (!ft->seekable)
         {
@@ -198,7 +162,7 @@ int st_wvestopwrite(ft_t ft)
             return ST_SUCCESS;
         }
 
-        if (st_seeki(ft, 0L, 0) != 0)
+        if (st_seeki(ft, 0, 0) != 0)
         {
                 st_fail_errno(ft,errno,"Can't rewind output file to rewrite Psion header.");
                 return(ST_EOF);
@@ -212,7 +176,7 @@ static void wvewriteheader(ft_t ft)
     char magic[16];
     short version;
     short zero;
-    wve_t p = (wve_t ) ft->priv;
+    wve_t p = (wve_t)ft->priv;
 
     strcpy(magic,PSION_MAGIC);
     version=PSION_VERSION;
@@ -223,6 +187,7 @@ static void wvewriteheader(ft_t ft)
     st_writeb(ft, 0);
 
     st_writew(ft, version);
+
     st_writedw(ft, p->length);
     st_writew(ft, p->padding);
     st_writew(ft, p->repeats);
@@ -233,7 +198,7 @@ static void wvewriteheader(ft_t ft)
 }
 
 /* Psion .wve */
-static char *wvenames[] = {
+static const char *wvenames[] = {
   "wve",
   NULL
 };
@@ -241,9 +206,9 @@ static char *wvenames[] = {
 static st_format_t st_wve_format = {
   wvenames,
   NULL,
-  ST_FILE_SEEK,
+  ST_FILE_SEEK | ST_FILE_BIG_END,
   st_wvestartread,
-  st_wveread,
+  st_rawread,
   st_rawstopread,
   st_wvestartwrite,
   st_wvewrite,
